/*
	AQTRONIX XML Configuration Editor
	Copyright 2002 Parcifal Aertssen

	This file is part of AQTRONIX XML Configuration Editor.

    AQTRONIX XML Configuration Editor is free software; you can redistribute
	it and/or modify it under the terms of the GNU General Public License as
	published by the Free Software Foundation; either version 2 of the
	License, or (at your option) any later version.

    AQTRONIX XML Configuration Editor is distributed in the hope that it will
	be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AQTRONIX XML Configuration Editor; if not, write to the 
	Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
// ConfigDoc.cpp : implementation of the CConfigDoc class
//

#include "stdafx.h"
#include "Config.h"

#include "ConfigDoc.h"

#include "XMLObject.h"
#include "XMLCheckbox.h"
#include "XMLEditbox.h"
#include "XMLList.h"
#include "XMLMap.h"
#include "XMLSeparator.h"
#include "XMLTitle.h"
#include "XMLDocument.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define MAX_XML_FILE 196000


/////////////////////////////////////////////////////////////////////////////
// CConfigDoc

IMPLEMENT_DYNCREATE(CConfigDoc, CDocument)

BEGIN_MESSAGE_MAP(CConfigDoc, CDocument)
	//{{AFX_MSG_MAP(CConfigDoc)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CConfigDoc construction/destruction

CConfigDoc::CConfigDoc()
{
	pCurrentDocument = NULL;
}

CConfigDoc::~CConfigDoc()
{
	DeleteContents();
}

BOOL CConfigDoc::OnNewDocument()
{
	DeleteContents();

	if (!CDocument::OnNewDocument())
		return FALSE;

	return TRUE;
}



/////////////////////////////////////////////////////////////////////////////
// CConfigDoc serialization

void CConfigDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// TODO: add loading code here
	}
}

/////////////////////////////////////////////////////////////////////////////
// CConfigDoc diagnostics

#ifdef _DEBUG
void CConfigDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CConfigDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CConfigDoc commands

BOOL CConfigDoc::OnOpenDocument(LPCTSTR lpszPathName) 
{
	//if (!CDocument::OnOpenDocument(lpszPathName))
	//	return FALSE;

	//open file
	CFile file;
	if(file.Open(lpszPathName,CFile::modeRead|CFile::shareDenyNone)){
		//read xml file
		char buf[MAX_XML_FILE+1] = "";				//buffer for xml file
		int bread = file.Read(buf,MAX_XML_FILE);	//read whole file
		buf[MAX_XML_FILE] = '\0';					//null terminate
		CString tmp = buf;

		//valid xml file?
		if(tmp.Left(6).CompareNoCase("<?xml ")!=0){
			AfxMessageBox("Could not open the file " + CString(lpszPathName) + " because it is not an XML file!");	
			return FALSE;
		}

		//delete current contents of file in memory
		DeleteContents();

		//copy contents of xml file to document
		xml = buf;

		//mark document as clean
		
		//close file
		file.Close();
		
		//parse XML
		XMLParser();

		return TRUE;
	}else{
		AfxMessageBox("Could not open the file " + CString(lpszPathName) + ".");
		return FALSE;
	}
}

BOOL CConfigDoc::OnSaveDocument(LPCTSTR lpszPathName) 
{
	//open file
	CFile file;
	if(file.Open(lpszPathName,CFile::modeCreate|CFile::modeWrite)){
		//make xml string from objects
		ObjectParser();

		//write xml file
		file.Write((LPCTSTR)xml,xml.GetLength());

		//mark document as clean
	
		//close file
		file.Close();
	}else{
		AfxMessageBox("Error: File could not be saved!");
		return FALSE;
	}

	return TRUE;
	//return CDocument::OnSaveDocument(lpszPathName);
}

void CConfigDoc::DeleteContents() 
{
	xml = "";
	id = 1000;
	POSITION pos;
	pos = ptrlist.GetHeadPosition();
	while(pos!=NULL){
		CXMLObject* p = (CXMLObject*)ptrlist.GetNext(pos);
		delete p;
	}
	ptrlist.RemoveAll();

	CDocument::DeleteContents();
}

void CConfigDoc::XMLParser()
{
	/*
	 * From xml to objects 
	 */
	int length = xml.GetLength();
	CString key; bool bkey(false);
	CString val; bool bval(false);
	bool blarge(false);
	CString tmp;
	CString type;
	int pos;
	CString list;
	bool nostop(false);
	for(int i=0;i<length;i++){
		switch(xml[i]){

		case '<':	//start of a key/end of value
			if(nostop){
				int lenkey = key.Find(' ');
				if(lenkey!=-1 && i+2+lenkey<length){
					CString keyname(key.Left(lenkey));
					if(keyname == xml.Mid(i+2,lenkey)){
						nostop = false;
						blarge = true;
					}else{
						//if closing tag is already read, because of empty val
						int posclosetag = tmp.Find("</");
						if(posclosetag!=-1 && posclosetag+1+lenkey<tmp.GetLength()){
							if(tmp.Mid(posclosetag+2,lenkey)==keyname){
								nostop=false;
								blarge=true;
								tmp = tmp.Mid(0,posclosetag);
							}else{
								tmp += xml[i];
							}
						}else{
							tmp += xml[i];
						}
					}
				}
			}
			if(!nostop){
				val = tmp;
				tmp = "";
				bkey = true;
			}
			if(!nostop && bkey && bval){
				if(key.Left(1)!='?'){	//ignore this (<?xml version...)
					if(key.Left(1)!='/'){	//is it end of entity?
						pos = key.Find("App=");
						if(pos!=-1){
							int posend = key.Find(' ',pos);
							if(posend!=-1){
								type = key.Mid(pos+4,posend);
							}else{
								type = key.Mid(pos+4);
							}
						}else{
							type="";
						}
						type.MakeLower();
						if(type.Find("option")!=-1){
							//draw checkbox
							AddCheckBox(KeyParameter(key,"name"),val=="1"?true:false,KeyParameter(key,"Default=")=="1"?true:false,KeyParameter(key,"Explanation="));
							bval = bkey = false;
						}else if(type.Find("text")!=-1){
							//draw textbox
							AddEditBox(KeyParameter(key,"name"),val,KeyParameter(key,"Default="),KeyParameter(key,"Explanation="));
							bval = bkey = false;
						}else if(type.Find("separator")!=-1){
							//draw separator (line,...)
							AddSeparator(KeyParameter(key,"name"));
							bval = bkey = false;
						}else if(type.Find("title")!=-1){
							//draw title
							AddTitle(KeyParameter(key,"name"));
							bval = bkey = false;
						}else if(type.Find("document")!=-1){
							//draw title
							AddDocument(KeyParameter(key,"name"));
							bval = bkey = false;
						}else if(type.Find("list")!=-1){
							if(blarge){
								AddList(KeyParameter(key,"name"),val,KeyParameter(key,"Explanation="));
								blarge = bval = bkey = false;
							}else{
								tmp += xml[i];
								nostop = true;
							}
						}else if(type.Find("map")!=-1){
							if(blarge){
								AddMap(KeyParameter(key,"name"),val,KeyParameter(key,"Explanation="));
								blarge = bval = bkey = false;
							}else{
								tmp += xml[i];
								nostop = true;
							}
						}else{
							//Unknown XML object
							AddUnknown(key);
							bval = bkey = false;
						}
					}
				}
			}
			break;

		case '>':	//start of a value
			if(nostop){ //don't stop, it's a list or map
				tmp += xml[i];
			}else{
				key = tmp;
				tmp = "";
				bval = true;
			}
			break;

		default:
			tmp += xml[i];
			break;
		}
	}
}

void CConfigDoc::ObjectParser()
{
	/*
	 * From objects to xml
	 */
	//add version
	xml = "<?xml version=\"1.0\" standalone='yes'?>\r\n";
	//add contents
	POSITION pos;
	pos = ptrlist.GetHeadPosition();
	while(pos!=NULL){
		CXMLObject* p = (CXMLObject*)ptrlist.GetNext(pos);
		xml += p->XML();
		xml += "\r\n";
	}
	//add closing xml entity
	CXMLObject* p = (CXMLObject*)ptrlist.GetHead();
	CString tmp;
	tmp = p->XML();
	xml += "</";
	xml += KeyParameter(tmp,"name");
	xml += ">\r\n";
}

void CConfigDoc::AddCheckBox(CString& label, bool checked, bool defaultvalue, CString& explanation)
{
	id++;
	CXMLCheckBox* xmlchk = new CXMLCheckBox;
	xmlchk->SetValues(label,id,checked,defaultvalue,explanation);
	ptrlist.AddTail(xmlchk);
}

void CConfigDoc::AddEditBox(CString& label, CString& value, CString& defaultvalue, CString& explanation)
{
	id++;
	CXMLEditBox* xmledit = new CXMLEditBox;
	xmledit->SetValues(label,id,value,defaultvalue,explanation);
	ptrlist.AddTail(xmledit);
}

void CConfigDoc::AddList(CString& label, CString& values, CString& explanation)
{
	id++;
	CStringList list;
	CString tmp;
	for(int i=0;i<values.GetLength();i++){
		switch(values[i]){
		case '<':
			if(i+1<values.GetLength()){
				if(values[i+1]=='/'){
					if(i!=0){
						list.AddTail(tmp);
					}
				}
			}
			tmp = "";
			break;

		case '>':	//start of a value
			tmp = "";
			break;

		default:
			tmp += values[i];
			break;
		}
	}
	CXMLList* xmllist = new CXMLList;
	xmllist->SetValues(label,id,list,explanation);
	ptrlist.AddTail(xmllist);
}

void CConfigDoc::AddMap(CString& label, CString& values, CString& explanation)
{
	id++;
	CMapStringToString map; bool bkey(false); CString key; CString tmp;
	for(int i=0;i<values.GetLength();i++){
		switch(values[i]){
		case '<':
			if(i+1<values.GetLength()){
				if(values[i+1]=='/'){
					if(i!=0){
						if (!bkey){
							bkey = true;
							key = tmp;
						}else{
							bkey = false;
							map.SetAt(key,tmp);
						}
					}
				}
			}
			tmp = "";
			break;

		case '>':	//start of a value
			tmp = "";
			break;

		default:
			tmp += values[i];
			break;
		}
	}
	
	CXMLMap* xmlmap = new CXMLMap;
	xmlmap->SetValues(label,id,map,explanation);
	ptrlist.AddTail(xmlmap);
}

void CConfigDoc::AddSeparator(CString& label)
{
	id++;
	CXMLSeparator* xmlsep = new CXMLSeparator;
	if(pCurrentDocument==NULL){
		xmlsep->SetValues(label,id);
	}else{
		xmlsep->SetValues(label,id,pCurrentDocument);
	}
	ptrlist.AddTail(xmlsep);
}

void CConfigDoc::AddTitle(CString &label)
{
	id++;
	CXMLTitle* xmltitle = new CXMLTitle;
	xmltitle->SetValues(label,id);
	ptrlist.AddTail(xmltitle);
}

void CConfigDoc::AddUnknown(CString& name_)
{
	id++;
	CXMLObject* xmlobj = new CXMLObject;
	xmlobj->SetValues(name_,id);
	ptrlist.AddTail(xmlobj);
}

CString CConfigDoc::KeyParameter(CString key, CString par)
{
	int pos;
	if(par.CompareNoCase("name")==0){
		CString tmp(key);
		pos = key.Find(' ');
		if(pos!=-1){
			tmp = key.Left(pos);
		}
		tmp.TrimLeft('<');
		tmp.TrimLeft('/');
		tmp.TrimRight('>');
		return tmp;
	}
	pos = key.Find(par);
	if(pos!=-1){
		CString tmp;
		tmp = key.Mid(pos+par.GetLength());
		tmp.TrimLeft('\'');
		tmp.TrimLeft('"');
		
		pos = tmp.FindOneOf("'\"");
		if(pos!=-1){
			tmp = tmp.Left(pos);
		}

		return tmp;
	}
	return "";
}

BOOL CConfigDoc::CanCloseFrame(CFrameWnd* pFrame) 
{
	/*
	 * Check if document is changed, if so ask to make changes.
	 */
	 
	POSITION pos;
	pos = ptrlist.GetHeadPosition();
	while(pos!=NULL){
		CXMLObject* p = (CXMLObject*)ptrlist.GetNext(pos);
		if(p->IsChanged()){
			SetModifiedFlag();
			break;
		}
	}
		
	return CDocument::CanCloseFrame(pFrame);
}

void CConfigDoc::AddDocument(CString &name)
{
	id++;
	CXMLDocument* xmldoc = new CXMLDocument;
	pCurrentDocument = xmldoc;
	xmldoc->SetValues(name,id);
	ptrlist.AddTail(xmldoc);
}
