/*
	AQTRONIX XML Configuration Editor
	Copyright 2003 Parcifal Aertssen

	This file is part of AQTRONIX XML Configuration Editor.

    AQTRONIX XML Configuration Editor is free software; you can redistribute
	it and/or modify it under the terms of the GNU General Public License as
	published by the Free Software Foundation; either version 2 of the
	License, or (at your option) any later version.

    AQTRONIX XML Configuration Editor is distributed in the hope that it will
	be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AQTRONIX XML Configuration Editor; if not, write to the 
	Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/// XMLDocument.cpp: implementation of the CXMLDocument class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Config.h"
#include "XMLDocument.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CXMLDocument::CXMLDocument()
{
	//Create menu control
	DocumentMenu.CreatePopupMenu();

	//set main window menu pointer
	pAppWindow = AfxGetMainWnd();
	if(pAppWindow!=NULL)
		pAppMenu = pAppWindow->GetMenu();

	//initialize ID_Count
	ID_Count = 4500;

}

CXMLDocument::~CXMLDocument()
{
	//detach from appmenu
	if(pAppMenu!=NULL)
		pAppMenu->Detach();

	//delete posmap
	void* pkey; void* pval;
	POSITION pos = PosMap.GetStartPosition();
	while(pos!=NULL){
		PosMap.GetNextAssoc(pos,pkey,pval);
		delete (long*)pkey;
		delete (long*)pval;
	}
	PosMap.RemoveAll();
}

CString CXMLDocument::XML()
{
	return "<" + XMLEncode(name) + " App='Document'/>";
}

CPoint CXMLDocument::Create(CWnd *pParentWindow_, CPoint pos)
{
	//save CWnd pointer for later use
	pViewWindow = pParentWindow_;

	//add item to application menu
	if(pAppMenu!=NULL){
		CString name_disp = name;
		name_disp.Replace('_',' ');
		pAppMenu->InsertMenu(1, MF_ENABLED|MF_BYPOSITION|MF_POPUP, (UINT)DocumentMenu.GetSafeHmenu(), name_disp) ;
		ID_Count++;
		DrawMenus();
	}

	return CPoint(0,0);
}

void CXMLDocument::AddItem(CString name, CPoint pos)
{
	//save position
	name.Replace('_',' ');
	UINT* pkey = new UINT;
	long* pval = new long;
	*pkey = ID_Count;
	*pval = pos.y;
	PosMap.SetAt(pkey,pval);

	//add item to document menu
	DocumentMenu.InsertMenu(-1, MF_BYPOSITION|MF_STRING|MF_ENABLED,0, name);

	//add id count
	ID_Count++;
		
	//redraw menus
	DrawMenus();
}

void CXMLDocument::DrawMenus()
{
	//draw main menu bar
	if(pAppWindow!=NULL)
		pAppWindow->DrawMenuBar();
}

long CXMLDocument::LookupPos(UINT nID)
{
	void* pkey; void* pval;
	POSITION pos = PosMap.GetStartPosition();
	while(pos!=NULL){
		PosMap.GetNextAssoc(pos,pkey,pval);
		if(*(UINT*)pkey==nID){
			return *(UINT*)pkey;
		}
	}
	return -1;
}
