/*
	AQTRONIX XML Configuration Editor
	Copyright 2002 Parcifal Aertssen

	This file is part of AQTRONIX XML Configuration Editor.

    AQTRONIX XML Configuration Editor is free software; you can redistribute
	it and/or modify it under the terms of the GNU General Public License as
	published by the Free Software Foundation; either version 2 of the
	License, or (at your option) any later version.

    AQTRONIX XML Configuration Editor is distributed in the hope that it will
	be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AQTRONIX XML Configuration Editor; if not, write to the 
	Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
// XMLMap.cpp: implementation of the CXMLMap class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Config.h"
#include "XMLObject.h"
#include "XMLMap.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CXMLMap::CXMLMap()
{
}

CXMLMap::~CXMLMap()
{
}

CString CXMLMap::XML()
{
	GetValues(values);
	CString tmp('<');
	tmp += XMLEncode(name);
	tmp += " App='Map' Explanation='";
	tmp += XMLEncode(explanation);
	tmp += "'>\r\n";

	POSITION pos; CString key; CString val;
	pos = values.GetStartPosition();
	while(pos!=NULL){
		values.GetNextAssoc(pos,key,val);
		tmp += "<Key>";
		tmp += XMLEncode(key);
		tmp += "</Key><Val>";
		tmp += XMLEncode(val);
		tmp += "</Val>\r\n";
	}

	tmp += "</";
	tmp += XMLEncode(name);
	tmp += ">";
	return tmp;
}

bool CXMLMap::IsChanged()
{
	CMapStringToString map;
	GetValues(map);

	POSITION pos; POSITION posv;
	CString mkey; CString mval; CString vkey; CString vval;
	pos = map.GetStartPosition();
	posv = values.GetStartPosition();
	while(pos!=NULL && posv!=NULL){
		map.GetNextAssoc(pos,mkey,mval);
		values.GetNextAssoc(posv,vkey,vval);
		if(mkey!=vkey || mval!=vval){
			return true;
			break;
		}
	}
	if(pos!=NULL)
		return true;
	if(posv!=NULL)
		return true;

	return false;
}

CPoint CXMLMap::Create(CWnd *pParentWindow, CPoint pos)
{
	//set font
	LOGFONT logfont;
	logfont.lfHeight = 14;
	//logfont.lfWidth = 8;
	logfont.lfEscapement = 0;
	logfont.lfOrientation = 0;
	logfont.lfWeight = FW_NORMAL;
	logfont.lfItalic = FALSE;
	logfont.lfUnderline = FALSE;
	logfont.lfStrikeOut = FALSE;
	logfont.lfCharSet = ANSI_CHARSET;
	logfont.lfOutPrecision = OUT_DEFAULT_PRECIS;
	logfont.lfClipPrecision = CLIP_DEFAULT_PRECIS;
	logfont.lfQuality = DEFAULT_QUALITY;
	logfont.lfPitchAndFamily = DEFAULT_PITCH;
	strcpy(logfont.lfFaceName,"Verdana");
	
	CString tmp;
	tmp = NameDecode(name);
	tmp += ':';

	//label
	label.Create(tmp,WS_CHILD|WS_VISIBLE|SS_NOPREFIX,CRect(CPoint(pos.x+200,pos.y),CPoint(pos.x+400,pos.y+20)),pParentWindow);
	font_label.CreateFontIndirect(&logfont);
	label.SetFont(&font_label);

	//value
	listctrl.Create(WS_CHILD|WS_VISIBLE|WS_TABSTOP|WS_BORDER|LVS_REPORT|LVS_EDITLABELS,CRect(CPoint(pos.x+220,pos.y+20),CPoint(pos.x+810,pos.y+180)),pParentWindow,id);
	font_values.CreateFontIndirect(&logfont);
	listctrl.SetFont(&font_values);
	listctrl.SetExtendedStyle(LVS_EX_GRIDLINES);
	listctrl.InsertColumn(0,"Key",LVCFMT_LEFT,285,0);
	listctrl.InsertColumn(1,"Value",LVCFMT_LEFT,285,1);
	POSITION posl; int i=0; CString key; CString val;
	posl = values.GetStartPosition();
	while (posl!=NULL){
		values.GetNextAssoc(posl,key,val);
		listctrl.InsertItem(i,key);
		i++;
	}

	posl = values.GetStartPosition();
	i=0;
	while (posl!=NULL){
		values.GetNextAssoc(posl,key,val);
		listctrl.SetItemText(i,1,val);
		i++;
	}
	//to avoid a 'bug' (=feature) with CMap: position != sequence of values, so reload!
	GetValues(values);

	//explanation
	lblexp.Create(explanation,WS_CHILD|WS_VISIBLE|SS_NOPREFIX,CRect(CPoint(pos.x+220,pos.y+180),CPoint(pos.x+810,pos.y+220)),pParentWindow);
	logfont.lfHeight = 12;
	font_explanation.CreateFontIndirect(&logfont);
	lblexp.SetFont(&font_explanation);

	return CPoint(0,230);
}

void CXMLMap::SetValues(CString& name_, WORD id_, CMapStringToString& values_, CString& explanation_)
{
	name = XMLDecode(name_);
	explanation = XMLDecode(explanation_);
	id = id_;
	POSITION pos; CString key; CString val;
	pos = values_.GetStartPosition();
	values.RemoveAll();
	while(pos!=NULL){
		values_.GetNextAssoc(pos,key,val);
		values.SetAt(XMLDecode(key),XMLDecode(val));
	}
}

bool CXMLMap::GetValues(CMapStringToString &map)
{
	map.RemoveAll();
	TCHAR key[4097];
	TCHAR val[4097];
	int count = listctrl.GetItemCount();
	for(int i=0; i<count; i++){
		listctrl.GetItemText(i,0,key,4096);
		listctrl.GetItemText(i,1,val,4096);
		key[4096]='\0';	//make sure it is null terminated
		val[4096]='\0';	//make sure it is null terminated
		map.SetAt(key,val);
	}
	return true;
}
