/*
    AQTRONIX C++ Security Library
    Copyright 2002 Parcifal Aertssen

    This file is part of AQTRONIX C++ Security Library.

    AQTRONIX C++ Security Library is free software; you can redistribute
    it and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.

    AQTRONIX C++ Security Library is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AQTRONIX C++ Security Library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
*/
// FileName.cpp: implementation of the CFileName class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "FileName.h"
#include  <io.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

/* TO DO
 * - Unicode ready
 */

/* CHANGELOG
 *
 *	2003.10.16 Added SetFileName() and Exists functions
 *	2003.05.18 Added overloaded functions DeleteFile(fn) and CreatePath()
 *	2003.05.04 Added DeleteFile()
 *	2003.03.24 Initial version finished
 */

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CFileName::CFileName(CString fn)
{
	SetFileName(fn);
}

CFileName::~CFileName()
{

}

CFileName& CFileName::operator =(const CString& fn)
{
	//set full path
	FullPath = fn;
	FullPathLCase = FullPath;
	FullPathLCase.MakeLower();

	//split filename from full name (from full path)
	int i; int j; int p;
	i = fn.ReverseFind('/');
	j = fn.ReverseFind('\\');
	i>j?p=i:p=j;
	if(p!=-1){
		Path = fn.Left(p+1);
		FileName = fn.Right(fn.GetLength()-p-1);
	}else{
		Path = fn;
		FileName = fn;
	}
	
	//split extension from filename
	p = FileName.ReverseFind('.');
	if (p!=-1)
		Extension = FileName.Right(FileName.GetLength()-p);

	//set lowercase path
	PathLCase = Path;
	PathLCase.MakeLower();

	//set lowercase filename
	FileNameLCase = FileName;
	FileNameLCase.MakeLower();

	//set lowercase extension
	ExtensionLCase = Extension;
	ExtensionLCase.MakeLower();

	return *this;
}


void CFileName::CreatePath(CString wsPath)
{
	DWORD attr; 
	int pos;
	
	// Check for trailing slash
	if (wsPath.Right(1)=='\\')
		wsPath = wsPath.Left(wsPath.GetLength()-1); //remove trailing '\'
		
	//Look for existing object
	attr = GetFileAttributes(LPCTSTR(wsPath));
	if (0xFFFFFFFF == attr){ //doesn't exist yet
		pos = wsPath.ReverseFind('\\');
		if (pos!=-1) {
			//Create parent dirs
			CreatePath(wsPath.Left(pos));
		} 
		//Create node
		CreateDirectory(LPCTSTR(wsPath), NULL); /* TO DO: set only system & admin
														  access rigths on dir */
	}
}

bool CFileName::DeleteFile()
{
	return DeleteFile(FullPath);
}

void CFileName::CreatePath()
{
	CreatePath(Path);
}

bool CFileName::DeleteFile(const CString &fn)
{
	try{
		CFile::Remove(fn);
		return true;
	}catch(CFileException* e){
		e->Delete();
		return false;
	}
}

bool CFileName::Exists(const CString &fn)
{
	try{
		if(_taccess(fn,0)!=-1){
			return true;
		}else{
			return false;
		}
	}catch(...){
		return false;
	}
}

bool CFileName::Exists()
{
	return Exists(FullPath);
}

void CFileName::SetFileName(CString fn)
{
	//set full path
	FullPath = fn;
	FullPathLCase = FullPath;
	FullPathLCase.MakeLower();

	//split filename from full name (from full path)
	int i; int j; int p;
	i = fn.ReverseFind('/');
	j = fn.ReverseFind('\\');
	i>j?p=i:p=j;
	if(p!=-1){
		Path = fn.Left(p+1);
		FileName = fn.Right(fn.GetLength()-p-1);
	}else{
		Path = fn;
		FileName = fn;
	}
	
	//split extension from filename
	p = FileName.ReverseFind('.');
	if (p!=-1)
		Extension = FileName.Right(FileName.GetLength()-p);

	//set lowercase path
	PathLCase = Path;
	PathLCase.MakeLower();

	//set lowercase filename
	FileNameLCase = FileName;
	FileNameLCase.MakeLower();

	//set lowercase extension
	ExtensionLCase = Extension;
	ExtensionLCase.MakeLower();

}
