/*
    AQTRONIX C++ Security Library
    Copyright 2002 Parcifal Aertssen

    This file is part of AQTRONIX C++ Security Library.

    AQTRONIX C++ Security Library is free software; you can redistribute
    it and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.

    AQTRONIX C++ Security Library is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AQTRONIX C++ Security Library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
*/
// LogFile.cpp: implementation of the CLogFile class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "LogFile.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

/* TO DO
 * - Unicode ready
 */

/* CHANGELOG
 *
 *	2003.05.26 Removed bug in errorhandling: added e->Delete();
 *	2003.03.24 Initial version finished
 */

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CLogFile::CLogFile(CString f):IsOpen(false)
{
	SetFileName(f);
}

CLogFile::~CLogFile()
{
	Close();
}

bool CLogFile::Open()
{
	if (FileName == "") {
		Error = "No filename specified!";
		return false;
	}

	if (!IsOpen) {
		//open it
		int openret;
		CFileException fe;
		openret = CFile::Open(FileName,CFile::modeCreate|CFile::modeNoTruncate|CFile::modeWrite|CFile::shareDenyWrite, &fe);
		if (openret > 0) {
			IsOpen = true;
			try{
				CFile::SeekToEnd();
				return true;
			}catch(CFileException* e){
				e->Delete();
				Error = "Could not seek to end of the logfile: " + FileName;
				return false;
			}
		}else{
			TCHAR errmsg[255];
			fe.GetErrorMessage(errmsg,255);
			Error = (CString)errmsg;
			return false;
		}
	}else{
		Error = "Logfile is already open!";
		return false;
	}
}

void CLogFile::Close()
{
	//if file is open, close it and set IsOpen to false
	if(IsOpen){
		try{
			CFile::Close();
			IsOpen = false;
		}catch(CFileException* e){
			e->Delete();
			Error = "Could not close the logfile " + FileName;
		}
	}
}

bool CLogFile::Write(CString msg)
{
	//if file is not open, then open it
	if (!IsOpen){
		Open();
	}
	//if it is open, then write to it
	if (IsOpen){
		try {
			CFile::Write(msg,strlen(msg));
			return true;
		}catch(CFileException* e){
			e->Delete();
			Error = "Could not write to the logfile " + FileName;
			return false;
		}
	} else{
		return false;
	}
}

void CLogFile::SetFileName(CString& pf)
{
	//if filename is not the same
	if (pf != FileName) {
		FileName = pf;	//change filename
		if (IsOpen) {	//reopen if logfile is open
			Close();
			Open();
		}
	}
}

CString CLogFile::GetLastError() const
{
	return Error;
}

bool CLogFile::NewLine()
{
	return Write("\r\n");
}

bool CLogFile::WriteLine(CString msg)
{
	msg += "\r\n";
	return Write(msg);
}