/*
    AQTRONIX C++ Security Library
    Copyright 2003 Parcifal Aertssen

    This file is part of AQTRONIX C++ Security Library.

    AQTRONIX C++ Security Library is free software; you can redistribute
    it and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.

    AQTRONIX C++ Security Library is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AQTRONIX C++ Security Library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
*/
// Settings.cpp: implementation of the CSettings class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Settings.h"
#include "Logger.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

/* TO DO
 * - Unicode ready
 */

/* CHANGELOG
 *  2003.09.17 Fixed bug in ReadFromFileXML where it would stop parsing if value
 *             is an empty string in a list or map entity.
 *  2003.08.31 Fixed bug when changing timezone while running: call _tzset() before GetCurrentTime()
 *  2003.06.08 made class
 */

CSettings::CSettings()
{
	LoadDefaults();	//always load defaults (failover redundancy)
}

CSettings::~CSettings()
{

}

void CSettings::CleanupString(CString& str)
{
	//cleanup the string
	int pos = str.Find(';');	//drop everything after ';' (including the ';')
	if(pos>1)					//only if ';' is found and not on first position
		str = str.Left(pos);
	str.TrimLeft();		//remove leading spaces & tabs & newlines
	str.TrimRight();	//remove trailing spaces & tabs & newlines
}

void CSettings::CleanupStringLCase(CString &str)
{
	CleanupString(str);	//clean it up
	str.MakeLower();	//make it lowercase
}

void CSettings::CleanupStringUCase(CString &str)
{
	CleanupString(str);	//clean it up
	str.MakeUpper();	//make it uppercase
}

void CSettings::Split(LPCTSTR p, int length, CStringList& list)
{
	/* Removes everything from the list if length > 0
	 * and adds all entries from the string to the list
	 */
	if(length>0){
		list.RemoveAll();	//remove everything
		list.AddTail(p);	//first entry
		for(int i=0;i+1<length;i++){
			if (p[i]=='\0'){
				if(p[i+1]!='\0')
					list.AddTail(&p[i+1]);	//rest of it
			}
		}
	}
}

CString CSettings::ConvertListToString(const CStringList &list)
{
	CString str("");
	POSITION pos = list.GetHeadPosition();
	while(pos!=NULL){
		str = str + list.GetNext(pos) + "\r\n";
	}
	return str + '\0'; //double terminate it
}

CString CSettings::ConvertMapToString(const CMapStringToString &map)
{
	CString str(""); CString k; CString v;
	POSITION pos = map.GetStartPosition();
	while(pos!=NULL){
		map.GetNextAssoc(pos,k,v);
		str = str + k + '=' + v + "\r\n";
	}
	return str + '\0'; //double terminate it
}

void CSettings::CleanupList(CStringList &list)
{
	POSITION pos;
	pos = list.GetHeadPosition();
	while (pos != NULL){
		CleanupString(list.GetNext(pos));
	}
}

void CSettings::CleanupListLCase(CStringList &list)
{
	POSITION pos;
	pos = list.GetHeadPosition();
	while (pos != NULL){
		CleanupStringLCase(list.GetNext(pos));
	}
}

void CSettings::CleanupListUCase(CStringList &list)
{
	POSITION pos;
	pos = list.GetHeadPosition();
	while (pos != NULL){
		CleanupStringUCase(list.GetNext(pos));
	}
}

CString CSettings::XMLKeyParameter(CString key, CString par)
{
	int pos;
	if(par.CompareNoCase("name")==0){
		CString tmp(key);
		pos = key.Find(' ');
		if(pos!=-1){
			tmp = key.Left(pos);
		}
		tmp.TrimLeft('<');
		tmp.TrimLeft('/');
		tmp.TrimRight('>');
		return tmp;
	}
	pos = key.Find(par);
	if(pos!=-1){
		CString tmp;
		tmp = key.Mid(pos+par.GetLength());
		tmp.TrimLeft('\'');
		tmp.TrimLeft('"');
		
		pos = tmp.FindOneOf("'\"");
		if(pos!=-1){
			tmp = tmp.Left(pos);
		}
		return tmp;
	}
	return "";
}

void CSettings::ConvertXMLToList(CString& val, CStringList& list)
{
	
	CStringList l;
	CString tmp;
	for(int i=0;i<val.GetLength();i++){
		switch(val[i]){
		case '<':
			if(i+1<val.GetLength()){
				if(val[i+1]=='/'){
					if(i!=0){
						l.AddTail(XMLDecode(tmp));
					}
				}
			}
			tmp = "";
			break;

		case '>':	//start of a value
			tmp = "";
			break;

		default:
			tmp += val[i];
			break;
		}
	}

	POSITION pos;
	pos = l.GetHeadPosition();
	if (pos!=NULL){
		list.RemoveAll();
		while(pos!=NULL){
			list.AddTail(l.GetNext(pos));
		}
	}
}

void CSettings::ConvertXMLToMap(CString& val, CMapStringToString& map)
{
	CMapStringToString m; bool bkey(false); CString key; CString tmp;
	for(int i=0;i<val.GetLength();i++){
		switch(val[i]){
		case '<':
			if(i+1<val.GetLength()){
				if(val[i+1]=='/'){
					if(i!=0){
						if (!bkey){
							bkey = true;
							key = tmp;
						}else{
							bkey = false;
							m.SetAt(XMLDecode(key),XMLDecode(tmp));
						}
					}
				}
			}
			tmp = "";
			break;

		case '>':	//start of a value
			tmp = "";
			break;

		default:
			tmp += val[i];
			break;
		}
	}
	POSITION pos;
	CString k;
	CString v;
	pos = m.GetStartPosition();
	if(pos!=NULL){
		map.RemoveAll();
		while(pos!=NULL){
			m.GetNextAssoc(pos,k,v);
			map.SetAt(k,v);
		}
	}
}

void CSettings::RemoveFrom(CStringList &list, LPCTSTR value)
{
	POSITION pos = list.Find(value);
	if(pos!=NULL)
		list.RemoveAt(pos);
}

void CSettings::AddTo(CStringList &list, LPCTSTR value)
{
	if(list.Find(value)==NULL)
		list.AddTail(value);
}

CString CSettings::ConvertStringToXML(LPCTSTR tag, LPCTSTR app, LPCTSTR value, LPCTSTR defaultvalue, LPCTSTR explanation)
{
	return CString("<" + XMLEncode(tag) + " App='" + XMLEncode(app) + "' Default='" + XMLEncode(defaultvalue) + "' Explanation='" + XMLEncode(explanation) + "'>" + XMLEncode(value) + "</" + XMLEncode(tag) + ">\r\n");
}

CString CSettings::ConvertListToXML(LPCTSTR tag, CStringList &List, LPCTSTR explanation)
{
	
	CString tmp("<");
	tmp += XMLEncode(tag);
	tmp += " App='List' Explanation='";
	tmp += XMLEncode(explanation);
	tmp += "'>\r\n";
	
	POSITION pos;
	pos = List.GetHeadPosition();
	while(pos!=NULL){
		tmp += "<Item>";
		tmp += XMLEncode(List.GetNext(pos));
		tmp += "</Item>\r\n";
	}

	tmp += "</";
	tmp += XMLEncode(tag);
	tmp += ">\r\n";
	return tmp;
}

CString CSettings::ConvertMapToXML(LPCTSTR tag, CMapStringToString &map, LPCTSTR explanation)
{
	CString tmp("<");
	tmp += XMLEncode(tag);
	tmp += " App='Map' Explanation='";
	tmp += XMLEncode(explanation);
	tmp += "'>\r\n";

	POSITION pos; CString key; CString val;
	pos = map.GetStartPosition();
	while(pos!=NULL){
		map.GetNextAssoc(pos,key,val);
		tmp += "<Key>";
		tmp += XMLEncode(key);
		tmp += "</Key>";
		tmp += "<Val>";
		tmp += XMLEncode(val);
		tmp += "</Val>\r\n";
	}

	tmp += "</";
	tmp += XMLEncode(tag);
	tmp += ">\r\n";
	return tmp;
}

inline CString CSettings::XMLEncode(LPCTSTR str)
{
	CString enc(str);
	enc.Replace("&","&amp;");
	enc.Replace("<","&lt;");
	enc.Replace(">","&gt;");
	enc.Replace("'","&apos;");
	enc.Replace("\"","&quot;");
	return enc;
}

inline CString CSettings::XMLDecode(LPCTSTR str)
{
	CString enc(str);
	enc.Replace("&lt;","<");
	enc.Replace("&gt;",">");
	enc.Replace("&apos;","'");
	enc.Replace("&quot;","\"");
	enc.Replace("&amp;","&");
	return enc;
}

bool CSettings::Update()
{
	//if enough time passed (every 1 minute)
	_tzset();
	CTime now = CTime::GetCurrentTime();
	if(now-Update_Time>CTimeSpan(60)){
		//set time last update
		Update_Time = now;
		
		//update settings (run-time update)
		return UpdateFromFile();
	}
	return false;
}

bool CSettings::UpdateFromFile()
{
	if (Update_fn != "") {
		CFileStatus fs;
		if(CFile::GetStatus(Update_fn,fs)){			//if fileinformation could be retrieved
			if(Update_fn_Timestamp != fs.m_mtime){	//if timestamp is different
				ReadFromFile(Update_fn);			//reload settings
				CFileName cfn(Update_fn);
				WriteToFile(cfn.Path + "Loaded" + cfn.Extension);//write loaded settings to debug file
				return true;
			}
		}
	}
	return false;
}

void CSettings::LoadDefaults()
{

}

bool CSettings::ReadFromFile(LPCTSTR fn)
{
	try{

		//if file does not exist, return false
		DWORD attr = GetFileAttributes(fn);
		if (0xFFFFFFFF == attr)//doesn't exist
			return false;

		//if file cannot be opened, return false
		CFile f;
		if(!f.Open(fn,CFile::modeRead|CFile::shareDenyWrite))
			return false;
		
		//set info for run-time update
		_tzset();
		Update_Time = CTime::GetCurrentTime();
		Update_fn = fn;
		CFileStatus fs;
		if(f.GetStatus(fs))
			Update_fn_Timestamp = fs.m_mtime;

		//close the file
		f.Close();


		//if file is a urlscan.ini file
		CFileName cfn(fn);
		if(cfn.ExtensionLCase == ".ini"){		//.ini file
			return ReadFromFileINI(fn);
		}else if(cfn.ExtensionLCase == ".xml"){		//.xml file
			return ReadFromFileXML(fn);
		}else{
			return false;							//unknown file type
		}
		
	}catch(...){
		return false;
	}
}

bool CSettings::ReadFromFileINI(LPCTSTR fn){
	return false;
}

bool CSettings::ReadFromFileXML(LPCTSTR fn)
{
	/*
	 * read settings from an XML file
	 */
	try{
		const int sz = MAX_XML_FILE_SIZE;
		char buf[sz+1]="";
		CFile file;
		if(file.Open(fn,CFile::modeRead|CFile::shareDenyWrite)){
			//read file
			int bread = file.Read(buf,sz);
			buf[bread] = '\0';	//null terminate
			file.Close();
			
			//xml string
			CString xml(buf);

			//parse string
			int length = xml.GetLength();
			CString key; bool bkey(false);
			CString val; bool bval(false);
			bool blarge(false);
			CString tmp;
			CString type;
			int pos;
			CString list;
			bool nostop(false);
			for(int i=0;i<length;i++){
				switch(xml[i]){	
				case '<':	//start of a key/end of value
					if(nostop){
						int lenkey = key.Find(' ');
						if(lenkey!=-1 && i+2+lenkey<length){
							CString keyname(key.Left(lenkey));
							if(keyname == xml.Mid(i+2,lenkey)){
								nostop = false;
								blarge = true;
							}else{
								//if closing tag is already read, because of empty val
								int posclosetag = tmp.Find("</");
								if(posclosetag!=-1 && posclosetag+1+lenkey<tmp.GetLength()){
									if(tmp.Mid(posclosetag+2,lenkey)==keyname){
										nostop=false;
										blarge=true;
										tmp = tmp.Mid(0,posclosetag);
									}else{
										tmp += xml[i];
									}
								}else{
									tmp += xml[i];
								}
							}
						}
					}
					if(!nostop){
						val = tmp;
						tmp = "";
						bkey = true;
					}
					if(!nostop && bkey && bval){
						if(key.Left(1)!='?'){	//ignore this (<?xml version...)
							if(key.Left(1)!='/'){	//is it end of entity?
								pos = key.Find("App=");
								if(pos!=-1){
									int posend = key.Find(' ',pos);
									if(posend!=-1){
										type = key.Mid(pos+4,posend);
									}else{
										type = key.Mid(pos+4);
									}
								}else{
									type="";
								}
								type.MakeLower();
								if(type.Find("list")!=-1){
									if(blarge){
										ParseXML(key,val);
										blarge = bval = bkey = false;
									}else{
										tmp += xml[i];
										nostop = true;
									}
								}else if(type.Find("map")!=-1){
									if(blarge){
										ParseXML(key,val);
										blarge = bval = bkey = false;
									}else{
										tmp += xml[i];
										nostop = true;
									}
								}else{
									ParseXML(key,val);
									bval = bkey = false;
								}
							}
						}
					}
					break;

				case '>':	//start of a value
					if(nostop){ //don't stop, it's a list or map
						tmp += xml[i];
					}else{
						key = tmp;
						tmp = "";
						bval = true;
					}
					break;

				default:
					tmp += xml[i];
					break;
				}
			}

			return true;
		}else{
			return false;
		}
	}catch(...){
		LoadDefaults();
		return false;
	}
}

void CSettings::ParseXML(CString key, CString val)
{

}

bool CSettings::ReadFromRegistry()
{
	return false;
}

bool CSettings::ReadFromActiveDirectory()
{
	return false;
}

bool CSettings::WriteToRegistry()
{
	return false;
}

bool CSettings::WriteToActiveDirectory()
{
	return false;
}

bool CSettings::WriteToFile(LPCTSTR fn)
{
	/*
	 * Write settings to a given file (instance policy)
	 */
	try{

		//if file already exists, delete it
		DWORD attr = GetFileAttributes(fn);
		if (0xFFFFFFFF != attr){	//if it exists
			if(!CFileName::DeleteFile(fn))		//delete file
				return false;		//return false if file could not be deleted
		}

		//if path does not exist, make it
		CFileName::CreatePath(CFileName(fn).Path);
		
		//look at extension to see what type of file it has to be
		CFileName cfn(fn);
		if (cfn.ExtensionLCase == ".xml"){		//XML file
			return WriteToFileXML(fn);
		}else if(cfn.ExtensionLCase == ".ini"){	//INI file
			return WriteToFileINI(fn);
		}else{
			//unknown file type
			CFile f; int openret;
			openret = f.Open(fn,CFile::modeCreate|CFile::modeWrite|CFile::shareDenyWrite);
			if (openret > 0) {
				const char buf[] = "AQTRONIX Settings\r\nError: unknown file type!\r\nAQTRONIX Settings encountered an unknown file type (extension) to write its settings to.\r\nTo implement this file type add the required functionality to the function WriteToFile() in the class CSettings.";
				f.Write(buf,strlen(buf));
				f.Close();
				return true;
			}else{
				return false;
			}
		}

	}catch(...){
		return false;
	}
}

bool CSettings::WriteToFileXML(LPCTSTR fn)
{
	return false;
}

bool CSettings::WriteToFileINI(LPCTSTR fn)
{
	return false;
}