/*
	AQTRONIX WebKnight - ISAPI Filter for securing IIS
	Copyright 2002 Parcifal Aertssen

	This file is part of AQTRONIX WebKnight.

    AQTRONIX WebKnight is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as
	published by the Free Software Foundation; either version 2 of the
	License, or (at your option) any later version.

    AQTRONIX WebKnight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AQTRONIX WebKnight; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
// WEBKNIGHT.CPP - Implementation file for your Internet Server
//    WebKnight Filter

#include "stdafx.h"
#include "WebKnight.h"

#ifdef IISBUILD
#define SITEINSTANCE "W3SVC"		//IIS site labelling
#else
#define SITEINSTANCE "Instance "	//default site labelling
#endif

/* AQTRONIX WebKnight
 *
 * See README.TXT for information about the source code
 * For global defines, look at the file Globals.h
 *
 * 13 August    2002: Project started
 * 26 October   2002: Started coding a GUI for settings
 *  4 November  2002: Beta 1.0 released (without GUI)
 * 12 January   2003: First version of GUI finished
 * 25 February  2003: Windows Installer package finished
 * 24 March     2003: Final 1.0 released (+config 1.0, +installer 1.0)
 * 14 July      2003: Final 1.1 released (+config 1.1, +installer 1.1)
 *  8 September 2003: Final 1.2 released (+config 1.1, +installer 1.2)
 *  ? November  2003: Final 1.3 released (+config 1.2, +installer 1.3)
 *
 * You can not define _UNICODE because this program is not made for it and the
 * ISAPI framework does not (yet) support Unicode (I use Visual C++ 6.0).
 * If you still want to enable this you have to review every piece of code and
 * adjust all char types, "str" functions,...
 *
 * For IIS6 (and up) you cannot register for the OnReadRawData event if IIS is
 * running in Worker Process mode (only works in IIS 5.0 Isolation mode).
 */

/* TO DO
 * -----
 * WEBKNIGHT
 *  <1.4>
 * - enable WAP (see wap.txt)
 *
 * - lowercase/uppercase conversion of xml lists/map in settings
 * - screenshots of settings/possibilities of webknight on web site
 * - set permissions on WebKnight dir & logdir (createpath function)?
 * - headers in request (set value/block winhttp.dll headers)
 * - application/x-www-UTF8-encoded & UTF (url,query,header,data) + check invalid UTF sequences
 * - generic variable length (query, post, header, data)
 * - help file (faq.htm)
 *
 *  <1.5>
 *
 *									    	Function / Settings / Testing / Finished
 * - compatible with Outlook Web Access		   OK		  OK	    OK
 * - compatible with Frontpage Extensions	   OK         OK       TO DO
 * - compatible with Coldfusion				   OK        TO DO     TO DO
 * - compatible with Flash MX				   OK        TO DO     TO DO
 * - compatible with WebDAV					   OK         OK        OK        YES
 * - compatible with IISADMPWD				   OK		  OK	   TO DO
 * - compatible with Sharepoint Portal Server  OK	      OK       TO DO
 * - compatible with Sharepoint Team Services  OK		  OK       TO DO
 * - compatible with Certificate Services	   OK         OK       TO DO
 * - compatible with BizTalk				   OK         OK       TO DO
 * - compatible with Commerce Server		   OK         OK       TO DO
 * - compatible with Small Business Server	   OK         OK       TO DO
 * - compatible with ASP.NET				   OK         OK       TO DO
 * - compatible with ASP					   OK         OK        OK        YES
 * - compatible with Static HTML			   OK         OK        OK        YES
 *
 * - see ini files (shield)
 *
 *
 * FEATURES UNDER CONSIDERATION
 * - print loaded settings (only options) to the log file (when a load/reload of settings happens)
 * - compare settings with previously loaded settings when printing settings to log file
 *
 * WANTED FEATURES
 * - Settings in Active Directory (schedules, organizational unit, multiple policies)
 * - ODBC Logging
 * - MMC
 *
 * REJECTED FEATURES
 * - validHostport (see kodeit), this is stupid because of HTTP 0.9 and HTTP 1.0 requests still possible
 * - Settings in Registry: stupid invention anyway!
 * - Logging: W3C extended log format: too limited!
 *
 * POSSIBLE EXPLOITS IN IIS:
 * - %00 exploit?
 * - Ms-Echo-Request header??
 * - VERBS (from winhttp.dll & SEARCH exploit?)
 * - headers (from winhttp.dll)
 * - ASP puts error messages in querystring???
 *
 * BUGS IN WEBKNIGHT
 * - Settings: INI file: writing ';' in SQL Keywords cannot be read by ReadFrom function -> WILL NOT BE FIXED
 *
 * BUGS IN ISAPI FRAMEWORK:
 * 1) if attack detected in OnReadRawData -> no OnSendRawData event (server header not
 *    changed or removed and filtercontext not reset). SOLUTION: added code in HackResponse().
 *    Only works with ResponseDirectly=1! If you redirect to a url or only send back a
 *    ResponseStatus then server header is not changed.
 */

///////////////////////////////////////////////////////////////////////
// The one and only CWebKnightFilter object

CWebKnightFilter theFilter;

///////////////////////////////////////////////////////////////////////
// CWebKnightFilter implementation

CWebKnightFilter::CWebKnightFilter(){

	CString strinit; CString strinit2;

	//get the directory of the .dll file
	char dir[MAX_PATH] = "";
	CFileName dll;
	if(GetModuleFileName(GetModuleHandle("WebKnight.dll"),dir,MAX_PATH))
		dll = dir;

	CSingleLock lock(&crit, TRUE);

	DWORD attr;

#ifdef XML_ENABLED
	//initialize settings
	if(s.ReadFromFile(dll.Path + "WebKnight.xml")){			//WebKnight.xml
		strinit = "INFO: Settings loaded from WebKnight.xml file";
	}else{
		if(s.ReadFromFile(dll.Path + "WebKnight.ini")){		//WebKnight.ini
			strinit = "INFO: Settings loaded from WebKnight.ini file";
		}else{
			if(s.ReadFromFile(dll.Path + "urlscan.ini")){	//urlscan.ini
				strinit = "INFO: Settings loaded from urlscan.ini file (will be converted to a WebKnight.xml file)";
			}else{
				strinit = "WARNING: Failed to load settings from WebKnight.xml, WebKnight.ini or urlscan.ini file (defaults loaded)";
			}
		}
	}

	//make a WebKnight.xml file if it doesn't exist
	attr = GetFileAttributes(dll.Path + "WebKnight.xml");
	if (0xFFFFFFFF == attr){ //doesn't exist yet
		if (s.WriteToFile(dll.Path + "WebKnight.xml")){
			strinit2 = "INFO: A file WebKnight.xml was made with the currently loaded settings";
		}else{
			strinit2 = "WARNING: Failed to make a settings file WebKnight.xml";
		}
	}
#else
	//initialize settings
	if(s.ReadFromFile(dll.Path + "WebKnight.ini")){		//WebKnight.ini
		strinit = "INFO: Settings loaded from WebKnight.ini file";
	}else{
		if(s.ReadFromFile(dll.Path + "urlscan.ini")){	//urlscan.ini
			strinit = "INFO: Settings loaded from urlscan.ini file (will be converted to a WebKnight.ini file)";
		}else{
			strinit = "WARNING: Failed to load settings from WebKnight.ini or urlscan.ini file (defaults loaded)";
		}
	}

	//make a WebKnight.ini file if it doesn't exist
	attr = GetFileAttributes(dll.Path + "WebKnight.ini");
	if (0xFFFFFFFF == attr){ //doesn't exist yet
		if (s.WriteToFile(dll.Path + "WebKnight.ini")){
			strinit2 = "INFO: A file WebKnight.ini was made with the currently loaded settings";
		}else{
			strinit2 = "WARNING: Failed to make a settings file WebKnight.ini";
		}
	}
#endif
	
	logger.SetHeader("AQTRONIX WebKnight 1.3");
	SetLoggerFields();

	if(s.LogPerProcess){
		DWORD id = GetCurrentProcessId();
		CString dir = s.LogDir;
		char buf[34]="";
		dir += ltoa(id,buf,10);
		dir += '\\';
		logger.SetLogDir(dir);
	}else{
		logger.SetLogDir(s.LogDir);
	}
	logger.SetRetention(s.LogRetention);
	logger.Enabled = s.Logging;
	logger.GMTLogging = s.LogGMT;
	logger.NewEntryFlush("AQTRONIX WebKnight loaded");
	logger.NewEntryFlush(strinit);
	if ((strinit2)!="")
		logger.NewEntryFlush(strinit2);

	//are we in logging only mode? if so, warning!
	if(s.ResponseLogOnly)
		logger.NewEntryFlush("WARNING: Firewall is in logging only mode! Nothing will be blocked!");

#ifdef XML_ENABLED
	//write loaded settings to a file (debug/additional check)
	if(s.WriteToFile(dll.Path + "Loaded.xml"))
		logger.NewEntryFlush("INFO: To check if WebKnight is loaded correctly, you can have a look at the currently loaded settings in the file 'Loaded.xml' (start config.exe and open this file).");
#else
	//write loaded settings to a file (debug/additional check)
	if(s.WriteToFile(dll.Path + "Loaded.ini"))
		logger.NewEntryFlush("INFO: To check if WebKnight is loaded correctly, you can have a look at the currently loaded settings in the file 'Loaded.ini'.");
#endif

	lock.Unlock();
}

CWebKnightFilter::~CWebKnightFilter()
{
	CSingleLock lock(&crit, TRUE);
	logger.NewEntryFlush("AQTRONIX WebKnight unloaded");
	lock.Unlock();
}

BOOL CWebKnightFilter::GetFilterVersion(PHTTP_FILTER_VERSION pVer)
{
	// Call default implementation for initialization
	CHttpFilter::GetFilterVersion(pVer);
	// Clear the flags set by base class
	pVer->dwFlags &= ~SF_NOTIFY_ORDER_MASK;

	CSingleLock lock(&crit, TRUE);

	// Set the filter priority
	if(s.AllowLateScanning){
		pVer->dwFlags |= SF_NOTIFY_ORDER_LOW;
		logger.NewEntryFlush("WARNING: Firewall is installed as low priority (possible risk)");
	}else{
		pVer->dwFlags |= SF_NOTIFY_ORDER_HIGH;
		logger.NewEntryFlush("INFO: Firewall is installed as high priority (very secure)");
	}
	
	//are we a global filter? if yes, we can use the OnReadRawData event
	if(s.IsInstalledAsGlobalFilter){
		pVer->dwFlags |= SF_NOTIFY_READ_RAW_DATA;
		logger.NewEntryFlush("INFO: Firewall will process raw data because it is a global filter (OnReadRawData event)");
		logger.NewEntryFlush("WARNING: If the firewall fails to load (red arrow down), make sure it is installed as a global filter or uncheck 'Is Installed As Global Filter' in Global Filter Capabilities (needs a restart)");
	}else{
		logger.NewEntryFlush("WARNING: Firewall will not process raw data because it is not a global filter (no OnReadRawData event, all other capabilities of the firewall will still be used)");
		logger.NewEntryFlush("INFO: If you want to use the additional raw data scanning capabilities of the firewall, you need to install it as a global filter and make sure the 'Is Installed As Global Filter' under Global Filter Capabilities is checked (needs a restart)");
	}
	
	//do we need to scan non-secure port?
	if(s.ScanNonSecurePort){
		pVer->dwFlags |= SF_NOTIFY_NONSECURE_PORT;
		logger.NewEntryFlush("INFO: HTTP traffic will be monitored");
	}else{
		logger.NewEntryFlush("WARNING: HTTP traffic will NOT be monitored");
	}
	
	//do we need to scan secure port?
	if(s.ScanSecurePort){
		pVer->dwFlags |= SF_NOTIFY_SECURE_PORT;
		logger.NewEntryFlush("INFO: HTTPS traffic will be monitored");
	}else{
		logger.NewEntryFlush("WARNING: HTTPS traffic will NOT be monitored");
	}

	pVer->dwFlags	|= SF_NOTIFY_SEND_RAW_DATA | SF_NOTIFY_URL_MAP
					| SF_NOTIFY_PREPROC_HEADERS;

	// Load description string
	TCHAR sz[SF_MAX_FILTER_DESC_LEN+1];
	ISAPIVERIFY(::LoadString(AfxGetResourceHandle(),IDS_FILTER, sz, SF_MAX_FILTER_DESC_LEN));
	_tcscpy(pVer->lpszFilterDesc, sz);

	lock.Unlock();
	return TRUE;
}

DWORD CWebKnightFilter::OnReadRawData(CHttpFilterContext* pCtxt,PHTTP_FILTER_RAW_DATA pRawData)
{

#ifdef ERROR_HANDLE
	try{
#endif

	/*
	 * This event is only available if the filter is installed as a global
	 * filter (not on individual web sites) and will only be called if
	 * s.IsInstalledAsGlobalFilter=1
	 * WARNING: if the value is set to 1 and the filter is not installed as
	 * a global filter then IIS will not load the filter (red arrow down)!
	 */
	CSingleLock lock(&crit, TRUE);

#ifdef LOG_DEBUG_ONREADRAWDATA
	char buf[34]="";
	logger.NewEntry("GLOBAL");
	logger.Append("OnReadRawDataDebug");
	LogClientInfo(pCtxt);
	logger.Append("pRawData->cbInBuffer");
	logger.Append(itoa((int)pRawData->cbInBuffer,buf,10));
	logger.Append("pRawData->cbInData");
	logger.Append(itoa((int)pRawData->cbInData,buf,10));
	logger.Append("pFilterContext");
	logger.Append(itoa((int)pCtxt->m_pFC->pFilterContext,buf,10));
	logger.Flush();
#endif

	logger.NewEntry("GLOBAL");
	logger.Append("OnReadRawData");
	LogClientInfo(pCtxt);

	bool logonly(s.ResponseLogOnly);
	bool hhack(false);
	bool dhack(false);
	
	if(		s.Deny_HEADER_EncodingExploits		||
			s.Deny_HEADER_DirectoryTraversal	||
			s.Deny_HEADER_HighBitShellcode		||
			s.Deny_HEADER_SQLinjection			||
			s.UseDenyHeaderSequences			||
			s.Deny_POST_EncodingExploits		||
			s.Deny_POST_DirectoryTraversal		||
			s.Deny_POST_HighBitShellcode		||
			s.Deny_POST_SQLinjection			||
			s.UseDenyPOSTSequences					){

		//prepare the raw data before processing
		DWORD length = pRawData->cbInData;
		CString str;
		if(length>0){
			LPSTR lps = new char[length + 1];
			strncpy(lps,(LPCSTR)pRawData->pvInData,length);
			//remove embedded nulls (change with spaces)
			for(DWORD i=0;i<length-1;i++){
				if(lps[i]=='\0')
					lps[i]=' ';
			}
			lps[length] = '\0';	//null terminate it (if indata=inbuffer it is not terminated)
			str = lps;
			delete[] lps;
		}else{
			str = "";
		}

#ifdef LOG_DEBUG_ONREADRAWDATA 
		logger.Append("\r\nRaw data:");
		logger.Append(str);
#endif
		
		CString strh; CString strhLog;				//for headers
		CString strd; CString strdLog;				//for post data

		if(pCtxt->m_pFC->pFilterContext == NULL){	//new request?

			int maxheader = str.Find("\r\n\r\n");	//maximum size header
		
			if(maxheader!=-1){						//if end of headers are found (if not, raw data is not complete!)
				strh = str.Left(maxheader);			//only headers
				//ignore the request line (!= header)
				int starth;
				starth = strh.Find("\r\n");
				if (starth==-1){
					strh = "";						//no headers (only request line)!
				}else{
					strh = strh.Right(strh.GetLength()-starth-2);
				}
				strd = str.Right(str.GetLength()-maxheader-4);//only look at the body (POST data)
				//mark request (all headers received)
				pCtxt->m_pFC->pFilterContext = (VOID *) 1;
			}else{
				logger.Append("WARNING: Could not determine end of headers, will process all data as headers");
				strh = str;
				//if double crlf was between events, process this as data as well
				if(str.Left(1)=="\r" || str.Left(1)=="\n"){
					logger.Append("WARNING: Raw data starts with <CR> or <LF>, so this is probably data part and not headers. Will process it as data as well!");
					strd = str;
				}
			}

		}else{	//is additional data
			strd = str;
		}

#ifdef LOG_DEBUG_ONREADRAWDATA
		logger.Append("\r\nHeaders:");	logger.Append(strh);
		logger.Append("\r\nData:");		logger.Append(strd);
#endif

		strhLog = strh.Left(1024); //for log file
		strdLog = strd.Left(1024); //for log file
	
		POSITION pos;
			
		//HEADERS: check for encoding abuse
		if(s.Deny_HEADER_EncodingExploits){
			if(strh.Find("%00")!=-1){
				hhack = true;
				logger.Append("BLOCKED: Encoding exploit in headers (abuse of: '%00')");
			}
		}
		
		strh = CURL::URLEncodedToASCII(strh,true);

		//HEADERS: Deny encoding exploits (normalize twice, except for '+', and reject if change occurs)
		if(s.Deny_HEADER_EncodingExploits){
			if(strh != CURL::URLEncodedToASCII(strh,false)){
				hhack = true;
				logger.Append("BLOCKED: Encoding exploit in headers");
			}
		}

		//HEADERS: Deny directory traversal (parent path)
		if(s.Deny_HEADER_DirectoryTraversal){
			if( (strh.Find("../") + strh.Find("..\\") + strh.Find("/..") + strh.Find("\\..")) != -4){
				hhack = true;
				logger.Append("BLOCKED: Directory traversal not allowed in headers");
			}
		}

		//HEADERS: Deny high bit characters (shellcode)
		if(s.Deny_HEADER_HighBitShellcode){
			char* pstr = strh.GetBuffer(1);
			for(unsigned int i=0;i<strlen(pstr);i++){
				if((unsigned char)pstr[i]>127){
					hhack = true;
					logger.Append("BLOCKED: High bit characters (shellcode) not allowed in headers");
					break;
				}
			}
		}

		strh.MakeLower();

		//HEADERS: Deny SQL injection (words in list should be lowercase)
		if(s.Deny_HEADER_SQLinjection){	
			int sqlcount( s.Deny_SQL_KeyWords.Find(";")!=NULL?-1:0 );
			pos = s.Deny_SQL_KeyWords.GetHeadPosition();
			while (pos != NULL){
				if(strh.Find(s.Deny_SQL_KeyWords.GetNext(pos))!=-1)
					sqlcount += 1;
			}
			if(sqlcount>0){
				if(sqlcount>1){
					hhack = true;
					logger.Append("BLOCKED: possible SQL injection in headers");
				}else{
					logger.Append("WARNING: SQL keyword found in headers (1 more will block request)");
				}
			}
		}

		//HEADERS: Deny certain sequences (words in list should be lowercase)
		if(s.UseDenyHeaderSequences){
			CString seq;
			pos = s.Deny_HEADER_Sequences.GetHeadPosition();
			while (pos != NULL){
				seq = s.Deny_HEADER_Sequences.GetNext(pos);
				if(strh.Find(seq)!=-1){
					hhack = true;
					logger.Append("BLOCKED: '" + seq + "' not allowed in headers");
				}
			}
		}

		if (hhack){
			strhLog.Replace('\r',' ');
			strhLog.Replace('\n',' ');
			logger.Append(strhLog);
		}

		//DATA: check for encoding abuse
		if(s.Deny_POST_EncodingExploits){
			if(strd.Find("%00")!=-1){
				dhack = true;
				logger.Append("BLOCKED: Encoding exploit in data (abuse of: '%00')");
			}
		}

		strd = CURL::URLEncodedToASCII(strd,true);

		//DATA: Deny encoding exploits (normalize twice, except for '+', and reject if change occurs)
		if(s.Deny_POST_EncodingExploits){
			if(strd != CURL::URLEncodedToASCII(strd,false)){
				dhack = true;
				logger.Append("BLOCKED: Encoding exploit in data");
			}		
		}

		//DATA: Deny directory traversal (parent path)
		if(s.Deny_POST_DirectoryTraversal){
			if( (strd.Find("../") + strd.Find("..\\") + strd.Find("/..") + strd.Find("\\..")) != -4){
				dhack = true;
				logger.Append("BLOCKED: Directory traversal not allowed in data");
			}
		}

		//DATA: Deny high bit characters (shellcode)
		if(s.Deny_POST_HighBitShellcode){
			char* pstr = strd.GetBuffer(1);
			for(unsigned int i=0;i<strlen(pstr);i++){
				if((unsigned char)pstr[i]>127){
					dhack = true;
					logger.Append("BLOCKED: High bit characters (shellcode) not allowed in data");
					break;
				}
			}
		}

		strd.MakeLower();

		//DATA: Deny SQL injection (words in list should be lowercase)
		if(s.Deny_POST_SQLinjection){
			int sqlcount(0);
			pos = s.Deny_SQL_KeyWords.GetHeadPosition();
			while (pos != NULL){
				if(strd.Find(s.Deny_SQL_KeyWords.GetNext(pos))!=-1)
					sqlcount += 1;
			}
			if(sqlcount>0){
				if(sqlcount>1){
					dhack = true;
					logger.Append("BLOCKED: possible SQL injection in data");
				}else{
					logger.Append("WARNING: SQL keyword found in data (1 more will block request)");
				}
			}
		}

		//DATA: deny certain sequences (list should be lowercase)
		if(s.UseDenyPOSTSequences){
			CString seq;
			pos = s.Deny_POST_Sequences.GetHeadPosition();
			while (pos != NULL){
				seq = s.Deny_POST_Sequences.GetNext(pos);
				if(strd.Find(seq)!=-1){
					dhack = true;
					logger.Append("BLOCKED: '" + seq + "' not allowed in data");
				}
			}
		}

		if(dhack){
			strdLog.Replace('\r',' ');
			strdLog.Replace('\n',' ');
			logger.Append(strdLog);
		}
	}

	//if possible hack
	if (hhack || dhack){
		logger.Flush();								//flush the log entry
		if(!logonly)								//if we have to do more than logging
			HackResponse(pCtxt);					//show page (BUGS: server header not removed & possible loop)
		lock.Unlock();								//unlock globals
		if(logonly){								//if we are only here to log
			return SF_STATUS_REQ_NEXT_NOTIFICATION;
		}else{										//if we have to do more than logging
			return SF_STATUS_REQ_FINISHED_KEEP_CONN;//make sure IIS does nothing with request
		}
	}else{
		logger.Append("ALLOWED");
		if(!s.LogAllowedRequests)					//do not log allowed requests if asked
			logger.Cancel();						//clear the log entry
		logger.Flush();
		lock.Unlock();								//unlock globals
		return SF_STATUS_REQ_NEXT_NOTIFICATION;
	}

#ifdef ERROR_HANDLE
	}catch(...){
		if(DENY_ON_ERROR){							//deny request on error if asked
			return SF_STATUS_REQ_FINISHED;			//don't keep connection, because of error
		}else{
			return SF_STATUS_REQ_NEXT_NOTIFICATION;
		}
	}
#endif
}

DWORD CWebKnightFilter::OnPreprocHeaders(CHttpFilterContext* pfc, PHTTP_FILTER_PREPROC_HEADERS pHeaders) 
{
#ifdef ERROR_HANDLE
	try{
#endif

	CString instance = GetServerVariable(pfc,"INSTANCE_ID");

	//mark all headers received
	pfc->m_pFC->pFilterContext = (VOID *) 1;

	CSingleLock lock(&crit, TRUE);
	
	if (s.Update()){	//run-time update
		SetLoggerFields(); //update logger header
		logger.NewEntryFlush("INFO: Detected change in settings, reload done!");
#ifdef XML_ENABLED
		logger.NewEntryFlush("INFO: You can have a look at the currently loaded settings in the file 'Loaded.xml' (start config.exe and open this file).");
#else
		logger.NewEntryFlush("INFO: You can have a look at the currently loaded settings in the file 'Loaded.ini'.");
#endif
	}
	
	logger.NewEntry(SITEINSTANCE + instance);
	logger.Append("OnPreprocHeaders");
	LogClientInfo(pfc);
	
	bool hack(false);
	bool logonly(s.ResponseLogOnly);

	if(IsExcluded(instance)){
		logger.Append("Web site instance is excluded from scanning");
	}else{

		POSITION pos;
		CString str;
		CString strLog;
		
		str = GetHeader(pfc,pHeaders,"method");
		strLog = str.Left(2048);
		logger.Append(str);
		//VERB: only allow certain verbs (verbs in list are case sensitive!)
		if(s.UseAllowVerbs){
			bool ok(false);
			pos = s.Allow_VERB_List.GetHeadPosition();
			while (pos != NULL){
				if (str==s.Allow_VERB_List.GetNext(pos)){
					ok = true;
					break;
				}
			}
			if(!ok){
				hack = true;
				logger.Append("BLOCKED: HTTP VERB '" + strLog + "' not allowed");
			}
		}
		
		//VERB: Deny certain verbs (verbs in list should be uppercase)
		str.MakeUpper(); //Also deny lowercase & variants!
		if(s.UseDenyVerbs){
			pos = s.Deny_VERB_List.GetHeadPosition();
			while (pos != NULL){
				if (str==s.Deny_VERB_List.GetNext(pos)){
					hack = true;
					logger.Append("BLOCKED: HTTP VERB '" + strLog + "' not allowed");
					break;
				}
			}
		}

		str = GetHeader(pfc,pHeaders,"url");
		CURL RawUrl(str);
		
		//URL: RFC compliant
		if(s.RFCCompliantURL){
			if(!RawUrl.IsRFCCompliant()){
				hack = true;
				logger.Append(str.Left(4096));
				logger.Append("BLOCKED: URL is not RFC compliant!");
			}
		}

		//URL Scanning (RAW)
		if(s.UseURLRawScan){

			//URL
			logger.Append(RawUrl.Url.Left(4096));

			//URL: Check size
			if(s.LimitURL){	
				if (RawUrl.Url.GetLength() > s.MaxURL){
					hack = true;
					logger.Append("BLOCKED: URL is too long!");
				}
			}

			//URL: check for encoding abuse
			if(s.Deny_URL_EncodingExploits){
				if(RawUrl.Url.Find("%00")!=-1){
					hack = true;
					logger.Append("BLOCKED: Encoding exploit in URL (abuse of: '%00')");
				}
			}

			RawUrl.Url = CURL::URLEncodedToASCII(RawUrl.Url);

			//URL: Encoding Exploits (normalize twice, except for '+', and reject if change occurs)
			if(s.Deny_URL_EncodingExploits){
				if(RawUrl.Url!=CURL::URLEncodedToASCII(RawUrl.Url,false)){
					hack = true;
					logger.Append("BLOCKED: URL embedded encoding exploit");
				}
			}
			//URL: scan
			hack |= ScanUrl(RawUrl.Url);
		}

		//QUERYSTRING (RAW)
		if(s.UseQuerystringRawScan && RawUrl.Querystring!=""){

			//QUERYSTRING
			logger.Append(RawUrl.Querystring.Left(4096));

			hack |= ScanQuerystring(RawUrl.Querystring);
		}
	
		str = GetHeader(pfc,pHeaders,"version");
		strLog = str.Left(1024);
		logger.Append(strLog);

		//VERSION: Check size
		if(s.LimitVersion){
			if (str.GetLength() > s.MaxVersion){
				hack = true;
				logger.Append("BLOCKED: HTTP version too long '" + strLog + "' (new hack?)");
			}
		}
		
		//VERSION: allowed HTTP versions
		if(s.UseAllowVersions){
			pos = s.Allow_VERSION_List.GetHeadPosition();
			CString seq;
			bool ok(false);
			while (pos != NULL){
				seq = s.Allow_VERSION_List.GetNext(pos);
				if(str == seq){
					ok = true;
					break;
				}
			}
			if(!ok){
				hack = true;
				logger.Append("BLOCKED: HTTP version not in allowed list");
			}
		}

		//TO DO: denied http versions?
		
		//VERSION: host header required for HTTP 1.1 (most likely worms/script kiddies/other stuff that is not RFC compliant)
		if(s.RFCCompliantHostHeader){
			if(str=="HTTP/1.1" && GetHeader(pfc,pHeaders,"Host:")==""){
				hack = true;
				logger.Append("BLOCKED: 'Host:' header required for HTTP 1.1 request");
			}
		}

		//HEADERS: Deny certain headers (list is case insensitive by RFC - GetHeader does this for us)
		if(s.UseDenyHeaders){
			pos = s.Deny_HEADER_List.GetHeadPosition();
			while (pos != NULL){
				str=s.Deny_HEADER_List.GetNext(pos);
				LPSTR lpsz = new char[str.GetLength()+1];
				strcpy(lpsz, str);
				CString h = GetHeader(pfc,pHeaders,lpsz);
				if(h!=""){
					hack = true;
					logger.Append("BLOCKED: Header '" + str + "' not allowed");
					logger.Append(h.Left(1024));
				}
				delete[] lpsz;
			}
		}

		//HEADERS: Check length of certain headers
		if(s.UseMaxHeaders){
			pos = s.MaxHeaders.GetStartPosition();
			CString hsize; CString h;
			while(pos!=NULL){
				s.MaxHeaders.GetNextAssoc(pos, h, hsize);
				LPSTR lpsz = new char[h.GetLength()+1];
				strcpy(lpsz, h);
				str = GetHeader(pfc,pHeaders,lpsz);
				if(str.GetLength()>atol(hsize)){
					hack = true;
					logger.Append("BLOCKED: Header '" + h + "' too long");
					logger.Append(str.Left(1024));
				}
				delete[] lpsz;
			}
		}

		//HEADERS: Check Content-Length value
		if(s.LimitContentLength){
			str = GetHeader(pfc,pHeaders,"Content-Length:");
			//to prevent overflow or other types of attack, block request if the length
			//of Content-Length header is longer than length of max allowed value
			//or longer than 9 characters (maximum value of long integer - 1 character)!
			if(str.GetLength() > s.MaxContentLength.GetLength() || str.GetLength() > 9){
				hack = true;
				logger.Append("BLOCKED: Content-Length is too long!");
			}else{
				if (atol(str) > atol(s.MaxContentLength)){
					hack = true;
					logger.Append("BLOCKED: Content-Length is too long!");
					logger.Append(str.Left(1024));
				}
			}
		}

		//HEADERS: Check Content-Types
		if(s.UseAllowHeaderContentType){
			str = GetHeader(pfc,pHeaders,"Content-Type:");
			strLog = str.Left(2048);
			pos = s.Allow_HEADER_ContentType.GetHeadPosition();
			CString seq;
			bool ok(false);
			while (pos != NULL){
				seq = s.Allow_HEADER_ContentType.GetNext(pos);
				if(str.Left(seq.GetLength()>0?seq.GetLength():str.GetLength())==seq){
					ok = true;
					break;
				}
			}
			if(!ok){
				hack = true;
				logger.Append("BLOCKED: Content-Type '" + strLog + "' not allowed");
			}
		}

		//TO DO: denied content-types?

		str = GetHeader(pfc,pHeaders,"Cookie:");
		logger.Append(str);

		//COOKIE: check for encoding abuse
		if(s.Deny_COOKIE_EncodingExploits){
			if(str.Find("%00")!=-1){
				hack = true;
				logger.Append("BLOCKED: Encoding exploit in cookie (abuse of: '%00')");
			}
		}

		str = CURL::URLEncodedToASCII(str,true); //can be encoded: http://wp.netscape.com/newsref/std/cookie_spec.html

		//COOKIE: Deny encoding exploits (normalize twice, except for '+', and reject if change occurs)
		if(s.Deny_COOKIE_EncodingExploits){
			if(str != CURL::URLEncodedToASCII(str,false)){
				hack = true;
				logger.Append("BLOCKED: Encoding exploit in cookie (embedded encoding)");
			}
		}

		//COOKIE: Deny SQL injection
		if(s.Deny_COOKIE_SQLinjection){
			int sqlcount( s.Deny_SQL_KeyWords.Find(";")!=NULL?-1:0 );
			pos = s.Deny_SQL_KeyWords.GetHeadPosition();
			while (pos != NULL){
				if(str.Find(s.Deny_SQL_KeyWords.GetNext(pos))!=-1)
					sqlcount += 1;
			}
			if(sqlcount>0){
				if(sqlcount>1){
					hack = true;
					logger.Append("BLOCKED: possible SQL injection in cookie");
				}else{
					logger.Append("WARNING: SQL keyword found in cookie (1 more will block request)");
				}
			}			
		}
	}

	//if possible hack
	if (hack){
		logger.Flush();								//flush the log entry
		if(!logonly)								//if we have to do more than logging
			HackResponse(pfc);						//show page
		lock.Unlock();								//unlock globals
		if(logonly){								//if we are only here to log
			return CHttpFilter::OnPreprocHeaders(pfc, pHeaders);
		}else{										//if we have to do more than logging
			return SF_STATUS_REQ_FINISHED_KEEP_CONN;//make sure IIS does nothing with request
		}
	}else{
		logger.Append("ALLOWED");
		if(!s.LogAllowedRequests)					//do not log allowed requests if asked
			logger.Cancel();						//clear the log entry
		logger.Flush();
		lock.Unlock();								//unlock globals
		return CHttpFilter::OnPreprocHeaders(pfc, pHeaders);
	}

#ifdef ERROR_HANDLE
	}catch(...){
		if(DENY_ON_ERROR){							//deny request on error if asked
			return SF_STATUS_REQ_FINISHED;			//don't keep connection, because of error
		}else{
			return CHttpFilter::OnPreprocHeaders(pfc, pHeaders);
		}
	}
#endif
}

DWORD CWebKnightFilter::OnUrlMap(CHttpFilterContext* pfc, PHTTP_FILTER_URL_MAP pUrlMap) 
{
#ifdef ERROR_HANDLE
	try{
#endif

	CString instance = GetServerVariable(pfc,"INSTANCE_ID");

	CSingleLock lock(&crit, TRUE);

	logger.NewEntry(SITEINSTANCE + instance);
	logger.Append("OnUrlMap");
	LogClientInfo(pfc);

	const char* p;
	bool hack(false);
	bool logonly(s.ResponseLogOnly);

	if(IsExcluded(instance)){
		logger.Append("Web site instance is excluded from scanning");
	}else{

		//URL: Log it
		p = pUrlMap->pszURL;
		logger.Append(CString(p).Left(9000));

		//URL Scanning
		hack |= ScanUrl(p);
	
		//log Path (mapped URL)
		p = pUrlMap->pszPhysicalPath;
		logger.Append(CString(p).Left(9000));
		
		//FULLPATH
		if(s.Deny_FULLPATH_ParentPath)
			hack |= DenyString(p,"..","BLOCKED: Parent Path attempt ('..')");

		if(s.Deny_FULLPATH_Escaping)
			hack |= DenyChar(p,'%',"BLOCKED: Possible encoding exploit");

		if(s.Deny_FULLPATH_NewLine)
			hack |= DenyChar(p,'\n',"BLOCKED: new line character (new hack?)");		//block newline

		if(s.Deny_FULLPATH_CarriageReturn)
			hack |= DenyChar(p,'\r',"BLOCKED: return character (new hack?)");		//block return

		if(s.Deny_FULLPATH_Backspace)
			hack |= DenyChar(p,'\b',"BLOCKED: backspace character (new hack?)");	//block backspace

		int i; char c; POSITION pos; CString str;
		
		for(i=0;i<s.Deny_FULLPATH_Characters.GetLength();i++){
			c = s.Deny_FULLPATH_Characters.GetAt(i);
			hack |= DenyChar(p,c,"BLOCKED: '" + CString(c) + "' not allowed in path");
		}

		//Start processing separate parts of the physical path
		CFileName fn(p);
		
		//PATH: Deny dot in path (!= full path)
		if(s.Deny_PATH_Dot)
			hack |= DenyChar(fn.Path,'.',"BLOCKED: '.' not allowed in path");
		
		//PATH: Only allow certain paths (I use full paths here, because when
		//      using path, virtual dirs don't work if they are set to c:\somedir
		//      instead of c:\somedir\											*/
		if(s.UseAllowPaths){
			pos = s.Allow_PATH_List.GetHeadPosition();
			bool ok(false);
			while (pos!= NULL){
				str = s.Allow_PATH_List.GetNext(pos);
				if(fn.FullPathLCase.Left(str.GetLength())==str){
					ok = true;
					break;
				}
			}
			if(!ok){
				hack = true;
				logger.Append("BLOCKED: Not in allowed path list '" + fn.FullPath + "'");
			}
		}

		//FILENAME: Deny default document
		if(s.Deny_FILENAME_DefaultDocument){
			if(fn.FileName == ""){
				hack = true;
				logger.Append("BLOCKED: Default document not allowed");
			}
		}

		//FILENAME: Deny certain characters in the filename (characters are case sensitive!)
		for(i=0;i<s.Deny_FILENAME_Characters.GetLength();i++){
			c = s.Deny_FILENAME_Characters.GetAt(i);
			hack |= DenyChar(fn.FileName,c,"BLOCKED: '" + CString(c) + "' not allowed in filename");
		}

		//FILENAME: Deny certain cgi applications (applications in list should be lowercase).
		if(s.UseDenyFilenameCGIApplications){
			pos = s.Deny_FILENAME_CGIApplications.GetHeadPosition();
			while (pos != NULL){
				str = s.Deny_FILENAME_CGIApplications.GetNext(pos);
				hack |= DenyString(fn.FileNameLCase,str,"BLOCKED: accessing/running '" + str + "' file");
			}
		}

		//EXTENSION: Only allow certain extensions (extensions in the list should be lowercase)
		if(s.UseAllowExtensions){
			pos = s.Allow_EXTENSION_List.GetHeadPosition();
			bool ok(false);
			while (pos!= NULL){
				str = s.Allow_EXTENSION_List.GetNext(pos);
				if(fn.ExtensionLCase==str){
					ok = true;
					break;
				}
			}
			if(!ok){
				hack = true;
				logger.Append("BLOCKED: accessing/running '" + fn.Extension + "' file");
			}
		}

		//EXTENSION: Deny certain extensions (extensions in the list should be lowercase)
		if(s.UseDenyExtensions){
			pos = s.Deny_EXTENSION_List.GetHeadPosition();
			while (pos != NULL){
				str = s.Deny_EXTENSION_List.GetNext(pos);
				hack |= DenyString(fn.ExtensionLCase,str,"BLOCKED: accessing/running '" + str + "' file");
			}
		}
		
		str = GetServerVariable(pfc,"QUERY_STRING");
		logger.Append(str.Left(9000));

		//QUERYSTRING
		hack |= ScanQuerystring(str);
	}
		

	//if possible hack attempt
	if(hack){
		logger.Flush();								//flush the log entry
		if(!logonly)								//if we have to do more than logging
			HackResponse(pfc);						//show page
		lock.Unlock();								//unlock globals
		if(logonly){								//if we are only here to log
			return CHttpFilter::OnUrlMap(pfc, pUrlMap);
		}else{										//if we have to do more than logging
			return SF_STATUS_REQ_FINISHED_KEEP_CONN;//make sure IIS does nothing with request
		}
	}else{
		logger.Append("ALLOWED");
		if(!s.LogAllowedRequests)					//do not log allowed requests if asked
			logger.Cancel();						//clear the log entry
		logger.Flush();
		lock.Unlock();								//unlock globals
		return CHttpFilter::OnUrlMap(pfc, pUrlMap);
	}

#ifdef ERROR_HANDLE
	}catch(...){
		if(DENY_ON_ERROR){							//deny request on error if asked
			return SF_STATUS_REQ_FINISHED;			//don't keep connection, because of error
		}else{
			return CHttpFilter::OnUrlMap(pfc, pUrlMap);
		}
	}
#endif
}


DWORD CWebKnightFilter::OnSendRawData(CHttpFilterContext* pfc, PHTTP_FILTER_RAW_DATA pRawData) 
{
	/*
	 * Change server header or remove it.
	 * Log only if an error occured, but deny nothing!
	 */ 
#ifdef ERROR_HANDLE
	try{
#endif

	bool berror(false);
	bool b_http_error(false);
	bool reset(true);

	CString instance = GetServerVariable(pfc,"INSTANCE_ID");

	CSingleLock lock(&crit, TRUE);

	if (instance==""){
		logger.NewEntry("GLOBAL");
	}else{
		logger.NewEntry(SITEINSTANCE + instance);
	}
	logger.Append("OnSendRawData");
	LogClientInfo(pfc);

	if(IsExcluded(instance)){
		logger.Append("Web site instance is excluded from scanning");
	}else{

		/*	HTTP message:
			generic-message = start-line
							  *(message-header CRLF)
							  CRLF
							  [ message-body ]
			start-line      = Request-Line | Status-Line
		*/


		//only if we did not already process this request and this is just additional data
		//if(pfc->m_pFC->pFilterContext != 2){ //this did not work!!! (with HTTP 100 Continue response,
		//next response from server: no server header changed/removed)
		
		//prepare the raw data before processing
		DWORD length = pRawData->cbInData;
		CString resp;
		if(length>0){
			LPSTR lps = new char[length + 1];
			lps = strncpy(lps,(LPCSTR)pRawData->pvInData,length);
			lps[length] = '\0';	//null terminate it (if indata=inbuffer it is not terminated)
			resp = lps;
			delete[] lps;
		}else{
			resp = "";
		}

#ifdef LOG_DEBUG_ONSENDRAWDATA
		logger.Append("Raw data:");
		logger.Append(resp);
#endif

		//start of a http response
		if(resp.Left(5)=="HTTP/"){			
			int pos;
			CString lresp;

			switch(resp[9]){
			case '1':
				//if this is not a class 1 response (100 Continue,101), reset filtercontext pointer
				//otherwise with a simple refresh in browser one might circumvent header checking (filtercontext = void* 1)
				//because filtercontext = per connection (with keep-alives -> same filtercontext)
				reset=false;
				break;
			case '4':
				if(s.Log_HTTP_ERROR_ClientError){
					logger.Append("HTTP Client Error");
					logger.Append(GetServerVariable(pfc,"REQUEST_METHOD"));
					logger.Append(GetServerVariable(pfc,"SCRIPT_NAME"));
					logger.Append(GetServerVariable(pfc,"PATH_INFO"));
					logger.Append(GetServerVariable(pfc,"QUERY_STRING"));
					logger.Append(GetServerVariable(pfc,"HTTP_REFERER"));
					lresp = resp;
					lresp.Replace('\r',' ');
					lresp.Replace('\n',' ');
					logger.Append(lresp);
					b_http_error = true;
				}
				break;
			case '5':
				if(s.Log_HTTP_ERROR_ServerError){
					logger.Append("HTTP Server Error");
					logger.Append(GetServerVariable(pfc,"REQUEST_METHOD"));
					logger.Append(GetServerVariable(pfc,"SCRIPT_NAME"));
					logger.Append(GetServerVariable(pfc,"PATH_INFO"));
					logger.Append(GetServerVariable(pfc,"QUERY_STRING"));
					logger.Append(GetServerVariable(pfc,"HTTP_REFERER"));
					lresp = resp;
					lresp.Replace('\r',' ');
					lresp.Replace('\n',' ');
					logger.Append(lresp);
					b_http_error = true;
				}
				break;
			}

			if(reset)
				pfc->m_pFC->pFilterContext = NULL;
			
			//if we have to do something with server header
			if (s.RemoveServerHeader||s.ChangeServerHeader){

				int maxheader; int poscr;	
				maxheader = resp.Find("\r\n\r\n");				//maximum size header

				if(maxheader!=-1 && maxheader<MAX_HEADER_SIZE){	//only if end of headers are found
					CString headers = resp.Left(maxheader);		//only look at the headers
					pos = headers.Find("Server:");				//position 'Server:' header

					if(pos!=-1){								//double CRLF is in header?
						CString preh = resp.Left(pos);			//copy everything before 'Server:' header
						resp = resp.Mid(pos,length-pos);
						poscr = resp.Find("\r\n");				//end of 'Server:' header

						if(poscr!=-1){							//if end is found
							CString posth(resp.Mid(poscr+2,length-poscr-2));

							//copy everything after 'Server:' header
							CString strres(preh);

							//change server header or remove it (remove has priority)
							if (!s.RemoveServerHeader){			//it really has to be '!' :)
								strres += "Server: ";
								strres += s.ServerHeader;
								strres += "\r\n";
							}
							//append rest of raw data after removed/altered header
							strres += posth;

							char* p = (char*) pfc->AllocMem(strres.GetLength()+1);
							strcpy(p,(LPCTSTR)strres);
							pRawData->pvInData = p;
							pRawData->cbInData = strres.GetLength();
							pRawData->cbInBuffer = strres.GetLength()+1;
							logger.Append("INFO: Changed/removed 'Server:' header");	
						}else{
							logger.Append("WARNING: Error adjusting 'Server:' header (could not find end of header)");
							berror = true;
						}
					}else{
						logger.Append("WARNING: Error adjusting 'Server:' header (header is outside of MAX_HEADER_SIZE or no header found)");
						berror = true;
					}
				}else{
					logger.Append("WARNING: Error adjusting 'Server:' header (could not determine end of headers)");
					berror = true;
				}
			}
		}
	}
	if(b_http_error)
		logger.Flush();

	//only log if an error occured
	if (berror && LOG_ONSENDRAW_ERROR){
		logger.Flush();
	}else{
		logger.Cancel();
	}

	lock.Unlock();

	return CHttpFilter::OnSendRawData(pfc, pRawData);

#ifdef ERROR_HANDLE
	}catch(...){
		//do nothing, some error occured removing/changing server header, but allow request
		return CHttpFilter::OnSendRawData(pfc, pRawData);
	}
#endif
}

// Do not edit the following lines, which are needed by ClassWizard.
#if 0
BEGIN_MESSAGE_MAP(CWebKnightFilter, CHttpFilter)
	//{{AFX_MSG_MAP(CWebKnightFilter)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()
#endif	// 0

///////////////////////////////////////////////////////////////////////
// If your extension will not use MFC, you'll need this code to make
// sure the extension objects can find the resource handle for the
// module.  If you convert your extension to not be dependent on MFC,
// remove the comments arounn the following AfxGetResourceHandle()
// and DllMain() functions, as well as the g_hInstance global.

/****

static HINSTANCE g_hInstance;

HINSTANCE AFXISAPI AfxGetResourceHandle()
{
	return g_hInstance;
}

BOOL WINAPI DllMain(HINSTANCE hInst, ULONG ulReason,
					LPVOID lpReserved)
{
	if (ulReason == DLL_PROCESS_ATTACH)
	{
		g_hInstance = hInst;
	}

	return TRUE;
}

****/

CString CWebKnightFilter::GetServerVariable(CHttpFilterContext* pCtxt,char* sv)
{
	/*
	 * This function will retrieve a server variable.
	 * Function starts with a buffer of MIN_HEADER_SIZE if this buffer is too
	 * small it will try again with the size the web server gave us, or if no
	 * size in indicated, try with a buffer of MAX_HEADER_SIZE
	 */
	DWORD bufsize = MIN_HEADER_SIZE;
	char buf[MIN_HEADER_SIZE]="";
    
	//get header value with MIN_HEADER_SIZE (average minimum header size)
	if(pCtxt->GetServerVariable(sv,buf,&bufsize)){
		for(DWORD i=0;i<bufsize-1;i++){	//replace all embedded nulls with space
			if(buf[i]=='\0') buf[i]=' ';
		}
		buf[bufsize-1]='\0';			//it should be null terminated already, but make sure it is. You never know!
		return CString(buf);
	}else{
		switch(GetLastError()){
			case ERROR_INSUFFICIENT_BUFFER:{	//if the buffer was too small, try again with value set by web server in bufsize
				CString tmp;				
				char* buf2 = new char[bufsize];
				if(pCtxt->GetServerVariable(sv,buf2,&bufsize)){
					for(DWORD i=0;i<bufsize-1;i++){	//replace all embedded nulls with space
						if(buf2[i]=='\0') buf2[i]=' ';
					}
					buf2[bufsize-1]='\0';	//it should be null terminated already, but make sure it is. You never know!
					tmp=buf2;
				}else{
					tmp="Error retrieving server variable"; //some error occured
				}
				delete[] buf2;
				return tmp;
				break;
			}
			case ERROR_MORE_DATA: //buffer was too small, but no size set in bufsize, try again with MAX_HEADER_SIZE
				bufsize = MAX_HEADER_SIZE;
				char buf2[MAX_HEADER_SIZE];
				if(pCtxt->GetServerVariable(sv,buf2,&bufsize)){
					for(DWORD i=0;i<bufsize-1;i++){	//replace all embedded nulls with space
						if(buf2[i]=='\0') buf2[i]=' ';
					}
					buf2[bufsize-1]='\0';	//it should be null terminated already, but make sure it is. You never know!
					return CString(buf2);
				}else{
					return "Error retrieving server variable"; //some error occured
				}
				break;
			default:
				return ""; //variable is not present or some other error occured
				break;
		}
	}
}

CString CWebKnightFilter::GetHeader(CHttpFilterContext* pfc, PHTTP_FILTER_PREPROC_HEADERS pHeaders, char* h)
{
	/*
	 * This function will retrieve a header value.
	 * Function starts with a buffer of MIN_HEADER_SIZE if this buffer is too
	 * small it will try again with the size the web server gave us, or if no
	 * size in indicated, try with a buffer of MAX_HEADER_SIZE
	 */
	DWORD bufsize = MIN_HEADER_SIZE;
	char buf[MIN_HEADER_SIZE]="";
    
	//get header value with MIN_HEADER_SIZE (average minimum header size)
	if(pHeaders->GetHeader(pfc->m_pFC,h,buf,&bufsize)){
		for(DWORD i=0;i<bufsize-1;i++){	//replace all embedded nulls with space
			if(buf[i]=='\0') buf[i]=' ';
		}
		buf[bufsize-1]='\0';			//it should be null terminated already, but make sure it is. You never know!
		return CString(buf);
	}else{
		switch(GetLastError()){
			case ERROR_INSUFFICIENT_BUFFER:{	//if the buffer was too small, try again with value set by web server in bufsize
				CString tmp;				
				char* buf2 = new char[bufsize];
				if(pHeaders->GetHeader(pfc->m_pFC,h,buf2,&bufsize)){
					for(DWORD i=0;i<bufsize-1;i++){	//replace all embedded nulls with space
						if(buf2[i]=='\0') buf2[i]=' ';
					}
					buf2[bufsize-1]='\0';	//it should be null terminated already, but make sure it is. You never know!
					tmp=buf2;
				}else{
					tmp="Error retrieving header"; //some error occured
				}
				delete[] buf2;
				return tmp;
				break;
			}
			case ERROR_MORE_DATA: //buffer was too small, but no size set in bufsize, try again with MAX_HEADER_SIZE
				bufsize = MAX_HEADER_SIZE;
				char buf2[MAX_HEADER_SIZE];
				if(pHeaders->GetHeader(pfc->m_pFC,h,buf2,&bufsize)){
					for(DWORD i=0;i<bufsize-1;i++){	//replace all embedded nulls with space
						if(buf2[i]=='\0') buf2[i]=' ';
					}
					buf2[bufsize-1]='\0';	//it should be null terminated already, but make sure it is. You never know!
					return CString(buf2);
				}else{
					return "Error retrieving header"; //some error occured
				}
				break;
			default:
				return ""; //header is not present or some other error occured
				break;
		}
	}
}

inline bool CWebKnightFilter::DenyString(const char *p, const char *pStr, const char *LogMsg)
{
	if(strstr(p,pStr)!=NULL) {
		logger.Append(LogMsg);
		return true;
	}else{
		return false;
	}
}

inline bool CWebKnightFilter::DenyChar(const char *p, char c, const char *LogMsg)
{
	if(strchr(p,c)!=NULL) {
		logger.Append(LogMsg);
		return true;
	}else{
		return false;
	}
}

inline void CWebKnightFilter::HackResponse(CHttpFilterContext* pfc)
{
	if(s.ResponseDirectly){
		//direct response to client (writeclient)
		char tmp[34];
		CString buf("HTTP/1.1 ");
		if(s.UseResponseStatus){
			buf += s.ResponseStatus;
		}else{
			buf += "404 Object Not Found";
		}
		if(!s.RemoveServerHeader){
			buf += "\r\nServer: ";
			if(s.ChangeServerHeader){
				buf += s.ServerHeader;
			}else{
				buf += "WebKnight/1.3";
			}
		}
		_tzset();
		CString d = CTime::GetCurrentTime().FormatGmt("%a, %d %b %Y %H:%M:%S GMT");
		buf += "\r\nDate: " + d;
		buf += "\r\nContent-Type: text/html; charset=windows-1252";
		buf += "\r\nContent-Length: "; buf += itoa(s.ResponseContents.GetLength(),tmp,10);
		buf += "\r\nPragma: no-cache";
		buf += "\r\nCache-control: no-cache";
		buf += "\r\nExpires: " + d;
		buf += "\r\n\r\n";
		buf += s.ResponseContents;
		DWORD szbuf(buf.GetLength());
		pfc->WriteClient(buf.GetBuffer(0),&szbuf);
	}else if(s.ResponseRedirect){
		//redirect request to a url
		CString str;
		str = "Location: " + s.ResponseRedirectURL + "\r\n";
		LPSTR lps = new char[str.GetLength()+1];
		strcpy(lps, str);
		pfc->AddResponseHeaders(lps,0);
		delete[] lps;
		pfc->ServerSupportFunction(SF_REQ_SEND_RESPONSE_HEADER,"302 Object Moved",NULL,NULL);
	}else if(s.UseResponseStatus){
		//respond with special header only
		pfc->ServerSupportFunction(SF_REQ_SEND_RESPONSE_HEADER,(VOID*)(LPCSTR)s.ResponseStatus,NULL,NULL);
	}
	pfc->m_pFC->pFilterContext = NULL; //reset for OnReadRawData
}

inline bool CWebKnightFilter::IsExcluded(CString &instance)
{
	if(s.UseExcludedInstances){
		POSITION pos = s.ExcludedInstances.GetHeadPosition();
		while (pos!= NULL){
			if(s.ExcludedInstances.GetNext(pos)==instance)
				return true;
		}
	}
	return false;
}

inline void CWebKnightFilter::LogClientInfo(CHttpFilterContext* pfc)
{
	if(s.LogClientIP) logger.Append(GetServerVariable(pfc,"REMOTE_ADDR"));
	if(s.LogUserName) logger.Append(GetServerVariable(pfc,"LOGON_USER"));
	if(s.Log_HTTP_VIA) logger.Append(GetServerVariable(pfc,"HTTP_VIA"));
	if(s.Log_HTTP_X_FORWARDED_FOR) logger.Append(GetServerVariable(pfc,"HTTP_X_FORWARDED_FOR"));
	if(s.LogUserAgent) logger.Append(GetServerVariable(pfc,"HTTP_USER_AGENT"));
}

inline void CWebKnightFilter::SetLoggerFields()
{
	//set logger fields
	CStringList fields;
	fields.AddTail("Time");
	fields.AddTail("Site Instance");
	fields.AddTail("Event");
	if(s.LogClientIP) fields.AddTail("Client IP");
	if(s.LogUserName) fields.AddTail("Username");
	if(s.Log_HTTP_VIA) fields.AddTail("Via header");
	if(s.Log_HTTP_X_FORWARDED_FOR) fields.AddTail("HTTP_X_FORWARDED_FOR");
	if(s.LogUserAgent) fields.AddTail("User agent");
	fields.AddTail("Additional info about request (event specific)");
	logger.SetHeaderFields(fields);
}

inline bool CWebKnightFilter::ScanUrl(const char *p)
{
	bool hack(false);
	CString str; POSITION pos;
	CString URL(p);	URL.MakeLower();

	//URL: Only allow certain characters as the start of a URL (list should be lowercase)
	if(s.UseAllowURL){
		pos = s.Allow_URL_List.GetHeadPosition();
		bool ok(false);
		while (pos!= NULL){
			str = s.Allow_URL_List.GetNext(pos);
			if(URL.Left(str.GetLength())==str){
				ok = true;
				break;
			}
		}
		if(!ok){
			hack = true;
			logger.Append("BLOCKED: URL not in allowed list");
		}
	}

	//URL: Deny parent path
	if (s.Deny_URL_ParentPath)
		hack |= DenyString(p,"..","BLOCKED: '..' not allowed in URL (parent path)");

	//URL: Deny trailing dot in directory
	if (s.Deny_URL_TrailingDotInDir)
		hack |= DenyString(p,"./","BLOCKED: './' not allowed in URL (trailing dot on a directory name)");
	
	//URL: Deny backslash
	if (s.Deny_URL_Backslash)
		hack |= DenyChar(p,'\\',"BLOCKED: '\\' not allowed in URL (backslashes)");

	//URL: Deny alternate stream access
	if (s.Deny_URL_AlternateStream)
		hack |= DenyChar(p,':',"BLOCKED: ':' not allowed in URL (alternate stream access)");

	//URL: Deny escaping
	if (s.Deny_URL_Escaping)
		hack |= DenyChar(p,'%',"BLOCKED: '%' not allowed in URL (escape character)");

	//URL: Deny running multiple applications
	if (s.Deny_URL_MultipleCGI)
		hack |= DenyChar(p,'&',"BLOCKED: '&' not allowed in URL (running multiple CGI processes in single request)");
	
	//URL: Deny certain characters
	char c;	int i;
	for(i=0;i<s.Deny_URL_Characters.GetLength();i++){
		c = s.Deny_URL_Characters.GetAt(i);
		hack |= DenyChar(p,c,"BLOCKED: '" + CString(c) + "' not allowed in URL");
	}

	//URL: Deny high bit characters (shellcode)
	if(s.Deny_URL_HighBitShellcode){
		for(unsigned int i=0;i<strlen(p);i++){
			if((unsigned char)p[i]>127){
				hack = true;
				logger.Append("BLOCKED: High bit characters (shellcode) not allowed in URL");
				break;
			}
		}
	}

	//URL: Deny certain sequences (list should be lowercase)
	if(s.UseDenyURLSequences){
		pos = s.Deny_URL_Sequences.GetHeadPosition();
		while (pos != NULL){
			str = s.Deny_URL_Sequences.GetNext(pos);
			hack |= DenyString((LPCTSTR)URL,str,"BLOCKED: '" + str + "' not allowed in URL");
		}
	}
	
	return hack;
}

inline bool CWebKnightFilter::ScanQuerystring(CString& str)
{
	bool hack(false);
	POSITION pos;

	//QUERYSTRING: size
	if(s.LimitQueryString){
		if (str.GetLength()>s.MaxQueryString){
			hack = true;
			logger.Append("BlOCKED: QueryString is too long!");
		}
	}

	//QUERYSTRING: encoding abuse
	if(s.Deny_QUERYSTRING_EncodingExploits){
		if(str.Find("%00")!=-1){
			hack = true;
			logger.Append("BLOCKED: Encoding exploit in querystring (abuse of: '%00')");
		}
	}
	
	str = CURL::URLEncodedToASCII(str,true);

	//QUERYSTRING: Encoding Exploits (normalize twice, except for '+', and reject if change occurs)
	if(s.Deny_QUERYSTRING_EncodingExploits){
		if(str!=CURL::URLEncodedToASCII(str,false)){
			hack = true;
			logger.Append("BLOCKED: Querystring embedded encoding exploit");
		}
	}

	//QUERYSTRING: Deny directory traversal (parent path)
	if(s.Deny_QUERYSTRING_DirectoryTraversal){
		if( (str.Find("../") + str.Find("..\\") + str.Find("/..") + str.Find("\\..")) != -4){
			hack = true;
			logger.Append("BLOCKED: Directory traversal not allowed in querystring");
		}
	}

	//QUERYSTRING: Deny high bit characters (shellcode)
	if(s.Deny_QUERYSTRING_HighBitShellcode){
		char* pstr = str.GetBuffer(1);
		for(unsigned int i=0;i<strlen(pstr);i++){
			if((unsigned char)pstr[i]>127){
				hack = true;
				logger.Append("BLOCKED: High bit characters (shellcode) not allowed in querystring");
				break;
			}
		}
	}

	str.MakeLower();
	
	//QUERYSTRING: SQL injection (words in list should be lowercase)
	if(s.Deny_QUERYSTRING_SQLinjection){
		int sqlcount(0);
		pos = s.Deny_SQL_KeyWords.GetHeadPosition();
		while (pos != NULL){
			if(str.Find(s.Deny_SQL_KeyWords.GetNext(pos))!=-1)
				sqlcount += 1;
		}
		if(sqlcount>0){
			if(sqlcount>1){
				hack = true;
				logger.Append("BLOCKED: possible SQL injection in querystring");
			}else{
				logger.Append("WARNING: SQL keyword found in querystring (1 more will block request)");
			}
		}
	}

	//QUERYSTRING: deny certain sequences (list should be lowercase)
	if(s.UseDenyQuerystringSequences){
		CString seq;
		pos = s.Deny_QUERYSTRING_Sequences.GetHeadPosition();
		while (pos != NULL){
			seq = s.Deny_QUERYSTRING_Sequences.GetNext(pos);
			if(str.Find(seq)!=-1){
				hack = true;
				logger.Append("BLOCKED: '" + seq + "' not allowed in querystring");
			}
		}
	}

	return hack;
}
