/*
	AQTRONIX WebKnight - ISAPI Filter for securing IIS
	Copyright 2002 Parcifal Aertssen

	This file is part of AQTRONIX WebKnight.

    AQTRONIX WebKnight is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.

    AQTRONIX WebKnight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AQTRONIX WebKnight; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 */

// WebKnightSettings.cpp: implementation of the CWebKnightSettings class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Globals.h"
#include "WebKnightSettings.h"
#include "FileName.h"

#define MAX_INI_FILE_SIZE 40960  //maximum size of an WebKnight ini file
#define MAX_XML_FILE_SIZE 196000 //maximum size of an WebKnight xml file

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CWebKnightSettings::CWebKnightSettings()
{
	LoadDefaults(); //always load defaults (failover redundancy)
}

CWebKnightSettings::~CWebKnightSettings()
{
}

void CWebKnightSettings::LoadDefaults()
{
#ifdef IISBUILD
	LoadIISDefaults();
#endif
#ifdef ISABUILD
	LoadISADefaults();
#endif
}

inline void CWebKnightSettings::LoadIISDefaults()
{
	/*
	 * Loads the default settings into memory
	 */

	//Scanning engine
	AllowLateScanning = false;			//high or low priority (true=low)
	ScanNonSecurePort = true;			//scan http requests
	ScanSecurePort = true;				//scan https requests

	//Scanning engine: excluded web instances. These are the web sites you
	//don't want to be scanned by this filter, because they are not normal
	//websites and use their own filter/extension to process requests.
	//Examples are Outlook Web Access, MS Proxy 2.0 (not ISA Server), SQL XML (only blocked if URL queries are enabled(SQL injection))
	UseExcludedInstances = true;
	ExcludedInstances.RemoveAll();
	ExcludedInstances.AddTail("100");	//Outlook Web Access - virtual HTTP server (custom)
	ExcludedInstances.AddTail("101");	//Outlook Web Access - virtual HTTP server (custom)
	ExcludedInstances.AddTail("102");	//Outlook Web Access - virtual HTTP server (custom)
	ExcludedInstances.AddTail("103");	//Outlook Web Access - virtual HTTP server (custom)
	ExcludedInstances.AddTail("104");	//Outlook Web Access - virtual HTTP server (custom)
	ExcludedInstances.AddTail("105");	//Outlook Web Access - virtual HTTP server (custom)
	//ExcludedInstances.AddTail("1");	//Default web site (ONLY if you are running MS Proxy 2.0 on the default instance and still want to protect other web instances)

	//get the directory of the .dll file
	char dir[MAX_PATH] = "";
	CFileName dll;
	if(GetModuleFileName(GetModuleHandle("WebKnight.dll"),dir,MAX_PATH))
		dll = dir;
	//get windows directory
	GetWindowsDirectory(dir,MAX_PATH);
	CString windir(dir);

	//Response handling
	ResponseDirectly = true;			//write contents of nohack.htm to client directly (without redirect)
	UseResponseStatus = true;			//if ResponseRedirect=true, then this has no effect
	//ResponseStatus = "200 OK";		//to see the next step of the script kiddie (for testing only)
	ResponseStatus = "999 No Hacking";	//"31337 No Hacking";//"404 Object Not Found"; //maybe an invalid response will crash script kiddiots tools (TO DO: check this)
	ResponseRedirect = false;			//redirect to a custom url
	ResponseRedirectURL = "/nohack.htm";
	ResponseLogOnly = false;			//if true, log only (debug function)

	//Reponse contents
	CFile rf;
	ResponseContents = "<HTML><HEAD><TITLE>WebKnight Application Firewall Alert</TITLE></HEAD><BODY><H1>WebKnight Application Firewall Alert</H1></BODY></HTML>";
	if(rf.Open(dll.Path + "nohack.htm",CFile::modeRead|CFile::shareDenyNone)){
		try{
			bool error(false);
			char buf[MAX_INI_FILE_SIZE+1];
			try{
				rf.Read(buf,MAX_INI_FILE_SIZE);
			}catch(CFileException* fe){
				fe->Delete();
				error = true;
			}
			buf[MAX_INI_FILE_SIZE]='\0';
			if(!error)
				ResponseContents = buf;
		}catch(...){
			//do nothing
		}
		rf.Close();
	}

	//logging
	Logging = true;

#ifdef SYSTEM_LOG_BUILD
	LogDir = windir + "\\System32\\LogFiles\\WebKnight\\"; //for global filter
	LogDirDefault = LogDir;
#pragma message("***** This build will log under the LogFiles folder of your Windows NT installation! *****")
#else
	LogDir = dll.Path  + "LogFiles\\";				   //<dir of dll>\LogFiles
	LogDirDefault = "<Path of WebKnight>\\LogFiles\\";
#endif
	
	LogGMT = true;
	LogPerProcess = false;			//unique logdir per process (for hosting with multiple instances of the webserver)
	LogRetention = 28;				//number of days to keep log files
	LogAllowedRequests = false;		//log requests which are allowed
	LogClientIP = true;
	LogUserName = true;
	Log_HTTP_VIA = false;			//HTTP VIA headers (used in TRACE)
	Log_HTTP_X_FORWARDED_FOR = false;//certain ISPs use a proxy which add this header (one of those ISPs is telenet.be)
	LogUserAgent = false;			//not essential information except maybe to identify script kiddie tools
#ifdef PRIVATEBUILD
	Log_HTTP_ERROR_ClientError = true;//http 404 errors, etc.
#else
	Log_HTTP_ERROR_ClientError = false;//http 404 errors, etc.
#endif

	Log_HTTP_ERROR_ServerError = true;//http 500 errors...

	//Limits
	LimitContentLength = true;
	LimitURL = true;
	LimitQueryString = true;
	LimitVersion = true;
	MaxContentLength="5300642";//(see bugtraq: Content-Length DoS 5300643)//"30000000";//max value of content length header (to prevent overflow, it cannot be longer than 9 characters)
	MaxURL = 1024; //16384;		//max length of url (MAX_PATH=260)
	MaxQueryString=1024;//4096;	//max length of querystring
	MaxVersion=22;				//max length of version string, in the form of HTTP/<major>.<minor> (e.g. HTTP/1.1) (HTTP/0.9 bug? buffer overflow? TO DO)
	UseAllowVersions = true;
	Allow_VERSION_List.RemoveAll();
	Allow_VERSION_List.AddTail("");	//HTTP/0.9
	Allow_VERSION_List.AddTail("HTTP/1.0");
	Allow_VERSION_List.AddTail("HTTP/1.1");
	UseMaxHeaders = true;
	MaxHeaders.SetAt("Accept:","1024");				//rfc & filters REQUEST
	MaxHeaders.SetAt("Accept-Charset:","1024");		//rfc			REQUEST
	MaxHeaders.SetAt("Accept-Encoding:","256");		//rfc & filters REQUEST
	MaxHeaders.SetAt("Accept-Language:","356");		//rfc & filters REQUEST (Accept language of Netscape/mozilla on Mac is 356 long)
	MaxHeaders.SetAt("Accept-Ranges:","128");		//rfc			REQUEST/RESPONSE
	//MaxHeaders.SetAt("Age:","32");				//rfc					RESPONSE
	MaxHeaders.SetAt("Allow:","512");				//rfc			REQUEST/RESPONSE
	MaxHeaders.SetAt("Authorization:","4000");		//rfc & filters REQUEST
	//MaxHeaders.SetAt("Authentication-Info:","4000");//rfc2069				RESPONSE
	MaxHeaders.SetAt("Cache-Control:","256");		//rfc			REQUEST/RESPONSE
	MaxHeaders.SetAt("Charge-To:","512");			//w3c			REQUEST
	MaxHeaders.SetAt("Connection:","256");			//rfc & filters REQUEST/RESPONSE
	MaxHeaders.SetAt("Content-Base:","1024");		//rfc MIME		REQUEST/RESPONSE
	MaxHeaders.SetAt("Content-Description:","1000");//rfc MIME		REQUEST/RESPONSE
	MaxHeaders.SetAt("Content-Disposition:","256");	//rfc MIME		REQUEST/RESPONSE
	MaxHeaders.SetAt("Content-Encoding:","128");	//rfc			REQUEST/RESPONSE
	MaxHeaders.SetAt("Content-Id:","1024");			//rfc MIME		REQUEST/RESPONSE
	MaxHeaders.SetAt("Content-Language:","256");	//rfc			REQUEST/RESPONSE
	//MaxHeaders.SetAt("Content-Length:","9");		//rfc - is checked elsewhere
	MaxHeaders.SetAt("Content-Location:","1024");	//rfc			REQUEST/RESPONSE
	MaxHeaders.SetAt("Content-MD5:","4000");		//rfc			REQUEST/RESPONSE
	MaxHeaders.SetAt("Content-Range:","256");		//rfc			REQUEST/RESPONSE
	MaxHeaders.SetAt("Content-Transfer-Encoding:","128");//winhttp.dll	NOT ALLOWED IN HTTP
	MaxHeaders.SetAt("Content-Type:","256");		//rfc			REQUEST/RESPONSE
	MaxHeaders.SetAt("Cookie:","4095");				//winhttp & filters
	//MaxHeaders.SetAt("Cost:","256");				//winhttp.dll			RESPONSE
	MaxHeaders.SetAt("Date:","64");					//rfc			REQUEST/RESPONSE
	//MaxHeaders.SetAt("DAV:","128");				//rfc 2518(DAV)			RESPONSE	(not in winhttp.dll)
	MaxHeaders.SetAt("Depth:","64");				//rfc 2518(DAV)	REQUEST				(not in winhttp.dll)
	MaxHeaders.SetAt("Derived-From:","9000");		//winhttp.dll	REQUEST/RESPONSE?
	MaxHeaders.SetAt("Destination:","1024");		//rfc 2518(DAV)	REQUEST				(not in winhttp.dll)
	//MaxHeaders.SetAt("Etag:","4000");				//rfc					RESPONSE
	MaxHeaders.SetAt("Expect:","256");				//rfc			REQUEST
	MaxHeaders.SetAt("Expires:","64");				//rfc			REQUEST?/RESPONSE
	MaxHeaders.SetAt("Forwarded:","9000");			//winhttp.dll	REQUEST/RESPONSE?
	MaxHeaders.SetAt("From:","321");				//rfc			REQUEST/RESPONSE
	MaxHeaders.SetAt("Host:","256");				//rfc & filters REQUEST
	MaxHeaders.SetAt("If:","4095");					//rfc 2518(DAV)	REQUEST
	MaxHeaders.SetAt("If-Match:","256");			//rfc			REQUEST
	MaxHeaders.SetAt("If-Modified-Since:","64");	//rfc & filters REQUEST
	MaxHeaders.SetAt("If-None-Match:","256");		//rfc & filters REQUEST
	MaxHeaders.SetAt("If-Range:","4000");			//rfc			REQUEST
	MaxHeaders.SetAt("If-Unmodified-Since:","64");	//rfc			REQUEST
	MaxHeaders.SetAt("Last-Modified:","64");		//rfc			REQUEST
	//MaxHeaders.SetAt("Link:","1024");				//winhttp.dll	        RESPONSE
	//MaxHeaders.SetAt("Location:","1024");			//rfc					RESPONSE
	MaxHeaders.SetAt("Lock-Token:","1024");			//rfc 2518(DAV)	REQUEST
	MaxHeaders.SetAt("Max-Forwards:","32");			//rfc			REQUEST
	MaxHeaders.SetAt("Message-id:","1024");			//winhttp.dll	REQUEST/RESPONSE
	MaxHeaders.SetAt("Mime-Version:","128");		//winhttp.dll	REQUEST?/RESPONSE? (winhttp.dll contains 1 header more than wininet.dll: "Mime-Version:")
	MaxHeaders.SetAt("Ms-Echo-Reply:","128");		//winhttp.dll	REQUEST?/RESPONSE?
	MaxHeaders.SetAt("Ms-Echo-Request:","128");		//winhttp.dll	REQUEST?/RESPONSE?
	MaxHeaders.SetAt("Orig-Uri:","1024");			//winhttp.dll	REQUEST
	MaxHeaders.SetAt("Overwrite:","64");			//rfc 2518(DAV)	REQUEST			(not in winhttp.dll)
	MaxHeaders.SetAt("PassportConfig:","4000");		//winhttp.dll		TO DO: in request & value?
	MaxHeaders.SetAt("PassportURLs:","4000");		//winhttp.dll		TO DO: in request & value?
	MaxHeaders.SetAt("Pragma:","256");				//rfc			REQUEST/RESPONSE
	//MaxHeaders.SetAt("Proxy-Authenticate:","256");//rfc					RESPONSE
	MaxHeaders.SetAt("Proxy-Authorization:","4000");//rfc			REQUEST
	MaxHeaders.SetAt("Proxy-Connection:","256");	//winhttp.dll	REQUEST/RESPONSE?
	MaxHeaders.SetAt("Proxy-Support:","4000");		//winhttp.dll		TO DO: in request & value?
	MaxHeaders.SetAt("Public:","4000");				//winhttp.dll		TO DO: in request & value?
	MaxHeaders.SetAt("Range:","256");				//rfc			REQUEST/RESPONSE?
	MaxHeaders.SetAt("Referer:","1024");			//rfc & filters REQUEST
	//MaxHeaders.SetAt("Refresh:","64");			//winhttp.dll			RESPONSE
	//MaxHeaders.SetAt("Retry-After:","64");		//rfc					RESPONSE
	//MaxHeaders.SetAt("Server:","256");			//rfc					RESPONSE
	//MaxHeaders.SetAt("Set-Cookie:","4095");		//winhttp.dll			RESPONSE
	//MaxHeaders.SetAt("Status-URI:","1024");		//rfc 2518(DAV)			RESPONSE (not in winhttp.dll)
	MaxHeaders.SetAt("Timeout:","1024");			//rfc 2518(DAV)	REQUEST			 (not in winhttp.dll)
	MaxHeaders.SetAt("Title:","9000");				//winhttp.dll	REQUEST?/RESPONSE
	MaxHeaders.SetAt("TE:","128");					//rfc only!		REQUEST/RESPONSE
	MaxHeaders.SetAt("Trailer:","128");				//rfc only!		REQUEST?/RESPONSE
	MaxHeaders.SetAt("Transfer-Encoding:","128");	//rfc & filters REQUEST/RESPONSE
	MaxHeaders.SetAt("Translate:","256");			//winhttp.dll	REQUEST
	MaxHeaders.SetAt("Unless-Modified-Since:","64");//winhttp.dll	REQUEST
	MaxHeaders.SetAt("Upgrade:","256");				//rfc			REQUEST/RESPONSE
	MaxHeaders.SetAt("Uri:","1024");				//winhttp.dll	REQUEST/RESPONSE
	MaxHeaders.SetAt("User-Agent:","256");			//rfc & filters	REQUEST
	MaxHeaders.SetAt("Vary:","128");				//rfc			REQUEST?/RESPONSE
	MaxHeaders.SetAt("Via:","9000");				//rfc			REQUEST/RESPONSE
	//MaxHeaders.SetAt("Warning:","4000");			//rfc					RESPONSE
	MaxHeaders.SetAt("Version:","128");				//www.w3.org	REQUEST/RESPONSE
	//MaxHeaders.SetAt("WWW-Authenticate:","256");	//rfc					RESPONSE

	//URL
	Deny_URL_EncodingExploits = true;
	Deny_URL_ParentPath = true;			//no ".." in url
	Deny_URL_TrailingDotInDir = true;	//no "./" in url
	Deny_URL_Backslash = true;			//no '\' in url
	Deny_URL_AlternateStream = false;	//you should allow ':' in url (see rfc 2616), urlscan blocks it, which is wrong!!!
	Deny_URL_Escaping = true;			//no '%' in url (after url normalization)
	Deny_URL_MultipleCGI = true;		//no '&' in url
	Deny_URL_Characters = "?#";			//other characters to block
	Deny_URL_HighBitShellcode = true;	//deny high bit shellcode (ascii>127)

	//URL: Deny certain sequences (must be lowercase)
	UseDenyURLSequences = true;
	Deny_URL_Sequences.RemoveAll();
	Deny_URL_Sequences.AddTail("/scripts");		//scripts (needed by MS Proxy 2.0, frontpage - not by ISA Server)
	Deny_URL_Sequences.AddTail("/iishelp");		//iishelp
	Deny_URL_Sequences.AddTail("/iisadmin");	//iisadmin (remote administration)
	Deny_URL_Sequences.AddTail("/msadc");		//Microsoft Advanced Data Connector = RDS (Remote Data Service)
	Deny_URL_Sequences.AddTail("/printers");	//installed printers
	Deny_URL_Sequences.AddTail("/samples");		//just samples
	Deny_URL_Sequences.AddTail("/iisadmpwd");	//change windows password
	Deny_URL_Sequences.AddTail("/_vti_aut");	//frontpage extensions
	Deny_URL_Sequences.AddTail("/_vti_bin");	//frontpage extensions
	Deny_URL_Sequences.AddTail("/_vti_rpc");	//frontpage extensions
	Deny_URL_Sequences.AddTail("/_vti_pvt");	//frontpage extensions
	Deny_URL_Sequences.AddTail("/admcgi");		//frontpage extensions
	Deny_URL_Sequences.AddTail("/admisapi");	//frontpage extensions
	Deny_URL_Sequences.AddTail("/_private");	//frontpage extensions
	Deny_URL_Sequences.AddTail("/_vti_cnf");	//frontpage extensions
	Deny_URL_Sequences.AddTail("/_vti_log");	//frontpage extensions
	Deny_URL_Sequences.AddTail("/_vti_script");//frontpage extensions
	Deny_URL_Sequences.AddTail("/_vti_txt");	//frontpage extensions
	Deny_URL_Sequences.AddTail("/_mem_bin");	//frontpage extensions
	Deny_URL_Sequences.AddTail("/ftp only");	//ftp only :)
	Deny_URL_Sequences.AddTail("/ftp_only");	//ftp only :)
	Deny_URL_Sequences.AddTail("/ftp-only");	//ftp only :)
	//Deny_URL_Sequences.AddTail("/winnt");		//operating system sensitive dirs
	//Deny_URL_Sequences.AddTail("/system");
	Deny_URL_Sequences.AddTail("/system32");
	Deny_URL_Sequences.AddTail("/adsamples");
	Deny_URL_Sequences.AddTail("/pbserver");
	Deny_URL_Sequences.AddTail("/rpc");
	Deny_URL_Sequences.AddTail("/cfdocs");				//coldfusion docs
	Deny_URL_Sequences.AddTail("/cfdocs/exampleapp");	//coldfusion samples
	Deny_URL_Sequences.AddTail("/cfdocs/snippets");		//coldfusion samples
	Deny_URL_Sequences.AddTail("/cfdocs/expeval");		//coldfusion
	Deny_URL_Sequences.AddTail("/cfdocs/examples");		//coldfusion samples
	Deny_URL_Sequences.AddTail("/cfappman");			//coldfusion samples
	Deny_URL_Sequences.AddTail("/cfide/administrator");//coldfusion
	Deny_URL_Sequences.AddTail("/siteserver");
	Deny_URL_Sequences.AddTail("/advworks");
	Deny_URL_Sequences.AddTail("/ssi/envout.bat");
	Deny_URL_Sequences.AddTail("/cgi-bin");		//CGI
	Deny_URL_Sequences.AddTail("/cgi-local");	//CGI
	Deny_URL_Sequences.AddTail("/cgi-win");		//CGI
	Deny_URL_Sequences.AddTail("/htbin");		//CGI
	Deny_URL_Sequences.AddTail("/cgibin");		//CGI
	Deny_URL_Sequences.AddTail("/cgis");		//CGI
	Deny_URL_Sequences.AddTail("/cgi");			//CGI
	Deny_URL_Sequences.AddTail("/test-cgi");	//CGI
	Deny_URL_Sequences.AddTail("/ows-bin/");
	Deny_URL_Sequences.AddTail("/bin/");
	Deny_URL_Sequences.AddTail("/sbin/");		
	Deny_URL_Sequences.AddTail("/etc/");
	Deny_URL_Sequences.AddTail("/database/");	//database
	Deny_URL_Sequences.AddTail("/databases/");	//database
	Deny_URL_Sequences.AddTail("/dbase/");		//database
	Deny_URL_Sequences.AddTail("/db/");			//database
	Deny_URL_Sequences.AddTail("/storedb/");	//database
	Deny_URL_Sequences.AddTail("/fpdb/");		//frontpage extensions (database)
	Deny_URL_Sequences.AddTail("/log/");		//logs
	Deny_URL_Sequences.AddTail("/logs/");
	Deny_URL_Sequences.AddTail("/logfile/");
	Deny_URL_Sequences.AddTail("/logfiles/");
	Deny_URL_Sequences.AddTail("/logger/");
	Deny_URL_Sequences.AddTail("/server_stats/");
	Deny_URL_Sequences.AddTail("/trafficlog/");
	Deny_URL_Sequences.AddTail("/weblog/");
	Deny_URL_Sequences.AddTail("/weblogs/");
	Deny_URL_Sequences.AddTail("/webstats/");
	Deny_URL_Sequences.AddTail("/wstats/");
	Deny_URL_Sequences.AddTail("/wusage/");
	Deny_URL_Sequences.AddTail("/wwwlog/");
	Deny_URL_Sequences.AddTail("/wwwstats/");
	Deny_URL_Sequences.AddTail("/mall_log_files/");
	Deny_URL_Sequences.AddTail("/admin");		//admin stuff
	Deny_URL_Sequences.AddTail("/admin_/");
	Deny_URL_Sequences.AddTail("/_admin/");
	Deny_URL_Sequences.AddTail("/srchadm/");
	Deny_URL_Sequences.AddTail("/admnlogin");
	Deny_URL_Sequences.AddTail("/adminlogin");
	Deny_URL_Sequences.AddTail("/w3perl/admin");
	Deny_URL_Sequences.AddTail("/webaccess/");
	Deny_URL_Sequences.AddTail("/account/");
	Deny_URL_Sequences.AddTail("/administrator/");
	Deny_URL_Sequences.AddTail("/config/");
	Deny_URL_Sequences.AddTail("/fpadmin/");
	Deny_URL_Sequences.AddTail("/srchadm/");
	Deny_URL_Sequences.AddTail("/admin_files");
	Deny_URL_Sequences.AddTail("/passwords/");
	Deny_URL_Sequences.AddTail("/etc/passwd");
	Deny_URL_Sequences.AddTail("/exchange");	//Outlook Web Access on Default web site
	Deny_URL_Sequences.AddTail("/exchweb");		//Outlook Web Access on Default web site
	Deny_URL_Sequences.AddTail("/public");		//Outlook Web Access (Public folders)
	Deny_URL_Sequences.AddTail("/exadmin");		//Outlook Web Access Administration
	Deny_URL_Sequences.AddTail("/hit_tracker/");//counter stuff
	Deny_URL_Sequences.AddTail("/hitmatic/");
	Deny_URL_Sequences.AddTail("/counter/");
	Deny_URL_Sequences.AddTail("/c/");			//other things
	Deny_URL_Sequences.AddTail("/d/");
	Deny_URL_Sequences.AddTail("/doc-html/");
	Deny_URL_Sequences.AddTail("/ftp/");
	Deny_URL_Sequences.AddTail("/htdocs/");
	Deny_URL_Sequences.AddTail("/install/");
	Deny_URL_Sequences.AddTail("/intranet/");
	Deny_URL_Sequences.AddTail("/jdbc/");
	Deny_URL_Sequences.AddTail("/msql/");
	Deny_URL_Sequences.AddTail("/odbc/");
	Deny_URL_Sequences.AddTail("/oracle/");
	Deny_URL_Sequences.AddTail("/private/");
	Deny_URL_Sequences.AddTail("/sql/");
	Deny_URL_Sequences.AddTail("/temp/");		//temp & test stuff
	Deny_URL_Sequences.AddTail("/tmp/");
	Deny_URL_Sequences.AddTail("/test/");
	Deny_URL_Sequences.AddTail("/webdriver");
	Deny_URL_Sequences.AddTail(" /http/1.");
	Deny_URL_Sequences.AddTail("/glimpse");
	Deny_URL_Sequences.AddTail("/aglimpse");
	Deny_URL_Sequences.AddTail("/htmlscript");
	Deny_URL_Sequences.AddTail("/info2www");
	Deny_URL_Sequences.AddTail("/nph-test-cgi");
	Deny_URL_Sequences.AddTail("/nph-publish");
	Deny_URL_Sequences.AddTail("/view-source");
	Deny_URL_Sequences.AddTail("/w3-msql");
	Deny_URL_Sequences.AddTail("/www-sql");
	Deny_URL_Sequences.AddTail("/level/*/exec/");
	Deny_URL_Sequences.AddTail("/nessus_is_probing_you_");
	Deny_URL_Sequences.AddTail("/~root");
	Deny_URL_Sequences.AddTail("/handler");
	Deny_URL_Sequences.AddTail("/backup");
	Deny_URL_Sequences.AddTail("/filemail");
	Deny_URL_Sequences.AddTail("/plusmail");
	Deny_URL_Sequences.AddTail("/ultraboard");
	Deny_URL_Sequences.AddTail("/empower");
	Deny_URL_Sequences.AddTail("/pals-cgi");
	Deny_URL_Sequences.AddTail("/htgrep");
	Deny_URL_Sequences.AddTail("/.nsconfig");
	Deny_URL_Sequences.AddTail("/catinfo");
	Deny_URL_Sequences.AddTail("/sweditservlet");
	Deny_URL_Sequences.AddTail("/cybercop");
	Deny_URL_Sequences.AddTail("/webcart/");
	Deny_URL_Sequences.AddTail("/rmp_query");
	Deny_URL_Sequences.AddTail("/rpm_query");
	Deny_URL_Sequences.AddTail("/servlet/servletexec");
	Deny_URL_Sequences.AddTail("/admin-serv/config/admpw");
	Deny_URL_Sequences.AddTail("/c32web.exe/changeadminpassword");
	Deny_URL_Sequences.AddTail("/graphics/sml3com");
	Deny_URL_Sequences.AddTail("/jsp/snp/*.snp");
	Deny_URL_Sequences.AddTail("_authchangeurl");

	//URL: Only allow certain first characters of a URL (must be lowercase)
	UseAllowURL = true;
	Allow_URL_List.RemoveAll();
	Allow_URL_List.AddTail("/");		//allow all relative URLs (http 1.0)
	Allow_URL_List.AddTail("http://");	//absolute URLs (http 1.1 compliant webserver or proxy server)
	Allow_URL_List.AddTail("https://"); //https absolute URL
	//Allow_URL_List.AddTail("ftp://");		//ftp request on http port (allow proxy? network request is not supported? can be exploited? TO DO: check this)
	//Allow_URL_List.AddTail("gopher://");	//gopher request on http port (allow proxy?)
	
	RFCCompliantURL = true;
	UseURLRawScan = true;			//Use URL raw scanning
	
	//FULLPATH
	Deny_FULLPATH_ParentPath = true;
	Deny_FULLPATH_Backspace = true;			//new hack?
	Deny_FULLPATH_CarriageReturn = true;	//new hack?
	Deny_FULLPATH_NewLine = true;			//new hack?
	Deny_FULLPATH_Escaping = true;			//if true, you cannot use '%' in a filename
	Deny_FULLPATH_Characters = "*?\"<>|$^#+=";	//a windows filename cannot contain "\/:*?"<>|" and block "$^#+=" as well

	//PATH
	Deny_PATH_Dot = false; //deny dot in path

	//PATH: allowed paths (must be lowercase)
	//      NO ending '\' otherwise problems with virtual dirs (if those virtual dirs
	//      are mapped to a physical directory with no ending '\'). For best practices
	//      I suggest you add a trailing '\' for each virtual dir in IIS (this tightens
	//      security). Once you have done that, you can add a trailing '\' in here too.
	//      I wonder if you can exploit this? (TO DO)
	UseAllowPaths = true;
	Allow_PATH_List.RemoveAll();
	Allow_PATH_List.AddTail("c:\\inetpub\\wwwroot");			//default web site & wwwroot2 ...
	Allow_PATH_List.AddTail("c:\\inetpub\\scripts");			//Scripts (/scripts)
	Allow_PATH_List.AddTail("c:\\winnt\\help\\iishelp");		//IISHelp (/iishelp)
	Allow_PATH_List.AddTail("c:\\inetpub\\iissamples");			//IIS samples
	Allow_PATH_List.AddTail("c:\\winnt\\web\\printers");		//Printers (/printers)
	Allow_PATH_List.AddTail("c:\\winnt\\system32\\inetsrv\\iisadmin");//IIS Administration Website & /iisadmin (individual site administration)
	//Allow_PATH_List.AddTail("c:\\winnt\\system32\\inetsrv\\iisadmpwd");//IIS change password interface (/iisadmpwd)
	Allow_PATH_List.AddTail("c:\\winnt\\system32\\certsrv");	//Certificate Services
	Allow_PATH_List.AddTail("c:\\program files\\common files\\system\\msadc"); //MSADC (/msadc) & 
	Allow_PATH_List.AddTail("c:\\program files\\common files\\microsoft shared\\web server extensions"); //Web Server Extensions (FrontPage) (/_vti_bin)
	Allow_PATH_List.AddTail("m:\\");							//Outlook Web Access
	Allow_PATH_List.AddTail("c:\\program files\\exchange\\exchweb");//Outlook Web Access
	Allow_PATH_List.AddTail("d:\\exchange\\exchweb");			//custom exchange installation
	Allow_PATH_List.AddTail("\\\\.\\backofficestorage");		//Exchange admin ("/ExAdmin")
	Allow_PATH_List.AddTail("c:\\program files");				//allow all applications to add functionality via the web server
	Allow_PATH_List.AddTail("d:\\www");							//custom web site
	Allow_PATH_List.AddTail("d:\\web");							//custom web site
	Allow_PATH_List.AddTail("e:\\www");							//custom web site
	Allow_PATH_List.AddTail("e:\\web");							//custom web site
	Allow_PATH_List.AddTail("f:\\www");							//custom web site
	Allow_PATH_List.AddTail("f:\\web");							//custom web site

	//FILENAME
	Deny_FILENAME_Characters = "\\:/*?\"<>|$^#+="; //a windows filename cannot contain "\/:*?"<>|" and block "$^#+=" as well
	Deny_FILENAME_DefaultDocument = false;

	//FILENAME: Deny almost all Windows 2000 commands (must be lowercase)
	UseDenyFilenameCGIApplications = true;
	Deny_FILENAME_CGIApplications.RemoveAll();
	Deny_FILENAME_CGIApplications.AddTail("arp.exe");
	Deny_FILENAME_CGIApplications.AddTail("at.exe");
	Deny_FILENAME_CGIApplications.AddTail("atmadm.exe");
	Deny_FILENAME_CGIApplications.AddTail("attrib.exe");
	Deny_FILENAME_CGIApplications.AddTail("cacls.exe");
	Deny_FILENAME_CGIApplications.AddTail("chkdsk.exe");
	Deny_FILENAME_CGIApplications.AddTail("chkntfs.exe");
	Deny_FILENAME_CGIApplications.AddTail("cipher.exe");
	Deny_FILENAME_CGIApplications.AddTail("cluster.exe");
	Deny_FILENAME_CGIApplications.AddTail("cmd.exe");
	Deny_FILENAME_CGIApplications.AddTail("comp.exe");
	Deny_FILENAME_CGIApplications.AddTail("date.exe");
	Deny_FILENAME_CGIApplications.AddTail("debug.exe");
	Deny_FILENAME_CGIApplications.AddTail("diskcomp.com");
	Deny_FILENAME_CGIApplications.AddTail("diskcopy.com");
	Deny_FILENAME_CGIApplications.AddTail("diskperf.exe");
	Deny_FILENAME_CGIApplications.AddTail("doskey.exe");
	Deny_FILENAME_CGIApplications.AddTail("edlin.exe");
	Deny_FILENAME_CGIApplications.AddTail("exe2bin.exe");
	Deny_FILENAME_CGIApplications.AddTail("expand.exe");
	Deny_FILENAME_CGIApplications.AddTail("fc.exe");
	Deny_FILENAME_CGIApplications.AddTail("find.exe");
	Deny_FILENAME_CGIApplications.AddTail("findstr.exe");
	Deny_FILENAME_CGIApplications.AddTail("forcedos.exe");
	Deny_FILENAME_CGIApplications.AddTail("format.exe");
	Deny_FILENAME_CGIApplications.AddTail("ftp.exe");
	Deny_FILENAME_CGIApplications.AddTail("graphics.com");
	Deny_FILENAME_CGIApplications.AddTail("hostname.exe");
	Deny_FILENAME_CGIApplications.AddTail("ipconfig.exe");
	Deny_FILENAME_CGIApplications.AddTail("ipxroute.exe");
	Deny_FILENAME_CGIApplications.AddTail("label.exe");
	Deny_FILENAME_CGIApplications.AddTail("loadfix.exe");
	Deny_FILENAME_CGIApplications.AddTail("mem.exe");
	Deny_FILENAME_CGIApplications.AddTail("mode.com");
	Deny_FILENAME_CGIApplications.AddTail("mountvol.exe");
	Deny_FILENAME_CGIApplications.AddTail("nbtstat.exe");
	Deny_FILENAME_CGIApplications.AddTail("net.exe");
	Deny_FILENAME_CGIApplications.AddTail("netsh.exe");
	Deny_FILENAME_CGIApplications.AddTail("netstat.exe");
	Deny_FILENAME_CGIApplications.AddTail("nslookup.exe");
	Deny_FILENAME_CGIApplications.AddTail("pathping.exe");
	Deny_FILENAME_CGIApplications.AddTail("ping.exe");
	Deny_FILENAME_CGIApplications.AddTail("rcp.exe");
	Deny_FILENAME_CGIApplications.AddTail("replace.exe");
	Deny_FILENAME_CGIApplications.AddTail("rexec.exe");
	Deny_FILENAME_CGIApplications.AddTail("route.exe");
	Deny_FILENAME_CGIApplications.AddTail("rsh.exe");
	Deny_FILENAME_CGIApplications.AddTail("runas.exe");
	Deny_FILENAME_CGIApplications.AddTail("rundll.exe");	//deny running dlls
	Deny_FILENAME_CGIApplications.AddTail("rundll32.exe");	//deny running dlls
	Deny_FILENAME_CGIApplications.AddTail("runonce.exe");
	Deny_FILENAME_CGIApplications.AddTail("setver.exe");
	Deny_FILENAME_CGIApplications.AddTail("subst.exe");
	Deny_FILENAME_CGIApplications.AddTail("tcmsetup.exe");
	Deny_FILENAME_CGIApplications.AddTail("telnet.exe"); //not in win2k help -> command reference!
	Deny_FILENAME_CGIApplications.AddTail("tftp.exe"); //is also blocked by scanning for ftp.exe
	Deny_FILENAME_CGIApplications.AddTail("time.exe");
	Deny_FILENAME_CGIApplications.AddTail("winnt.exe");
	Deny_FILENAME_CGIApplications.AddTail("winnt32.exe");
	Deny_FILENAME_CGIApplications.AddTail("xcopy.exe");
	//FILENAME: Deny common applications used in hacking/scanning
	Deny_FILENAME_CGIApplications.AddTail("command.com");
	Deny_FILENAME_CGIApplications.AddTail("cmd1.exe");
	Deny_FILENAME_CGIApplications.AddTail("root.exe");
	Deny_FILENAME_CGIApplications.AddTail("shell.exe");
	Deny_FILENAME_CGIApplications.AddTail("iisreset.exe");
	Deny_FILENAME_CGIApplications.AddTail("formmail.");		//.cgi & .pl
	Deny_FILENAME_CGIApplications.AddTail("mailform.");		//.cgi & .pl
	Deny_FILENAME_CGIApplications.AddTail("nc.exe");		//netcat
	Deny_FILENAME_CGIApplications.AddTail("netcat.exe");	//netcat
	Deny_FILENAME_CGIApplications.AddTail("sumthin");		//found in some script kiddie stuff
	//Deny_FILENAME_CGIApplications.AddTail("default.ida");
	Deny_FILENAME_CGIApplications.AddTail("whisker.ida");
	Deny_FILENAME_CGIApplications.AddTail("whisker.idc");
	Deny_FILENAME_CGIApplications.AddTail("whisker.idq");
	Deny_FILENAME_CGIApplications.AddTail("whisker.htw");
	Deny_FILENAME_CGIApplications.AddTail("whisker.htr");
	Deny_FILENAME_CGIApplications.AddTail("carbo.dll");
	Deny_FILENAME_CGIApplications.AddTail("ctguestb.idc");
	Deny_FILENAME_CGIApplications.AddTail("details.idc");
	Deny_FILENAME_CGIApplications.AddTail("w3proxy.dll");	//ms proxy access
	Deny_FILENAME_CGIApplications.AddTail("sam._");
	Deny_FILENAME_CGIApplications.AddTail("sensepost.exe");
	Deny_FILENAME_CGIApplications.AddTail("achg.htr");
	Deny_FILENAME_CGIApplications.AddTail("anot.htr");
	Deny_FILENAME_CGIApplications.AddTail("adctest.asp");
	Deny_FILENAME_CGIApplications.AddTail("ism.dll");
	Deny_FILENAME_CGIApplications.AddTail("bdir.htr");
	Deny_FILENAME_CGIApplications.AddTail("codebrws.asp");
	Deny_FILENAME_CGIApplications.AddTail("form_jscript.asp");
	Deny_FILENAME_CGIApplications.AddTail("form_vbscript.asp");
	Deny_FILENAME_CGIApplications.AddTail("servervariables_jscript.asp");
	Deny_FILENAME_CGIApplications.AddTail("fpcount.exe");
	Deny_FILENAME_CGIApplications.AddTail("_vti_inf.html");	//frontpage extensions
	Deny_FILENAME_CGIApplications.AddTail("postinfo.html"); //frontpage extensions
	Deny_FILENAME_CGIApplications.AddTail("fp30reg.dll");	//frontpage extensions
	Deny_FILENAME_CGIApplications.AddTail("fp4areg.dll");	//frontpage extensions
	Deny_FILENAME_CGIApplications.AddTail("shtml.dll");		//frontpage extensions
	Deny_FILENAME_CGIApplications.AddTail("shtml.exe");		//frontpage extensions
	Deny_FILENAME_CGIApplications.AddTail("fpsrvadm.exe");	//frontpage extensions
	Deny_FILENAME_CGIApplications.AddTail("fpremadm.exe");	//frontpage extensions
	Deny_FILENAME_CGIApplications.AddTail("fpadmin.htm");	//frontpage extensions
	Deny_FILENAME_CGIApplications.AddTail("fpadmcgi.exe");	//frontpage extensions
	Deny_FILENAME_CGIApplications.AddTail("cfgwiz.exe");	//frontpage extensions
	Deny_FILENAME_CGIApplications.AddTail("authors.pwd");	//frontpage extensions
	Deny_FILENAME_CGIApplications.AddTail("author.exe");	//frontpage extensions
	Deny_FILENAME_CGIApplications.AddTail("author.dll");	//frontpage extensions
	Deny_FILENAME_CGIApplications.AddTail("administrators.pwd");//frontpage extensions
	Deny_FILENAME_CGIApplications.AddTail("access.cnf");	//frontpage extensions
	Deny_FILENAME_CGIApplications.AddTail("service.cnf");	//frontpage extensions
	Deny_FILENAME_CGIApplications.AddTail("service.pwd");	//frontpage extensions
	Deny_FILENAME_CGIApplications.AddTail("service.stp");	//frontpage extensions
	Deny_FILENAME_CGIApplications.AddTail("services.cnf");	//frontpage extensions
	Deny_FILENAME_CGIApplications.AddTail("svcacl.cnf");	//frontpage extensions
	Deny_FILENAME_CGIApplications.AddTail("users.pwd");		//frontpage extensions
	Deny_FILENAME_CGIApplications.AddTail("writeto.cnf");	//frontpage extensions
	Deny_FILENAME_CGIApplications.AddTail("dvwssr.dll");	//frontpage extensions
	Deny_FILENAME_CGIApplications.AddTail("getdrvs.exe");
	Deny_FILENAME_CGIApplications.AddTail("global.asa");
	Deny_FILENAME_CGIApplications.AddTail("$data");
	Deny_FILENAME_CGIApplications.AddTail("msadcs.dll");
	Deny_FILENAME_CGIApplications.AddTail("newdsn.exe");
	Deny_FILENAME_CGIApplications.AddTail("search97.vts");
	Deny_FILENAME_CGIApplications.AddTail("viewcode.");
	Deny_FILENAME_CGIApplications.AddTail("showcode.");
	Deny_FILENAME_CGIApplications.AddTail("site.csc");
	Deny_FILENAME_CGIApplications.AddTail("srch.htm");
	Deny_FILENAME_CGIApplications.AddTail("uploadn.asp");
	Deny_FILENAME_CGIApplications.AddTail("logonfrm.asp");	//owa dos
	Deny_FILENAME_CGIApplications.AddTail("cgimail.exe");
	Deny_FILENAME_CGIApplications.AddTail("quickstore.cfg");
	Deny_FILENAME_CGIApplications.AddTail("bigconf.cgi");
	Deny_FILENAME_CGIApplications.AddTail("storemgr.pw");
	Deny_FILENAME_CGIApplications.AddTail("test.cgi");
	Deny_FILENAME_CGIApplications.AddTail("password.php3");
	Deny_FILENAME_CGIApplications.AddTail("password.txt");
	Deny_FILENAME_CGIApplications.AddTail("passwd.txt");
	Deny_FILENAME_CGIApplications.AddTail("passwd.php");
	Deny_FILENAME_CGIApplications.AddTail("passwd.php3");
	Deny_FILENAME_CGIApplications.AddTail("submit.cgi");
	Deny_FILENAME_CGIApplications.AddTail("ss.cfg");
	Deny_FILENAME_CGIApplications.AddTail("ncl_items.html");
	Deny_FILENAME_CGIApplications.AddTail("stat_what.log");
	Deny_FILENAME_CGIApplications.AddTail("easylog.html");
	Deny_FILENAME_CGIApplications.AddTail("analyse.cgi");
	Deny_FILENAME_CGIApplications.AddTail("admin.cgi");
	Deny_FILENAME_CGIApplications.AddTail("admin.php");
	Deny_FILENAME_CGIApplications.AddTail("admin.pl");
	Deny_FILENAME_CGIApplications.AddTail("access-options.txt");
	Deny_FILENAME_CGIApplications.AddTail("access.log");
	Deny_FILENAME_CGIApplications.AddTail("access-log");
	Deny_FILENAME_CGIApplications.AddTail("awebvisit.stat");
	Deny_FILENAME_CGIApplications.AddTail("dan_o.dat");
	Deny_FILENAME_CGIApplications.AddTail("hits.txt");
	Deny_FILENAME_CGIApplications.AddTail("log.htm");
	Deny_FILENAME_CGIApplications.AddTail("log.html");
	Deny_FILENAME_CGIApplications.AddTail("logfile");
	Deny_FILENAME_CGIApplications.AddTail("logfile.htm");
	Deny_FILENAME_CGIApplications.AddTail("logfile.html");
	Deny_FILENAME_CGIApplications.AddTail("logfile.txt");
	Deny_FILENAME_CGIApplications.AddTail("logger.html");
	Deny_FILENAME_CGIApplications.AddTail("stat.htm");
	Deny_FILENAME_CGIApplications.AddTail("stats.htm");
	Deny_FILENAME_CGIApplications.AddTail("stats.html");
	Deny_FILENAME_CGIApplications.AddTail("stats.txt");
	Deny_FILENAME_CGIApplications.AddTail("webaccess.htm");
	Deny_FILENAME_CGIApplications.AddTail("whois_raw.cgi");
	Deny_FILENAME_CGIApplications.AddTail("localstart.asp");	//local start document
	Deny_FILENAME_CGIApplications.AddTail(".asp.");				//.asp. bug
	Deny_FILENAME_CGIApplications.AddTail("trace.axd");			//ASP.NET tracing
	Deny_FILENAME_CGIApplications.AddTail("webplus");
	Deny_FILENAME_CGIApplications.AddTail("websendmail");
	Deny_FILENAME_CGIApplications.AddTail("dcboard.cgi");
	Deny_FILENAME_CGIApplications.AddTail("dcforum.cgi");
	Deny_FILENAME_CGIApplications.AddTail("mmstdod.cgi");
	Deny_FILENAME_CGIApplications.AddTail("cvsweb.cgi");
	Deny_FILENAME_CGIApplications.AddTail("php.cgi");
	Deny_FILENAME_CGIApplications.AddTail("maillist.pl");
	Deny_FILENAME_CGIApplications.AddTail("perl.exe");
	Deny_FILENAME_CGIApplications.AddTail("rguest.exe");
	Deny_FILENAME_CGIApplications.AddTail("rwwwshell.pl");
	Deny_FILENAME_CGIApplications.AddTail("textcounter.pl");
	Deny_FILENAME_CGIApplications.AddTail("uploader.exe");
	Deny_FILENAME_CGIApplications.AddTail("webhits.exe");
	Deny_FILENAME_CGIApplications.AddTail("webgais");
	Deny_FILENAME_CGIApplications.AddTail("finger");
	Deny_FILENAME_CGIApplications.AddTail("perlshop.cgi");
	Deny_FILENAME_CGIApplications.AddTail("pfdisplay.cgi");
	Deny_FILENAME_CGIApplications.AddTail("args.bat");
	Deny_FILENAME_CGIApplications.AddTail("at-admin.cgi");
	Deny_FILENAME_CGIApplications.AddTail("bnbform.cgi");
	Deny_FILENAME_CGIApplications.AddTail("wais.pl");
	Deny_FILENAME_CGIApplications.AddTail("wguest.exe");
	Deny_FILENAME_CGIApplications.AddTail("classifieds.cgi");
	Deny_FILENAME_CGIApplications.AddTail("environ.pl");
	Deny_FILENAME_CGIApplications.AddTail("filemail.pl");
	Deny_FILENAME_CGIApplications.AddTail("man.sh");
	Deny_FILENAME_CGIApplications.AddTail("snork.bat");
	Deny_FILENAME_CGIApplications.AddTail("blat.exe");
	Deny_FILENAME_CGIApplications.AddTail("day5datacopier.cgi");
	Deny_FILENAME_CGIApplications.AddTail("day5datanotifier.cgi");
	Deny_FILENAME_CGIApplications.AddTail("hsx.cgi");
	Deny_FILENAME_CGIApplications.AddTail("s.cgi");
	Deny_FILENAME_CGIApplications.AddTail("yabb.cgi");
	Deny_FILENAME_CGIApplications.AddTail("post-query");
	Deny_FILENAME_CGIApplications.AddTail("visadmin.exe");
	Deny_FILENAME_CGIApplications.AddTail("dumpenv.pl");
	Deny_FILENAME_CGIApplications.AddTail("snorkerz.cmd");
	Deny_FILENAME_CGIApplications.AddTail("win-c-sample.exe");
	Deny_FILENAME_CGIApplications.AddTail("w3tvars.pm");
	Deny_FILENAME_CGIApplications.AddTail("lwgate");
	Deny_FILENAME_CGIApplications.AddTail("flexform");
	Deny_FILENAME_CGIApplications.AddTail("www-admin.pl");
	Deny_FILENAME_CGIApplications.AddTail("sendform.cgi");
	Deny_FILENAME_CGIApplications.AddTail("ppdscgi.exe");
	Deny_FILENAME_CGIApplications.AddTail("upload.pl");
	Deny_FILENAME_CGIApplications.AddTail("anyform2");
	Deny_FILENAME_CGIApplications.AddTail("machineinfo");
	Deny_FILENAME_CGIApplications.AddTail("bb-hist.sh");
	Deny_FILENAME_CGIApplications.AddTail("pals-cgi");
	Deny_FILENAME_CGIApplications.AddTail("webspirs.cgi");
	Deny_FILENAME_CGIApplications.AddTail("tstisapi.dll");
	Deny_FILENAME_CGIApplications.AddTail("testisapi.dll");
	Deny_FILENAME_CGIApplications.AddTail("sendmessage.cgi");
	Deny_FILENAME_CGIApplications.AddTail("lastlines.cgi");
	Deny_FILENAME_CGIApplications.AddTail("zml.cgi");
	Deny_FILENAME_CGIApplications.AddTail("ads.cgi");
	Deny_FILENAME_CGIApplications.AddTail("%00.jsp");
	Deny_FILENAME_CGIApplications.AddTail("postinfo.asp");
	Deny_FILENAME_CGIApplications.AddTail("repost.asp");
	Deny_FILENAME_CGIApplications.AddTail("queryhit.htm");
	Deny_FILENAME_CGIApplications.AddTail("counter.exe");
	Deny_FILENAME_CGIApplications.AddTail("cached_feed.cgi");
	Deny_FILENAME_CGIApplications.AddTail("shopping_cart.mdb");
	Deny_FILENAME_CGIApplications.AddTail("password.mdb");
	Deny_FILENAME_CGIApplications.AddTail("check.txt");
	Deny_FILENAME_CGIApplications.AddTail("checks.txt");
	Deny_FILENAME_CGIApplications.AddTail("mylog.phtml");
	Deny_FILENAME_CGIApplications.AddTail("mlog.phtml");
	Deny_FILENAME_CGIApplications.AddTail("convert.bas");
	Deny_FILENAME_CGIApplications.AddTail("cpshost.dll");
	Deny_FILENAME_CGIApplications.AddTail("smssend.php");
	Deny_FILENAME_CGIApplications.AddTail("txt2html.cgi");
	Deny_FILENAME_CGIApplications.AddTail("console.exe");
	Deny_FILENAME_CGIApplications.AddTail("sojourn.cgi");
	Deny_FILENAME_CGIApplications.AddTail("ping");
	Deny_FILENAME_CGIApplications.AddTail("ftp.pl");
	Deny_FILENAME_CGIApplications.AddTail("poll_it_ssi_v2.0.cgi");
	Deny_FILENAME_CGIApplications.AddTail("source.asp");
	Deny_FILENAME_CGIApplications.AddTail("guestbook.pl");
	Deny_FILENAME_CGIApplications.AddTail("import.txt");
	Deny_FILENAME_CGIApplications.AddTail("count.cgi");
	Deny_FILENAME_CGIApplications.AddTail("catalog.nsf");
	Deny_FILENAME_CGIApplications.AddTail("domcfg.nsf");
	Deny_FILENAME_CGIApplications.AddTail("domlog.nsf");
	Deny_FILENAME_CGIApplications.AddTail("log.nsf");
	Deny_FILENAME_CGIApplications.AddTail("names.nsf");
	Deny_FILENAME_CGIApplications.AddTail("windmail.exe");
	Deny_FILENAME_CGIApplications.AddTail("quikstore.cfg");
	Deny_FILENAME_CGIApplications.AddTail("order.log");
	Deny_FILENAME_CGIApplications.AddTail("webdist.cgi");
	Deny_FILENAME_CGIApplications.AddTail("ws_ftp.ini");
	Deny_FILENAME_CGIApplications.AddTail("jdkrqnotify.exe");
	Deny_FILENAME_CGIApplications.AddTail("infosrch.cgi");
	Deny_FILENAME_CGIApplications.AddTail("code.php3");
	Deny_FILENAME_CGIApplications.AddTail("search.vts");
	Deny_FILENAME_CGIApplications.AddTail("ax-admin.cgi");
	Deny_FILENAME_CGIApplications.AddTail("axs.cgi");
	Deny_FILENAME_CGIApplications.AddTail("cachemgr.cgi");
	Deny_FILENAME_CGIApplications.AddTail("dfire.cgi");
	Deny_FILENAME_CGIApplications.AddTail("web-map.cgi");
	Deny_FILENAME_CGIApplications.AddTail("responder.cgi");
	Deny_FILENAME_CGIApplications.AddTail("read.php3");
	Deny_FILENAME_CGIApplications.AddTail("violation.php3");
	Deny_FILENAME_CGIApplications.AddTail("get32.exe");
	Deny_FILENAME_CGIApplications.AddTail("cgitest.exe");
	Deny_FILENAME_CGIApplications.AddTail("ftpsavecsp.dll");
	Deny_FILENAME_CGIApplications.AddTail("ftpsavecvp.dll");
	Deny_FILENAME_CGIApplications.AddTail("ftpsave.dll");
	Deny_FILENAME_CGIApplications.AddTail("contextadmin.html");
	Deny_FILENAME_CGIApplications.AddTail("architext_query.pl");
	Deny_FILENAME_CGIApplications.AddTail("wwwboard.pl");
	Deny_FILENAME_CGIApplications.AddTail("db_mysql.inc");
	Deny_FILENAME_CGIApplications.AddTail("cs.exe");
	Deny_FILENAME_CGIApplications.AddTail("bizdb1-search.cgi");
	Deny_FILENAME_CGIApplications.AddTail("bb-hostsvc.sh");
	Deny_FILENAME_CGIApplications.AddTail("pscoerrpage.htm");
	Deny_FILENAME_CGIApplications.AddTail("gwweb.exe");
	Deny_FILENAME_CGIApplications.AddTail("openview5.exe");

	//coldfusion
	Deny_FILENAME_CGIApplications.AddTail("cfcache.map");		//coldfusion exploit
	Deny_FILENAME_CGIApplications.AddTail("exprcalc.cfm");		//coldfusion exploit
	Deny_FILENAME_CGIApplications.AddTail("beaninfo.cfm");		//coldfusion exploit
	Deny_FILENAME_CGIApplications.AddTail("application.cfm");	//coldfusion exploit
	Deny_FILENAME_CGIApplications.AddTail("getfile.cfm");		//coldfusion exploit
	Deny_FILENAME_CGIApplications.AddTail("addcontent.cfm");	//coldfusion exploit
	Deny_FILENAME_CGIApplications.AddTail("fileexists.cfm");	//coldfusion exploit
	Deny_FILENAME_CGIApplications.AddTail("evaluate.cfm");		//coldfusion exploit
	Deny_FILENAME_CGIApplications.AddTail("displayopenedfile.cfm");//coldfusion exploit
	Deny_FILENAME_CGIApplications.AddTail("mainframeset.cfm");	//coldfusion exploit
	Deny_FILENAME_CGIApplications.AddTail("cfmlsyntaxcheck.cfm");//coldfusion exploit
	Deny_FILENAME_CGIApplications.AddTail("onrequestend.cfm");	//coldfusion exploit
	Deny_FILENAME_CGIApplications.AddTail("startstop.html");	//coldfusion exploit
	Deny_FILENAME_CGIApplications.AddTail("gettempdirectory.cfm");//coldfusion exploit
	//FILENAME: Deny DOS devices
	Deny_FILENAME_CGIApplications.AddTail("nul");
	Deny_FILENAME_CGIApplications.AddTail("lpt1");
	Deny_FILENAME_CGIApplications.AddTail("lpt2");
	Deny_FILENAME_CGIApplications.AddTail("lpt3");
	Deny_FILENAME_CGIApplications.AddTail("lpt4");
	Deny_FILENAME_CGIApplications.AddTail("aux");
	Deny_FILENAME_CGIApplications.AddTail("prn");
	Deny_FILENAME_CGIApplications.AddTail("com1");
	Deny_FILENAME_CGIApplications.AddTail("com2");
	Deny_FILENAME_CGIApplications.AddTail("com3");
	Deny_FILENAME_CGIApplications.AddTail("com4");
	//FILENAME: Deny certain *nix commands
	/*
	//Deny_FILENAME_CGIApplications.AddTail("ps");
	Deny_FILENAME_CGIApplications.AddTail("wget");
	Deny_FILENAME_CGIApplications.AddTail("uname");
	//Deny_FILENAME_CGIApplications.AddTail("id");
	Deny_FILENAME_CGIApplications.AddTail("echo");
	Deny_FILENAME_CGIApplications.AddTail("kill");
	Deny_FILENAME_CGIApplications.AddTail("chmod");
	Deny_FILENAME_CGIApplications.AddTail("chgrp");
	Deny_FILENAME_CGIApplications.AddTail("chsh");
	Deny_FILENAME_CGIApplications.AddTail("gcc");
	//Deny_FILENAME_CGIApplications.AddTail("cc");
	//Deny_FILENAME_CGIApplications.AddTail("cpp");
	Deny_FILENAME_CGIApplications.AddTail("g++");
	Deny_FILENAME_CGIApplications.AddTail("python");
	Deny_FILENAME_CGIApplications.AddTail("tclsh");
	Deny_FILENAME_CGIApplications.AddTail("nasm");
	Deny_FILENAME_CGIApplications.AddTail("perl");
	Deny_FILENAME_CGIApplications.AddTail("traceroute");
	Deny_FILENAME_CGIApplications.AddTail("nmap");
	Deny_FILENAME_CGIApplications.AddTail("lsof");
	//Deny_FILENAME_CGIApplications.AddTail("mail");
	Deny_FILENAME_CGIApplications.AddTail("inetd.conf");
	Deny_FILENAME_CGIApplications.AddTail("motd");
	Deny_FILENAME_CGIApplications.AddTail("shadow");
	Deny_FILENAME_CGIApplications.AddTail("httpd.conf"); //*/

	//EXTENSION: Allowed extensions (Common MIME types)
	UseAllowExtensions = false;
	Allow_EXTENSION_List.RemoveAll();
	Allow_EXTENSION_List.AddTail("");			//always allow default document & no extension!!!
	Allow_EXTENSION_List.AddTail(".htm");		//text/html
	Allow_EXTENSION_List.AddTail(".html");		//text/html
	Allow_EXTENSION_List.AddTail(".mdl");		//text/html
	Allow_EXTENSION_List.AddTail(".htt");		//text/webviewhtml
	Allow_EXTENSION_List.AddTail(".htc");		//text/x-component
	Allow_EXTENSION_List.AddTail(".xml");		//text/xml
	Allow_EXTENSION_List.AddTail(".wml");		//wml
	Allow_EXTENSION_List.AddTail(".dtd");		//text/html
	Allow_EXTENSION_List.AddTail(".css");		//text/css (style sheet)
	Allow_EXTENSION_List.AddTail(".uls");		//text/iuls (netmeeting link)
	Allow_EXTENSION_List.AddTail(".wsc");		//text/scriptlet
	Allow_EXTENSION_List.AddTail(".vcf");		//text/x-vcard
	Allow_EXTENSION_List.AddTail(".sgm");		//sgml
	Allow_EXTENSION_List.AddTail(".sgml");		//sgml
	Allow_EXTENSION_List.AddTail(".vrml");		//vrml
	Allow_EXTENSION_List.AddTail(".wrl");		//vrml
	Allow_EXTENSION_List.AddTail(".cur");
	Allow_EXTENSION_List.AddTail(".ani");
	Allow_EXTENSION_List.AddTail(".js");		//javascript download
	Allow_EXTENSION_List.AddTail(".class");		//java applet
	Allow_EXTENSION_List.AddTail(".sfw");		//shockwave flash
	Allow_EXTENSION_List.AddTail(".cfm");		//ColdFusion
	Allow_EXTENSION_List.AddTail(".cfml");		//ColdFusion
	Allow_EXTENSION_List.AddTail(".txt");		//text/plain
	Allow_EXTENSION_List.AddTail(".asc");		//ascii file
	Allow_EXTENSION_List.AddTail(".doc");		//doc
	Allow_EXTENSION_List.AddTail(".ai");		//postscript
	Allow_EXTENSION_List.AddTail(".eps");		//postscript
	Allow_EXTENSION_List.AddTail(".ps");		//postscript
	Allow_EXTENSION_List.AddTail(".vsd");		//visio
	Allow_EXTENSION_List.AddTail(".mpp");		//ms project
	Allow_EXTENSION_List.AddTail(".pdf");		//pdf
	Allow_EXTENSION_List.AddTail(".wk4");		
	Allow_EXTENSION_List.AddTail(".rtf");		//rich text format
	Allow_EXTENSION_List.AddTail(".wmf");		//windows media file
	Allow_EXTENSION_List.AddTail(".mcw");
	Allow_EXTENSION_List.AddTail(".wps");
	Allow_EXTENSION_List.AddTail(".wpg");
	Allow_EXTENSION_List.AddTail(".xls");		//excel
	Allow_EXTENSION_List.AddTail(".csv");		//comma separated
	Allow_EXTENSION_List.AddTail(".xlw");		//excel
	Allow_EXTENSION_List.AddTail(".ppt");		//powerpoint
	Allow_EXTENSION_List.AddTail(".pot");		//powerpoint template
	Allow_EXTENSION_List.AddTail(".png");		//image
	Allow_EXTENSION_List.AddTail(".jpe");		//image
	Allow_EXTENSION_List.AddTail(".jpg");		//image
	Allow_EXTENSION_List.AddTail(".jpeg");		//image
	Allow_EXTENSION_List.AddTail(".gif");		//image
	Allow_EXTENSION_List.AddTail(".tif");		//image
	Allow_EXTENSION_List.AddTail(".tiff");		//image
	Allow_EXTENSION_List.AddTail(".bmp");		//image
	Allow_EXTENSION_List.AddTail(".xbm");		//image
	Allow_EXTENSION_List.AddTail(".ico");		//image
	Allow_EXTENSION_List.AddTail(".pcx");		//image
	Allow_EXTENSION_List.AddTail(".ief");		//image
	Allow_EXTENSION_List.AddTail(".rgb");		//image
	Allow_EXTENSION_List.AddTail(".ppm");		//image
	Allow_EXTENSION_List.AddTail(".pbm");		//image
	Allow_EXTENSION_List.AddTail(".pnm");		//image
	Allow_EXTENSION_List.AddTail(".mpg");		//video
	Allow_EXTENSION_List.AddTail(".mpeg");		//video
	Allow_EXTENSION_List.AddTail(".mpe");		//video
	Allow_EXTENSION_List.AddTail(".mp2");		//video
	Allow_EXTENSION_List.AddTail(".avi");		//video
	Allow_EXTENSION_List.AddTail(".mov");		//video
	Allow_EXTENSION_List.AddTail(".qt");		//video
	Allow_EXTENSION_List.AddTail(".asf");		//video
	Allow_EXTENSION_List.AddTail(".ivf");		//video
	Allow_EXTENSION_List.AddTail(".lsx");		//video
	Allow_EXTENSION_List.AddTail(".wm");		//video/x-ms-wm
	Allow_EXTENSION_List.AddTail(".wmv");		//video/x-ms-wmv
	Allow_EXTENSION_List.AddTail(".wmx");		//video/x-ms-wmx
	Allow_EXTENSION_List.AddTail(".wvx");		//video/x-ms-wvx
	Allow_EXTENSION_List.AddTail(".asd");		//asf description
	Allow_EXTENSION_List.AddTail(".asx");		//video
	Allow_EXTENSION_List.AddTail(".divx");		//video
	Allow_EXTENSION_List.AddTail(".wma");		//audio/x-ms-wma
	Allow_EXTENSION_List.AddTail(".mp3");		//audio
	Allow_EXTENSION_List.AddTail(".m3u");		//audio
	Allow_EXTENSION_List.AddTail(".aif");		//audio
	Allow_EXTENSION_List.AddTail(".aiff");		//audio
	Allow_EXTENSION_List.AddTail(".aifc");		//audio
	Allow_EXTENSION_List.AddTail(".au");		//audio
	Allow_EXTENSION_List.AddTail(".snd");		//audio
	Allow_EXTENSION_List.AddTail(".ra");		//realaudio
	Allow_EXTENSION_List.AddTail(".ram");		//realaudio
	Allow_EXTENSION_List.AddTail(".wav");		//audio
	Allow_EXTENSION_List.AddTail(".mod");		//audio
	Allow_EXTENSION_List.AddTail(".mid");		//audio
	Allow_EXTENSION_List.AddTail(".midi");		//audio
	Allow_EXTENSION_List.AddTail(".cdf");		//channel definition format (w3c)
	Allow_EXTENSION_List.AddTail(".pac");		//proxy auto configuration
	Allow_EXTENSION_List.AddTail(".zip");		//compress
	Allow_EXTENSION_List.AddTail(".rar");		//compress
	Allow_EXTENSION_List.AddTail(".tar");		//compress
	Allow_EXTENSION_List.AddTail(".gtar");		//compress
	Allow_EXTENSION_List.AddTail(".arj");		//compress
	Allow_EXTENSION_List.AddTail(".jar");		//compress
	Allow_EXTENSION_List.AddTail(".gz");		//compress
	Allow_EXTENSION_List.AddTail(".z");			//compress
	Allow_EXTENSION_List.AddTail(".tgz");		//compress
	Allow_EXTENSION_List.AddTail(".cab");		//potentially dangerous?
	Allow_EXTENSION_List.AddTail(".exe");		//download
	Allow_EXTENSION_List.AddTail(".hqx");		//mac binhex
	Allow_EXTENSION_List.AddTail(".msi");/*
	Allow_EXTENSION_List.AddTail(".idq");
	Allow_EXTENSION_List.AddTail(".htw");
	Allow_EXTENSION_List.AddTail(".ida");
	Allow_EXTENSION_List.AddTail(".idc");
	Allow_EXTENSION_List.AddTail(".shtm");
	Allow_EXTENSION_List.AddTail(".shtml");
	Allow_EXTENSION_List.AddTail(".stm");
	Allow_EXTENSION_List.AddTail(".htr");*/
	Allow_EXTENSION_List.AddTail(".jsp");	//java server pages
	Allow_EXTENSION_List.AddTail(".asp");	//asp
	//Allow_EXTENSION_List.AddTail(".asa");	//asp global.asa
	Allow_EXTENSION_List.AddTail(".aspx");	//asp.NET
	Allow_EXTENSION_List.AddTail(".ashx");	//asp.net web handler
	Allow_EXTENSION_List.AddTail(".asmx");	//web service
	Allow_EXTENSION_List.AddTail(".cer");	//certificates
	Allow_EXTENSION_List.AddTail(".p7b");	//Certificate Services
	Allow_EXTENSION_List.AddTail(".crl");	//certificate revocation list
	//Allow_EXTENSION_List.AddTail(".cdx");
	//Allow_EXTENSION_List.AddTail(".printer");

	//EXTENSION: Denied extensions
	UseDenyExtensions = true;
	Deny_EXTENSION_List.RemoveAll();/*
	//EXTENSION: Deny ASP requests
	Deny_EXTENSION_List.AddTail(".asp");
	Deny_EXTENSION_List.AddTail(".aspx");
	Deny_EXTENSION_List.AddTail(".ashx");
	Deny_EXTENSION_List.AddTail(".cer");
	Deny_EXTENSION_List.AddTail(".cdx");
	Deny_EXTENSION_List.AddTail(".crl");*/
	Deny_EXTENSION_List.AddTail(".asa");//asp global file, should not be downloaded
	//Deny ASP.NET critical files
	Deny_EXTENSION_List.AddTail(".ascx");		//forbidden by asp.net
	Deny_EXTENSION_List.AddTail(".axd");		//ASP.NET tracing (should be disabled!!)
	Deny_EXTENSION_List.AddTail(".config");		//forbidden by asp.net (config)
	//Deny_EXTENSION_List.AddTail(".cs");		//forbidden by asp.net (c#) (also blocks ".css" style sheet!)
	Deny_EXTENSION_List.AddTail(".csproj");		//forbidden by asp.net (c# project)
	Deny_EXTENSION_List.AddTail(".licx");		//forbidden by asp.net (licenses)
	Deny_EXTENSION_List.AddTail(".rem");		//remoting ASP.NET
	Deny_EXTENSION_List.AddTail(".resources");	//forbidden by asp.net (resources)
	Deny_EXTENSION_List.AddTail(".resx");		//forbidden by asp.net (resources - xml)
	Deny_EXTENSION_List.AddTail(".soap");		//remoting & handling soap requests
	Deny_EXTENSION_List.AddTail(".vb");			//forbidden by asp.net (vb code)
	Deny_EXTENSION_List.AddTail(".vbproj");		//forbidden by asp.net (vb project)
	Deny_EXTENSION_List.AddTail(".vsdisco");	//dynamic discovery: security issue!!!! http://msdn.microsoft.com/msdnmag/issues/02/08/XMLFiles/default.aspx
	Deny_EXTENSION_List.AddTail(".webinfo");	//forbidden by asp.net
	//EXTENSION: Deny executables that could run on the server
	//Deny_EXTENSION_List.AddTail(".exe");		//this file should be allowed (download)
	//Deny_EXTENSION_List.AddTail(".dll");		//deny isapi extension
	//Deny_EXTENSION_List.AddTail(".ocx");		//(TO DO: check this)
	//Deny_EXTENSION_List.AddTail(".drv");
	//Deny_EXTENSION_List.AddTail(".sys");
	//Deny_EXTENSION_List.AddTail(".vxd");
	Deny_EXTENSION_List.AddTail(".scr");
	Deny_EXTENSION_List.AddTail(".vbs");		//(TO DO: check this)
	Deny_EXTENSION_List.AddTail(".bat");
	Deny_EXTENSION_List.AddTail(".btr");		//Frontpage dependency files
	Deny_EXTENSION_List.AddTail(".cmd");
	//Deny_EXTENSION_List.AddTail(".cgi");
	Deny_EXTENSION_List.AddTail(".com");
	Deny_EXTENSION_List.AddTail(".cpl");
	Deny_EXTENSION_List.AddTail(".pif");		//??? can be run??? need to check this out */
	//EXTENSION: Deny infrequently used scripts
	Deny_EXTENSION_List.AddTail(".htw");		//Maps to webhits.dll - Index Server
	Deny_EXTENSION_List.AddTail(".ida");		//Maps to idq.dll - Index Server
	Deny_EXTENSION_List.AddTail(".idq");		//Maps to idq.dll - Index Server
	Deny_EXTENSION_List.AddTail(".htr");		//Maps to ism.dll - Legacy administrative tool
	Deny_EXTENSION_List.AddTail(".idc");		//Maps to httpodbc.dll - Legacy database access tool
	Deny_EXTENSION_List.AddTail(".shtm");		//Maps to ssinc.dll - Server Side Includes
	Deny_EXTENSION_List.AddTail(".shtml");		//Maps to ssinc.dll - Server Side Includes
	Deny_EXTENSION_List.AddTail(".stm");		//Maps to ssinc.dll - Server Side Includes
	Deny_EXTENSION_List.AddTail(".printer");	//Maps to msw3prt.dll - Internet Printing Services
	//EXTENSION: Deny various static files
	Deny_EXTENSION_List.AddTail(".bin");		//IIS MetaBase
	Deny_EXTENSION_List.AddTail(".dmp");		//dumps & sql server logs
	Deny_EXTENSION_List.AddTail(".dns");		//dns files
	Deny_EXTENSION_List.AddTail(".evt");		//Event viewer log files
	Deny_EXTENSION_List.AddTail(".ini");
	Deny_EXTENSION_List.AddTail(".mdb");
	Deny_EXTENSION_List.AddTail(".mde");
	Deny_EXTENSION_List.AddTail(".ldb");		//access locking file
	Deny_EXTENSION_List.AddTail(".sav");		//backup registry files
	Deny_EXTENSION_List.AddTail(".adp");
	Deny_EXTENSION_List.AddTail(".dbf");
	Deny_EXTENSION_List.AddTail(".dbx");
	Deny_EXTENSION_List.AddTail(".cfg");
	Deny_EXTENSION_List.AddTail(".cnf");
	Deny_EXTENSION_List.AddTail(".conf");
	//Deny_EXTENSION_List.AddTail(".config");	//already denied for ASP.NET
	Deny_EXTENSION_List.AddTail(".ids");		//snort log
	Deny_EXTENSION_List.AddTail(".rules");		//snort rules
	Deny_EXTENSION_List.AddTail(".log");
	Deny_EXTENSION_List.AddTail(".pol");
	Deny_EXTENSION_List.AddTail(".dom");		//policy template
	Deny_EXTENSION_List.AddTail(".sec");
	Deny_EXTENSION_List.AddTail(".bak");
	Deny_EXTENSION_List.AddTail(".tmp");
	Deny_EXTENSION_List.AddTail(".acl");
	Deny_EXTENSION_List.AddTail(".sch");
	Deny_EXTENSION_List.AddTail(".dat");
	Deny_EXTENSION_List.AddTail(".mmc");
	Deny_EXTENSION_List.AddTail(".msc");
	Deny_EXTENSION_List.AddTail(".sql");
	Deny_EXTENSION_List.AddTail(".tql");
	Deny_EXTENSION_List.AddTail(".cns");		//terminal client settings
	Deny_EXTENSION_List.AddTail(".inc");		//ASP include file
	Deny_EXTENSION_List.AddTail(".sam");		//password file
	Deny_EXTENSION_List.AddTail(".htgroup");	//found in snort rules (*nix only)
	Deny_EXTENSION_List.AddTail(".htpasswd");
	Deny_EXTENSION_List.AddTail(".htaccess");
	Deny_EXTENSION_List.AddTail(".wwwacl");
	Deny_EXTENSION_List.AddTail(".www_acl");
    Deny_EXTENSION_List.AddTail(".ewl");

	//HEADERS
	RemoveServerHeader = false;
	ChangeServerHeader = true;
	ServerHeader = "WWW Server/1.1";//"Apache/0.8.4"; //in the form of <application>/<version_major>.<version_minor>
	Deny_COOKIE_SQLinjection = true;
	Deny_COOKIE_EncodingExploits = true;
	RFCCompliantHostHeader = true;
	UseDenyHeaders = true;
	Deny_HEADER_List.RemoveAll();
	Deny_HEADER_List.AddTail("Translate:");			//needed by MS-WebDAV & Outlook Web Access
	Deny_HEADER_List.AddTail("If:");				//WebDAV
	Deny_HEADER_List.AddTail("Lock-Token:");		//WebDAV
	Deny_HEADER_List.AddTail("Timeout:");			//WebDAV
	Deny_HEADER_List.AddTail("Transfer-Encoding:"); //disables message encoding (chunked encoding,...) (TO DO: needed by ASP?)
	Deny_HEADER_List.AddTail("Content-Encoding:");	//disables entity encoding
	Deny_HEADER_List.AddTail("Content-Transfer-Encoding:");	//not allowed in HTTP (rfc 2616)
	Deny_HEADER_List.AddTail("Proxy-Authenticate");
	Deny_HEADER_List.AddTail("WWW-Authenticate");
	Deny_HEADER_List.AddTail("Ms-Echo-Request:");	//found this in asp.dll, don't know what it means, but looks very disturbing, so block it! (TO DO: check this) (also found in winhttp.dll,wininet.dll,msxml3.dll)
	Deny_HEADER_List.AddTail("Ms-Echo-Reply:");		//found in asp.dll: HTTP_MS_ECHO_REPLY ??? (TO DO)
	UseAllowHeaderContentType = true;
	Allow_HEADER_ContentType.RemoveAll();
	Allow_HEADER_ContentType.AddTail("");									//always allow no content-type
	Allow_HEADER_ContentType.AddTail("application/x-www-form-urlencoded");	//simple post
	//Allow_HEADER_ContentType.AddTail("multipart/form-data");				//posts like fileuploads
	//Allow_HEADER_ContentType.AddTail("text/xml");							//allow SOAP/XMLRPC posts
	
	//VERBS: Allowed verbs list (case sensitive: see RFC 2616!)
	UseAllowVerbs = true;
	Allow_VERB_List.RemoveAll();
	//Allow_VERB_List.AddTail("OPTIONS");	//rfc & biztalk & frontpage & sharepoint
	Allow_VERB_List.AddTail("GET");			//rfc - simple request (must allow!)
	Allow_VERB_List.AddTail("HEAD");		//rfc - simple request (must allow!)
	Allow_VERB_List.AddTail("POST");		//rfc - post data
	//Allow_VERB_List.AddTail("PUT");		//rfc & biztalk & commerce
	//Allow_VERB_List.AddTail("DELETE");	//rfc & biztalk
	//Allow_VERB_List.AddTail("TRACE");		//rfc
	//Allow_VERB_List.AddTail("CONNECT");	//rfc
	//Allow_VERB_List.AddTail("PROPFIND");	//biztalk & webdav
	//Allow_VERB_List.AddTail("PROPPATCH");	//OWA & webdav
	//Allow_VERB_List.AddTail("MKCOL");		//biztalk & webdav
	//Allow_VERB_List.AddTail("POLL");		//exchange 2000
	//Allow_VERB_List.AddTail("DEBUG");		//ASP.NET debugging

	//VERBS: Denied verbs list (must be uppercase! lowercase & variants will also be blocked)
	UseDenyVerbs = false;
	Deny_VERB_List.RemoveAll();
	Deny_VERB_List.AddTail("PROPFIND");  //these verbs will block WebDAV (warning: WebDAV
	Deny_VERB_List.AddTail("PROPPATCH"); //is required for Outlook Web Access and "web folders")
	Deny_VERB_List.AddTail("MKCOL");
	Deny_VERB_List.AddTail("DELETE");
	Deny_VERB_List.AddTail("PUT");
	Deny_VERB_List.AddTail("COPY");
	Deny_VERB_List.AddTail("MOVE");
	Deny_VERB_List.AddTail("LOCK");
	Deny_VERB_List.AddTail("UNLOCK");
	Deny_VERB_List.AddTail("OPTIONS");
	Deny_VERB_List.AddTail("POLL");
	Deny_VERB_List.AddTail("SEARCH");	//used by MS & rfc proposal (didn't make it)
	Deny_VERB_List.AddTail("CONNECT");	//I added this one, the ones above are from urlscan
	Deny_VERB_List.AddTail("TRACE");	//I added this one too (rfc)
	//found these in winhttp.dll (TO DO: what are these & exploit? IIS says "not implemented")
	Deny_VERB_List.AddTail("M-POST");		//winhttp.dll SOAP, xmlrpc,...
	Deny_VERB_List.AddTail("PIN");			//winhttp.dll TO DO: what is it & exploit?
	Deny_VERB_List.AddTail("INVOKE");		//winhttp.dll used by sharepoint portal server TO DO: what is it & exploit?
	Deny_VERB_List.AddTail("CHECKOUT");		//winhttp.dll & rfc proposal (didn't make it) TO DO: what is it & exploit?
	Deny_VERB_List.AddTail("CHECKIN");		//winhttp.dll & rfc proposal (didn't make it) TO DO: what is it & exploit?
	Deny_VERB_List.AddTail("NOTIFY");		//winhttp.dll TO DO: what is it & exploit?
	Deny_VERB_List.AddTail("UNSUBSCRIBE");	//winhttp.dll TO DO: what is it & exploit?
	Deny_VERB_List.AddTail("SUBSCRIBE");	//winhttp.dll TO DO: what is it & exploit?
	Deny_VERB_List.AddTail("BDELETE");		//winhttp.dll TO DO: what is it & exploit?
	Deny_VERB_List.AddTail("BPROPPATCH");	//winhttp.dll TO DO: what is it & exploit?
	Deny_VERB_List.AddTail("BPROPFIND");	//winhttp.dll TO DO: what is it & exploit?
	Deny_VERB_List.AddTail("BCOPY");		//winhttp.dll TO DO: what is it & exploit?
	Deny_VERB_List.AddTail("BMOVE");		//winhttp.dll TO DO: what is it & exploit?
	Deny_VERB_List.AddTail("UNLINK");		//winhttp.dll TO DO: what is it & exploit?
	Deny_VERB_List.AddTail("LINK");			//winhttp.dll TO DO: what is it & exploit?
	Deny_VERB_List.AddTail("TRACK");		//found in MSDN library: returns the original request (like echo reply) TO DO: exploit IIS does not log it?
	Deny_VERB_List.AddTail("SHOWMETHOD");	//http method proposal that didn't make it into the RFC
	Deny_VERB_List.AddTail("TEXTSEARCH");	//http method proposal that didn't make it into the RFC
	Deny_VERB_List.AddTail("SPACEJUMP");	//http method proposal that didn't make it into the RFC
	Deny_VERB_List.AddTail("SUBSCRIPTIONS");//urlscan (urlscan_exchange2000.ini)
	Deny_VERB_List.AddTail("ACL");			//urlscan (urlscan_exchange2000.ini)
	Deny_VERB_List.AddTail("NOTIFY");		//urlscan (urlscan_exchange2000.ini)
	Deny_VERB_List.AddTail("DEBUG");		//ASP.NET debugging
	
	//SQL Injection (words in list should be lowercase, if 2 or more are found, request will be blocked)
	//used resources: next generation security software documents, Snort rules
	Deny_SQL_KeyWords.RemoveAll();
	Deny_SQL_KeyWords.AddTail("'");	//don't allow escaping string
	Deny_SQL_KeyWords.AddTail(";");	//don't allow escaping numeric value
	Deny_SQL_KeyWords.AddTail("select");
	Deny_SQL_KeyWords.AddTail("insert");
	Deny_SQL_KeyWords.AddTail("update");
	Deny_SQL_KeyWords.AddTail("delete");
	Deny_SQL_KeyWords.AddTail("drop");
	Deny_SQL_KeyWords.AddTail("alter");
	Deny_SQL_KeyWords.AddTail("create");
	Deny_SQL_KeyWords.AddTail("inner join");
	Deny_SQL_KeyWords.AddTail("from");
	Deny_SQL_KeyWords.AddTail("where");
	Deny_SQL_KeyWords.AddTail("union");
	Deny_SQL_KeyWords.AddTail("group by");
	Deny_SQL_KeyWords.AddTail("having");
	Deny_SQL_KeyWords.AddTail("table");
	Deny_SQL_KeyWords.AddTail("shutdown");
	Deny_SQL_KeyWords.AddTail("kill");
	Deny_SQL_KeyWords.AddTail("declare");
	Deny_SQL_KeyWords.AddTail("openrowset");
	Deny_SQL_KeyWords.AddTail("opendatasource");
	Deny_SQL_KeyWords.AddTail("pwdencrypt");
	Deny_SQL_KeyWords.AddTail("msdasql");
	Deny_SQL_KeyWords.AddTail("sqloledb");
	Deny_SQL_KeyWords.AddTail("char(");	//"char" sometimes blocks legit requests (like in a name)
	Deny_SQL_KeyWords.AddTail("syslogins");
	Deny_SQL_KeyWords.AddTail("sysxlogins");
	Deny_SQL_KeyWords.AddTail("sysdatabases");
	Deny_SQL_KeyWords.AddTail("sysobjects");
	Deny_SQL_KeyWords.AddTail("syscomments");
	Deny_SQL_KeyWords.AddTail("raiserror");
	//Deny_SQL_KeyWords.AddTail("execute");  //detected by "exec"
	Deny_SQL_KeyWords.AddTail("exec");
	Deny_SQL_KeyWords.AddTail("xp_");
	Deny_SQL_KeyWords.AddTail("sp_");
	//these below will be blocked (because the 2 lines above "xp_" & "sp_" will also be detected)
	Deny_SQL_KeyWords.AddTail("xp_cmdshell");
	Deny_SQL_KeyWords.AddTail("xp_reg");
	//Deny_SQL_KeyWords.AddTail("xp_regread");
	//Deny_SQL_KeyWords.AddTail("xp_regwrite");
	//Deny_SQL_KeyWords.AddTail("xp_regdeletekey");
	//Deny_SQL_KeyWords.AddTail("xp_regdeletevalue");
	//Deny_SQL_KeyWords.AddTail("xp_regaddmultistring");
	//Deny_SQL_KeyWords.AddTail("xp_regenumkeys");
	//Deny_SQL_KeyWords.AddTail("xp_regenumvalues");
	//Deny_SQL_KeyWords.AddTail("xp_regremovemultistring");
	Deny_SQL_KeyWords.AddTail("xp_servicecontrol");
	Deny_SQL_KeyWords.AddTail("xp_setsqlsecurity");
	Deny_SQL_KeyWords.AddTail("xp_readerrorlog");
	Deny_SQL_KeyWords.AddTail("xp_controlqueueservice"); //buffer overflow extended procedures
	Deny_SQL_KeyWords.AddTail("xp_createprivatequeue");
	Deny_SQL_KeyWords.AddTail("xp_decodequeuecommand");
	Deny_SQL_KeyWords.AddTail("xp_deleteprivatequeue");
	Deny_SQL_KeyWords.AddTail("xp_deletequeue");
	Deny_SQL_KeyWords.AddTail("xp_displayqueuemesgs");
	Deny_SQL_KeyWords.AddTail("xp_dsinfo");
	Deny_SQL_KeyWords.AddTail("xp_mergelineages");
	Deny_SQL_KeyWords.AddTail("xp_readpkfromqueue");
	Deny_SQL_KeyWords.AddTail("xp_readpkfromvarbin");
	Deny_SQL_KeyWords.AddTail("xp_repl_encrypt");
	Deny_SQL_KeyWords.AddTail("xp_resetqueue");
	Deny_SQL_KeyWords.AddTail("xp_sqlinventory");
	Deny_SQL_KeyWords.AddTail("xp_unpackcab");
	Deny_SQL_KeyWords.AddTail("xp_sprintf");
	Deny_SQL_KeyWords.AddTail("xp_displayparamstmt");
	Deny_SQL_KeyWords.AddTail("xp_enumresult");
	Deny_SQL_KeyWords.AddTail("xp_showcolv");
	Deny_SQL_KeyWords.AddTail("xp_updatecolvbm");
	Deny_SQL_KeyWords.AddTail("xp_execresultset"); //can be used to bypass access control
	Deny_SQL_KeyWords.AddTail("xp_printstatements");
	Deny_SQL_KeyWords.AddTail("xp_peekqueue");
	Deny_SQL_KeyWords.AddTail("xp_proxiedmetadata");
	Deny_SQL_KeyWords.AddTail("xp_displayparamstmt");
	Deny_SQL_KeyWords.AddTail("xp_availablemedia");
	Deny_SQL_KeyWords.AddTail("xp_enumdsn");
	Deny_SQL_KeyWords.AddTail("xp_filelist");
	Deny_SQL_KeyWords.AddTail("sp_password");
	Deny_SQL_KeyWords.AddTail("sp_adduser");
	Deny_SQL_KeyWords.AddTail("sp_addextendedproc");
	Deny_SQL_KeyWords.AddTail("sp_dropextendedproc");
	Deny_SQL_KeyWords.AddTail("sp_add_job"); //sp_add_jobstep
	Deny_SQL_KeyWords.AddTail("sp_start_job");
	Deny_SQL_KeyWords.AddTail("sp_delete_alert");
	Deny_SQL_KeyWords.AddTail("sp_msrepl_startup");
	
	//QUERYSTRING
	Deny_QUERYSTRING_SQLinjection = true;			//Deny SQL injection in querystring
	Deny_QUERYSTRING_EncodingExploits = true;		//Deny embedded encoding exploits
	Deny_QUERYSTRING_DirectoryTraversal = true;
	Deny_QUERYSTRING_HighBitShellcode = false;		//deny high bit characters (only allow US-ASCII)
	UseQuerystringRawScan = true;
	UseDenyQuerystringSequences = true;				//Deny certain sequences
	Deny_QUERYSTRING_Sequences.RemoveAll();			//list has to be lowercase
	Deny_QUERYSTRING_Sequences.AddTail("<object");	//deny malicious html
	Deny_QUERYSTRING_Sequences.AddTail("<iframe");
	Deny_QUERYSTRING_Sequences.AddTail("<link");
	Deny_QUERYSTRING_Sequences.AddTail("<applet");
	Deny_QUERYSTRING_Sequences.AddTail("<embed");
	Deny_QUERYSTRING_Sequences.AddTail("<script");
	Deny_QUERYSTRING_Sequences.AddTail("<form");
	Deny_QUERYSTRING_Sequences.AddTail("javascript:");
	Deny_QUERYSTRING_Sequences.AddTail("administrators /add");
	Deny_QUERYSTRING_Sequences.AddTail("cirestriction=none");
	Deny_QUERYSTRING_Sequences.AddTail("cihilitetype=full");
	Deny_QUERYSTRING_Sequences.AddTail("ciwebhitsfile");
	Deny_QUERYSTRING_Sequences.AddTail("cmd.exe");
	Deny_QUERYSTRING_Sequences.AddTail("system32");
	Deny_QUERYSTRING_Sequences.AddTail("root.exe");
	Deny_QUERYSTRING_Sequences.AddTail("xp_cmdshell");
	Deny_QUERYSTRING_Sequences.AddTail("%u");
	Deny_QUERYSTRING_Sequences.AddTail("cf_setdatasourceusername()");	//coldfusion app exploit
	Deny_QUERYSTRING_Sequences.AddTail("cf_setdatasourcepassword()");	//coldfusion exploit
	Deny_QUERYSTRING_Sequences.AddTail("cf_iscoldfusiondatasource()");	//coldfusion exploit
	Deny_QUERYSTRING_Sequences.AddTail("cfusion_getodbcdsn()");			//coldfusion exploit
	Deny_QUERYSTRING_Sequences.AddTail("cfusion_dbconnections_flush()");//coldfusion exploit
	Deny_QUERYSTRING_Sequences.AddTail("cfusion_encrypt()");			//coldfusion exploit
	Deny_QUERYSTRING_Sequences.AddTail("cfusion_setodbcini()");			//coldfusion exploit
	Deny_QUERYSTRING_Sequences.AddTail("cfusion_settings_refresh()");	//coldfusion exploit
	Deny_QUERYSTRING_Sequences.AddTail("cfusion_verifymail()");			//coldfusion exploit
	Deny_QUERYSTRING_Sequences.AddTail("includedir=");
	Deny_QUERYSTRING_Sequences.AddTail("_phplib[libdir]");
	//Netscape Enterprise Server directory view exploit (only via querystring)
	Deny_QUERYSTRING_Sequences.AddTail("wp-verify-link");
	Deny_QUERYSTRING_Sequences.AddTail("wp-cs-dump");
	Deny_QUERYSTRING_Sequences.AddTail("wp-ver-info");
	Deny_QUERYSTRING_Sequences.AddTail("wp-ver-diff");
	Deny_QUERYSTRING_Sequences.AddTail("wp-start-ver");
	Deny_QUERYSTRING_Sequences.AddTail("wp-stop-ver");
	Deny_QUERYSTRING_Sequences.AddTail("wp-uncheckout");
	Deny_QUERYSTRING_Sequences.AddTail("wp-html-rend");
	Deny_QUERYSTRING_Sequences.AddTail("wp-usr-prop");

	//raw data (headers sequences & POST data)
	IsInstalledAsGlobalFilter = true;		//Raw data can only be scanned if filter is a global filter
	Deny_POST_SQLinjection = false;			//no SQL injection in POST data
	Deny_POST_EncodingExploits = false;		//embedded encoding needed by OWA
	Deny_POST_DirectoryTraversal = false;
	Deny_POST_HighBitShellcode = false;		//deny high bit characters (only allow US-ASCII)
	UseDenyPOSTSequences = true;
	Deny_POST_Sequences.RemoveAll();		//list has to be lowercase
	Deny_POST_Sequences.AddTail("<object");	//deny malicious html
	Deny_POST_Sequences.AddTail("<iframe");
	Deny_POST_Sequences.AddTail("<link");
	Deny_POST_Sequences.AddTail("<applet");
	Deny_POST_Sequences.AddTail("<embed");
	Deny_POST_Sequences.AddTail("<script");
	Deny_POST_Sequences.AddTail("<form");
	Deny_POST_Sequences.AddTail("javascript:");
	Deny_POST_Sequences.AddTail("cmd.exe");
	Deny_POST_Sequences.AddTail("system32");
	Deny_POST_Sequences.AddTail("root.exe");
	Deny_POST_Sequences.AddTail("xp_cmdshell");
	Deny_POST_Sequences.AddTail("%u");					//unicode encoding
	Deny_POST_Sequences.AddTail("content-encoding:");	//disables entity encoding (bodyparts)
	Deny_POST_Sequences.AddTail("content-transfer-encoding:"); //not allowed in HTTP (rfc 2616)
	Deny_POST_Sequences.AddTail("cf_setdatasourceusername()");	//coldfusion app exploit
	Deny_POST_Sequences.AddTail("cf_setdatasourcepassword()");	//coldfusion exploit
	Deny_POST_Sequences.AddTail("cf_iscoldfusiondatasource()");	//coldfusion exploit
	Deny_POST_Sequences.AddTail("cfusion_getodbcdsn()");			//coldfusion exploit
	Deny_POST_Sequences.AddTail("cfusion_dbconnections_flush()");//coldfusion exploit
	Deny_POST_Sequences.AddTail("cfusion_encrypt()");			//coldfusion exploit
	Deny_POST_Sequences.AddTail("cfusion_setodbcini()");			//coldfusion exploit
	Deny_POST_Sequences.AddTail("cfusion_settings_refresh()");	//coldfusion exploit
	Deny_POST_Sequences.AddTail("cfusion_verifymail()");			//coldfusion exploit
	Deny_POST_Sequences.AddTail("file=http\\://");			//PHP-Nuke remote file attempt
	Deny_POST_Sequences.AddTail("includedir=");
	Deny_POST_Sequences.AddTail("_phplib[libdir]");

	Deny_HEADER_SQLinjection = false;
	Deny_HEADER_EncodingExploits = false;				//needed for 'Referer:' (like with Google cache)
	Deny_HEADER_DirectoryTraversal = true;
	Deny_HEADER_HighBitShellcode = false;				//deny high bit characters (only allow US-ASCII)
	UseDenyHeaderSequences = true;
	Deny_HEADER_Sequences.RemoveAll();
	Deny_HEADER_Sequences.AddTail("cmd.exe");
	Deny_HEADER_Sequences.AddTail("root.exe");
	Deny_HEADER_Sequences.AddTail("system32");
	Deny_HEADER_Sequences.AddTail("xp_cmdshell");
	Deny_HEADER_Sequences.AddTail("cirestriction=none");
	Deny_HEADER_Sequences.AddTail("cihilitetype=full");
	Deny_HEADER_Sequences.AddTail("ciwebhitsfile");
	Deny_HEADER_Sequences.AddTail("%u");				//unicode encoding
	Deny_HEADER_Sequences.AddTail("<object");			//deny malicious html (some people log e.g. 'Referer:' header and show it in html page)
	Deny_HEADER_Sequences.AddTail("<iframe");
	Deny_HEADER_Sequences.AddTail("<link");
	Deny_HEADER_Sequences.AddTail("<applet");
	Deny_HEADER_Sequences.AddTail("<embed");
	Deny_HEADER_Sequences.AddTail("<script");
	Deny_HEADER_Sequences.AddTail("<form");
	Deny_HEADER_Sequences.AddTail("javascript:");
	Deny_HEADER_Sequences.AddTail("cf_setdatasourceusername()");	//coldfusion app exploit
	Deny_HEADER_Sequences.AddTail("cf_setdatasourcepassword()");	//coldfusion exploit
	Deny_HEADER_Sequences.AddTail("cf_iscoldfusiondatasource()");	//coldfusion exploit
	Deny_HEADER_Sequences.AddTail("cfusion_getodbcdsn()");			//coldfusion exploit
	Deny_HEADER_Sequences.AddTail("cfusion_dbconnections_flush()"); //coldfusion exploit
	Deny_HEADER_Sequences.AddTail("cfusion_encrypt()");				//coldfusion exploit
	Deny_HEADER_Sequences.AddTail("cfusion_setodbcini()");			//coldfusion exploit
	Deny_HEADER_Sequences.AddTail("cfusion_settings_refresh()");	//coldfusion exploit
	Deny_HEADER_Sequences.AddTail("cfusion_verifymail()");			//coldfusion exploit
	Deny_HEADER_Sequences.AddTail("file=http\\://");			//PHP-Nuke remote file attempt
	Deny_HEADER_Sequences.AddTail("user-agent: webtrends security analyzer");	//Webtrends Security Analyzer user agent string
	Deny_HEADER_Sequences.AddTail("includedir=");
	Deny_HEADER_Sequences.AddTail("_phplib[libdir]");
#ifdef PRIVATEBUILD
#pragma message("***** PRIVATE BUILD: This is a private personal build *****")
	//TO DO: block certain bots
	Deny_HEADER_Sequences.AddTail("nameprotect.com");		//nameprotect.com http://www.nameprotect.com/botinfo.html
	//Deny_HEADER_Sequences.AddTail("turnitin.com");			//http://www.turnitin.com/robot/crawlerinfo.html
#endif

	//Allow compatible web applications
	Allow_WEBAPP_OWA = false;
	Allow_WEBAPP_Frontpage = false;
	Allow_WEBAPP_Coldfusion = false;
	Allow_WEBAPP_Flash = false;
	Allow_WEBAPP_WebDAV = false;
	Allow_WEBAPP_IISADMPWD = false;
	Allow_WEBAPP_SPPS = false;
	Allow_WEBAPP_SPTS = false;
	Allow_WEBAPP_CertificateServices = false;
	Allow_WEBAPP_BizTalk = false;
	Allow_WEBAPP_Commerce = false;
	Allow_WEBAPP_SBS = false;
	Allow_WEBAPP_ASPNET = false;
	Allow_WEBAPP_ASP = false;
}

inline void CWebKnightSettings::LoadISADefaults(){

}

bool CWebKnightSettings::WriteToFileXML(LPCTSTR fn)
{
	/* 
	 * Write all the settings to an XML file
	 * All checks on the file should have been
	 * done before this function is called
	 */
	try{
		const int sz = 255;
		char buf[sz];
		char buf2[sz];


		CFile f; int openret;
		openret = f.Open(fn,CFile::modeCreate|CFile::modeWrite|CFile::shareDenyWrite);
		if (openret > 0) {
			CWebKnightSettings d; //defaults
			CString xml("");
			xml += "<?xml version=\"1.0\" standalone='yes'?>\r\n";	//xml declaration

			//xml += "<!DOCTYPE WebKnight[\r\n";			//start dtd
			//xml += "<!ELEMENT WebKnightSettings ANY >\r\n";
			//xml += " ]>\r\n";							//end dtd

			xml += "<WebKnightSettings>\r\n";	//start entity
			xml += "<AQTRONIX_WebKnight_Configuration App='Title'/>\r\n";

			//Scanning engine
			xml += "<Scanning_Engine App='Separator'/>\r\n";
			xml += ConvertStringToXML("Allow_Late_Scanning","Option",AllowLateScanning?"1":"0",d.AllowLateScanning?"1":"0","This will run WebKnight as a low priority filter instead of high priority. Recommended is high priority (is more secure as it precedes other ISAPI filters which might have potential buffer overflows, etc.).\r\nRequires restart of IIS!");
			xml += ConvertStringToXML("Scan_Non_Secure_Port","Option",ScanNonSecurePort?"1":"0",d.ScanNonSecurePort?"1":"0","Scan unencrypted (HTTP) web traffic (default port 80, but can be anything else).\r\nRequires restart of IIS!");
			xml += ConvertStringToXML("Scan_Secure_Port","Option",ScanSecurePort?"1":"0",d.ScanSecurePort?"1":"0","Scan encrypted (HTTPS) web traffic (default port 443, but can be anything else).\r\nRequires restart of IIS!");
			xml += ConvertStringToXML("Use_Excluded_Web_Instances","Option",UseExcludedInstances?"1":"0",d.UseExcludedInstances?"1":"0","Exclude certain web instances, recommended for web instances which have their own ISAPI filter like Outlook Web Access.");
			//Excluded Instances
			xml += ConvertListToXML("Excluded_Web_Instances",ExcludedInstances,"These are the web instances excluded from scanning by WebKnight. Examples are Outlook Web Access web sites (starting at site instance 100)");

			//Response Handling
			xml += "<Incident_Response_Handling App='Separator'/>\r\n";
			xml += ConvertStringToXML("Response_Directly","Option",ResponseDirectly?"1":"0",d.ResponseDirectly?"1":"0","If an attack is detected, send an immediate response to the client with a standard message. The message sent back is the contents of the nohack.htm file (in the directory of WebKnight)");
			xml += ConvertStringToXML("Response_Redirect","Option",ResponseRedirect?"1":"0",d.ResponseRedirect?"1":"0","If an attack is detected, redirect the client to a custom url (the url specified in 'Response Redirect URL').");
			xml += ConvertStringToXML("Response_Redirect_URL","Text",ResponseRedirectURL,d.ResponseRedirectURL,"This is the URL the client is redirected to if 'Response Redirect' is chosen and 'Response Directly' is disabled. This can be an absolute URL (like \"http://www.aqtronix.com\") or a relative one (like \"/nohack.htm\").");
			xml += ConvertStringToXML("Use_Response_Status","Option",UseResponseStatus?"1":"0",d.UseResponseStatus?"1":"0","Whenever an attack is detected, use the value in 'Response Status' as the HTTP response status that will be sent back to the client. This only works if you don't redirect the client to a custom URL.");
			xml += ConvertStringToXML("Response_Status","Text",ResponseStatus,d.ResponseStatus,"This is the HTTP response status like '31337 No Hacking' or '404 Object Not Found' that is sent back to the client when an attack is detected.");
			xml += ConvertStringToXML("Response_Log_Only","Option",ResponseLogOnly?"1":"0",d.ResponseLogOnly?"1":"0","If an attack is detected, only log and do not block it and let the web server execute the request.");

			//Logging
			xml += "<Logging App='Separator'/>\r\n";
			xml += ConvertStringToXML("Enabled","Option",Logging?"1":"0",d.Logging?"1":"0","Enable or disable logging.\r\nRequires restart of IIS!");
			xml += ConvertStringToXML("Log_Directory","Text",LogDir,LogDirDefault,"The directory where the log files will be placed.\r\nRequires restart of IIS!");
			xml += ConvertStringToXML("Use_GMT","Option",LogGMT?"1":"0",d.LogGMT?"1":"0","Log dates and times in GMT/UTC.\r\nRequires restart of IIS!");
			xml += ConvertStringToXML("Per_Process_Logging","Option",LogPerProcess?"1":"0",d.LogPerProcess?"1":"0","Make a unique log file per web server process. This is for web servers that can host WebKnight in more than 1 process concurrently.\r\nRequires restart of IIS!");
			xml += ConvertStringToXML("Log_Retention","Text",itoa(LogRetention,buf,10),itoa(d.LogRetention,buf2,10),"The rotation period (in days) to keep the log files.\r\nRequires restart of IIS!");
			xml += ConvertStringToXML("Log_Allowed_Requests","Option",LogAllowedRequests?"1":"0",d.LogAllowedRequests?"1":"0","In addition to logging blocked requests you can log allowed requests as well. This has high performance impact on heavy loaded systems and is not recommended!");
			xml += ConvertStringToXML("Log_Client_IP","Option",LogClientIP?"1":"0",d.LogClientIP?"1":"0","Log the client IP address.");
			xml += ConvertStringToXML("Log_User_Name","Option",LogUserName?"1":"0",d.LogUserName?"1":"0","Log the username the client is logged on with.");
			xml += ConvertStringToXML("Log_HTTP_VIA","Option",Log_HTTP_VIA?"1":"0",d.Log_HTTP_VIA?"1":"0","Log the 'Via:' header to have a clue where the original request came from (if the client uses 1 or more proxies). Note: you will not be able to log all used proxies (certain proxies don't have or remove this header)!");
			xml += ConvertStringToXML("Log_HTTP_X_FORWARDED_FOR","Option",Log_HTTP_X_FORWARDED_FOR?"1":"0",d.Log_HTTP_X_FORWARDED_FOR?"1":"0","Log the 'X_Forwarded_For:' header. Certain proxies (like NetCache) add this header to the request which indicate the source IP address of the request.");
			xml += ConvertStringToXML("Log_User_Agent","Option",LogUserAgent?"1":"0",d.LogUserAgent?"1":"0","Log the client user agent. This can indicate what software/tool is used to perform the attack. However it is not essential information for reporting an abuse.");
			xml += ConvertStringToXML("Log_HTTP_Client_Errors","Option",Log_HTTP_ERROR_ClientError?"1":"0",d.Log_HTTP_ERROR_ClientError?"1":"0","Log HTTP client side errors like '404 Not Found'. These errors start with a '4'.");
			xml += ConvertStringToXML("Log_HTTP_Server_Errors","Option",Log_HTTP_ERROR_ServerError?"1":"0",d.Log_HTTP_ERROR_ServerError?"1":"0","Log HTTP server side errors like '501 Not Implemented'. These errors start with a '5'.");

			//Request Limits
			xml += "<Request_Limits App='Separator'/>\r\n";
			xml += ConvertStringToXML("Limit_Content_Length","Option",LimitContentLength?"1":"0",d.LimitContentLength?"1":"0","Limit the value of the Content-Length header in a request to the value specified in 'Max Content Length'. This way you can limit the number of bytes sent to the server in requests.");
			xml += ConvertStringToXML("Limit_URL","Option",LimitURL?"1":"0",d.LimitURL?"1":"0","Limit the length of the url (more precisely everything in the url before the '?') to the value specified in 'Max URL'. Certain attacks involve long urls. You should not allow urls longer than the longest path your operating system allows.");
			xml += ConvertStringToXML("Limit_Querystring","Option",LimitQueryString?"1":"0",d.LimitQueryString?"1":"0","Limit the length of the querystring (everything after the '?' in a url) to the value specified in 'Max Querystring'.");
			xml += ConvertStringToXML("Limit_HTTP_Version","Option",LimitVersion?"1":"0",d.LimitVersion?"1":"0","Every request to the web server involves specifying the HTTP version (like 'HTTP/1.1'). You can limit the length of this to the value specified in 'Max HTTP Version'.");
			xml += ConvertStringToXML("Max_Content_Length","Text",MaxContentLength,d.MaxContentLength, "The maximum value of the Content-Length header in requests sent to the web server.");
			xml += ConvertStringToXML("Max_URL","Text",ltoa(MaxURL,buf,10),ltoa(d.MaxURL,buf2,10),"The maximum length of the url.");
			xml += ConvertStringToXML("Max_Querystring","Text",ltoa(MaxQueryString,buf,10),ltoa(d.MaxQueryString,buf2,10),"The maximum length of the querystring.");
			xml += ConvertStringToXML("Max_HTTP_Version","Text",ltoa(MaxVersion,buf,10),ltoa(d.MaxVersion,buf2,10),"The maximum length of the HTTP version string.");
			xml += ConvertStringToXML("Use_Allowed_HTTP_Versions","Option",UseAllowVersions?"1":"0",d.UseAllowVersions?"1":"0","Only allow certain HTTP versions specified in 'Allowed HTTP Versions'");
			xml += ConvertListToXML("Allowed_HTTP_Versions",Allow_VERSION_List,"These are the allowed HTTP versions. An empty line means you allow the HTTP version 0.9 (no http version)");
			xml += ConvertStringToXML("Use_Max_Headers","Option",UseMaxHeaders?"1":"0",d.UseMaxHeaders?"1":"0","Limit the length of additional headers. You can specify a custom header and a maximum length for each header in 'Max Headers'.");
			xml += ConvertMapToXML("Max_Headers",MaxHeaders,"These are the headers of which the length is limited to a specific value.");

			//URL Scanning
			xml += "<URL_Scanning App='Separator'/>\r\n";
			xml += ConvertStringToXML("RFC_Compliant_Url","Option",RFCCompliantURL?"1":"0",d.RFCCompliantURL?"1":"0","Check if the URL is RFC compliant. If it is not the request will be blocked.");
			xml += ConvertStringToXML("Use_Url_Raw_Scan","Option",UseURLRawScan?"1":"0",d.UseURLRawScan?"1":"0","Besides using the default scanning, also use the raw scanning capability to scan the URL before the web server decoded the URL (with built-in decoding engine).");
			xml += ConvertStringToXML("Deny_Url_Encoding_Exploits","Option",Deny_URL_EncodingExploits?"1":"0",d.Deny_URL_EncodingExploits?"1":"0","Do not allow encoding exploits (embedded encoding) in the URL.");
			xml += ConvertStringToXML("Deny_Url_Parent_Path","Option",Deny_URL_ParentPath?"1":"0",d.Deny_URL_ParentPath?"1":"0","Deny parent path ('..') attempt in the requested url.");
			xml += ConvertStringToXML("Deny_Url_Trailing_Dot_In_Dir","Option",Deny_URL_TrailingDotInDir?"1":"0",d.Deny_URL_TrailingDotInDir?"1":"0","Deny a trailing dot in a directory name. This will block all requests with './'.");
			xml += ConvertStringToXML("Deny_Url_Backslash","Option",Deny_URL_Backslash?"1":"0",d.Deny_URL_Backslash?"1":"0","Deny backward slashes ('\\') in the url.");
			xml += ConvertStringToXML("Deny_Url_Alternate_Stream","Option",Deny_URL_AlternateStream?"1":"0",d.Deny_URL_AlternateStream?"1":"0","This will block all requests with a ':' in the url.");
			xml += ConvertStringToXML("Deny_Url_Escaping","Option",Deny_URL_Escaping?"1":"0",d.Deny_URL_Escaping?"1":"0","Do not allow '%' in the url after decoding. This will block encoding exploits (embedded encoding) in the url.");
			xml += ConvertStringToXML("Deny_Url_Running_Multiple_CGI","Option",Deny_URL_MultipleCGI?"1":"0",d.Deny_URL_MultipleCGI?"1":"0","Do not allow using the ampersand ('&') in a url. This can be used to run multiple CGI applications.");
			xml += ConvertStringToXML("Deny_Url_Characters","Text",Deny_URL_Characters,d.Deny_URL_Characters,"Additional characters to block. If a requested url contains one of these, the request will be blocked.");
			xml += ConvertStringToXML("Deny_Url_HighBitShellcode","Option",Deny_URL_HighBitShellcode?"1":"0",d.Deny_URL_HighBitShellcode?"1":"0","Do not allow high bit shellcode (ascii>127). This will restrict the web sites to US-ASCII only and block characters not in this character set. Not recommended on non-US-English web sites. This will also block Unicode/UTF-8 and MBCS in the URL.");
			xml += ConvertStringToXML("Use_Denied_Url_Sequences","Option",UseDenyURLSequences?"1":"0",d.UseDenyURLSequences?"1":"0","Block the request if the url contains one or more sequences specified in 'URL Denied Sequences'.");
			xml += ConvertStringToXML("Use_Allowed_Url_Starts","Option",UseAllowURL?"1":"0",d.UseAllowURL?"1":"0","Only allow certain characters a url may start with. These are specified in 'URL Allowed Starts'.");
			xml += ConvertListToXML("URL_Denied_Sequences",Deny_URL_Sequences,"These are the sequences not allowed in a url.");
			xml += ConvertListToXML("URL_Allowed_starts",Allow_URL_List,"These are the allowed character sequences a url may start with.");
			
			//Mapped Path
			xml += "<Mapped_Path App='Separator'/>\r\n";
			xml += ConvertStringToXML("Deny_Parent_Path","Option",Deny_FULLPATH_ParentPath?"1":"0",d.Deny_FULLPATH_ParentPath?"1":"0","Deny parent path ('..') attempt in the mapped path.");
			xml += ConvertStringToXML("Deny_Backspace","Option",Deny_FULLPATH_Backspace?"1":"0",d.Deny_FULLPATH_Backspace?"1":"0","Deny backspace characters in the mapped path.");
			xml += ConvertStringToXML("Deny_Carriage_Return","Option",Deny_FULLPATH_CarriageReturn?"1":"0",d.Deny_FULLPATH_CarriageReturn?"1":"0","Deny carriage return characters in the mapped path.");
			xml += ConvertStringToXML("Deny_New_Line","Option",Deny_FULLPATH_NewLine?"1":"0",d.Deny_FULLPATH_NewLine?"1":"0","Deny newline characters in the mapped path.");
			xml += ConvertStringToXML("Deny_Escaping","Option",Deny_FULLPATH_Escaping?"1":"0",d.Deny_FULLPATH_Escaping?"1":"0","Deny encoding exploits in the mapped path by blocking '%'.");
			xml += ConvertStringToXML("Deny_Characters","Text",Deny_FULLPATH_Characters,d.Deny_FULLPATH_Characters,"Deny the request if one or more of these characters are present in the mapped path.");
			xml += ConvertStringToXML("Deny_Dot_In_Path","Option",Deny_PATH_Dot?"1":"0",d.Deny_PATH_Dot?"1":"0","Deny a dot in the path (except for the filename).");
			xml += ConvertStringToXML("Use_Allowed_Paths","Option",UseAllowPaths?"1":"0",d.UseAllowPaths?"1":"0","Only allow mapped paths which start with a path specified in 'Allowed Paths'");
			xml += ConvertListToXML("Allowed_Paths",Allow_PATH_List,"These are the allowed paths the mapped path may start with.");

			//Requested File
			xml += "<Requested_File App='Separator'/>\r\n";
			xml += ConvertStringToXML("Deny_Filename_Characters","Text",Deny_FILENAME_Characters,d.Deny_FILENAME_Characters,"Deny the request if the filename contains one of these characters.");
			xml += ConvertStringToXML("Deny_Default_Document","Option",Deny_FILENAME_DefaultDocument?"1":"0",d.Deny_FILENAME_DefaultDocument?"1":"0","Deny default document requests. The client can only request a specific file, not a directory.");
			xml += ConvertStringToXML("Use_Denied_Files","Option",UseDenyFilenameCGIApplications?"1":"0",d.UseDenyFilenameCGIApplications?"1":"0","Deny the filenames/CGI applications specified in 'Denied Files' from being accessed or run.");
			xml += ConvertListToXML("Denied_Files",Deny_FILENAME_CGIApplications,"These are the filenames/CGI applications that are not allowed.");
			//Requested Extension
			xml += ConvertStringToXML("Use_Allowed_Extensions","Option",UseAllowExtensions?"1":"0",d.UseAllowExtensions?"1":"0","Only allow requests for files with extensions specified in 'Allowed Extensions'.");
			xml += ConvertStringToXML("Use_Denied_Extensions","Option",UseDenyExtensions?"1":"0",d.UseDenyExtensions?"1":"0","Deny requests for files with extensions specified in 'Denied Extensions'.");
			xml += ConvertListToXML("Allowed_Extensions",Allow_EXTENSION_List,"These are the allowed extensions for the requested files.");
			xml += ConvertListToXML("Denied_Extensions",Deny_EXTENSION_List,"These are the denied extensions for the requested files.");

			//HTTP Headers
			xml += "<Headers App='Separator'/>\r\n";
			xml += ConvertStringToXML("Remove_Server_Header","Option",RemoveServerHeader?"1":"0",d.RemoveServerHeader?"1":"0","Remove the 'Server:' header in every response from the web server to the client. The server header is sensitive information that can be used by hackers or worms to probe for vulnerable systems or to know what they're up against.");
			xml += ConvertStringToXML("Change_Server_Header","Option",ChangeServerHeader?"1":"0",d.ChangeServerHeader?"1":"0","Instead of removing the server header, you can change it. This way you can fool hackers and worms by specifying another commercial web server in 'Server Header'. Note: 'Remove Server Header' has priority over this setting, so don't enable 'Remove Server Header' if you want to change the server header.");
			xml += ConvertStringToXML("Server_Header","Text",ServerHeader,d.ServerHeader,"The server header sent back to the client in every response. This has to be in the form '<software>/<version>' (like 'Apache/0.8.4'.");
			xml += ConvertStringToXML("Deny_Cookie_SQL_Injection","Option",Deny_COOKIE_SQLinjection?"1":"0",d.Deny_COOKIE_SQLinjection?"1":"0","Deny SQL injection in the 'Cookie:' header. This can be useful if your website is using a database and you are using cookies for storing information related to the database.");
			xml += ConvertStringToXML("Deny_Cookie_Encoding_Exploits","Option",Deny_COOKIE_EncodingExploits?"1":"0",d.Deny_COOKIE_EncodingExploits?"1":"0","Do not allow encoding exploits (embedded encoding) in cookies (in the 'Cookie:' header).");
			xml += ConvertStringToXML("RFC_Compliant_Host_Header","Option",RFCCompliantHostHeader?"1":"0",d.RFCCompliantHostHeader?"1":"0","Block the HTTP 1.1 request if it does not include a 'Host:' header (RFC compliant).");
			xml += ConvertStringToXML("Use_Denied_Headers","Option",UseDenyHeaders?"1":"0",d.UseDenyHeaders?"1":"0","Block the request if any of the headers specified in 'Denied Headers' are present.");
			xml += ConvertListToXML("Denied_Headers",Deny_HEADER_List,"These are the headers not allowed in requests to the web server.");
			xml += ConvertStringToXML("Use_Allowed_Content_Types","Option",UseAllowHeaderContentType?"1":"0",d.UseAllowHeaderContentType?"1":"0","Enable the checking of the Content-Type header in requests and deny the request if the Content-Type is not in 'Allowed Content Types' list.");
			xml += ConvertListToXML("Allowed_Content_Types",Allow_HEADER_ContentType,"These are the allowed Content-Types of requests. If for instance you want to enable all multipart types simply add 'multipart/'. This way you effectively enable 'multipart/form-data', 'multipart/mixed',... A blank line means no Content-Type is also allowed.");
	
			//HTTP Verbs (Request Methods)
			xml += "<Methods App='Separator'/>\r\n";
			xml += ConvertStringToXML("Use_Allowed_Verbs","Option",UseAllowVerbs?"1":"0",d.UseAllowVerbs?"1":"0","Only allow the request methods (HTTP verbs) specified in 'Allowed Verbs'.");
			xml += ConvertStringToXML("Use_Denied_Verbs","Option",UseDenyVerbs?"1":"0",d.UseDenyVerbs?"1":"0","Block request if the request method (HTTP verb) is one of those specified in 'Denied Verbs'.");
			xml += ConvertListToXML("Allowed_Verbs",Allow_VERB_List,"These are request methods (HTTP verbs) that are allowed.");
			xml += ConvertListToXML("Denied_Verbs",Deny_VERB_List,"These are the denied request methods (HTTP verbs).");

			//Querystring Scanning
			xml += "<Querystring App='Separator'/>\r\n";
			xml += ConvertStringToXML("Use_Querystring_Raw_Scan","Option",UseQuerystringRawScan?"1":"0",d.UseQuerystringRawScan?"1":"0","Besides using the default scanning, also use the raw scanning capability to scan the querystring before the web server decoded the URL (with built-in decoding engine).");
			xml += ConvertStringToXML("Deny_Querystring_SQL_Injection","Option",Deny_QUERYSTRING_SQLinjection?"1":"0",d.Deny_QUERYSTRING_SQLinjection?"1":"0","Do not allow SQL injection in the querystring.");
			xml += ConvertStringToXML("Deny_Querystring_Encoding_Exploits","Option",Deny_QUERYSTRING_EncodingExploits?"1":"0",d.Deny_QUERYSTRING_EncodingExploits?"1":"0","Do not allow encoding exploits (embedded encoding) in the querystring.");
			xml += ConvertStringToXML("Deny_Querystring_Directory_Traversal","Option",Deny_QUERYSTRING_DirectoryTraversal?"1":"0",d.Deny_QUERYSTRING_DirectoryTraversal?"1":"0","Do not allow directory traversal in the querystring. This will block any '..' preceding or following a slash ('/' or '\\').");
			xml += ConvertStringToXML("Deny_Querystring_High_Bit_Shellcode","Option",Deny_QUERYSTRING_HighBitShellcode?"1":"0",d.Deny_QUERYSTRING_HighBitShellcode?"1":"0","Do not allow high bit shellcode (ascii>127). This will restrict the web sites to US-ASCII only and block characters not in this character set. Not recommended for non-US-English web sites.");
			xml += ConvertStringToXML("Use_Denied_Querystring_Sequences","Option",UseDenyQuerystringSequences?"1":"0",d.UseDenyQuerystringSequences?"1":"0","Block the request if any of the sequences specified in 'Denied Querystring Sequences' are present.");
			xml += ConvertListToXML("Denied_Querystring_Sequences",Deny_QUERYSTRING_Sequences,"These are the character sequences not allowed in the querystring.");
			
			//Global Filter Capabilities
			xml += "<Global_Filter_Capabilities App='Separator'/>\r\n";
			xml += ConvertStringToXML("Is_Installed_As_Global_Filter","Option",IsInstalledAsGlobalFilter?"1":"0",d.IsInstalledAsGlobalFilter?"1":"0","Register for the OnReadRawData event. This is required for the global filter capabilities. This event can only be called if the filter is installed as a global filter. If this is not the case then WebKnight will fail to load. For IIS 6 (and up), you need to run IIS in IIS 5.0 Isolation mode (not Worker Process mode). Requires restart of IIS!");
			xml += ConvertStringToXML("Deny_Header_SQL_Injection","Option",Deny_HEADER_SQLinjection?"1":"0",d.Deny_HEADER_SQLinjection?"1":"0","Do not allow SQL injection in the headers sent to the web server.");
			xml += ConvertStringToXML("Deny_Header_Encoding_Exploits","Option",Deny_HEADER_EncodingExploits?"1":"0",d.Deny_HEADER_EncodingExploits?"1":"0","Do not allow encoding exploits (embedded encoding) in the headers sent to the web server.");
			xml += ConvertStringToXML("Deny_Header_Directory_Traversal","Option",Deny_HEADER_DirectoryTraversal?"1":"0",d.Deny_HEADER_DirectoryTraversal?"1":"0","Do not allow directory traversal (parent path) in the headers sent to the web server. This will block any '..' preceding or following a slash ('/' or '\\').");
			xml += ConvertStringToXML("Deny_Header_High_Bit_Shellcode","Option",Deny_HEADER_HighBitShellcode?"1":"0",d.Deny_HEADER_HighBitShellcode?"1":"0","Do not allow high bit shellcode (ascii>127). This will restrict the web sites to US-ASCII only and block characters not in this character set. Not recommended for non-US-English web sites.");
			xml += ConvertStringToXML("Use_Denied_Header_Sequences","Option",UseDenyHeaderSequences?"1":"0",d.UseDenyHeaderSequences?"1":"0","Block the request if any of the character sequences specified in 'Denied Header Sequences' are present in the headers.");
			xml += ConvertStringToXML("Deny_Postdata_SQL_Injection","Option",Deny_POST_SQLinjection?"1":"0",d.Deny_POST_SQLinjection?"1":"0","Do not allow SQL injection in the data (e.g. postdata) sent to the web server.");
			xml += ConvertStringToXML("Deny_Postdata_Encoding_Exploits","Option",Deny_POST_EncodingExploits?"1":"0",d.Deny_POST_EncodingExploits?"1":"0","Do not allow encoding exploits (embedded encoding) in the data (e.g. postdata) sent to the web server.");
			xml += ConvertStringToXML("Deny_Postdata_Directory_Traversal","Option",Deny_POST_DirectoryTraversal?"1":"0",d.Deny_POST_DirectoryTraversal?"1":"0","Do not allow directory traversal (parent path) in the data (e.g. postdata) sent to the web server. This will block any '..' preceding or following a slash ('/' or '\\').");
			xml += ConvertStringToXML("Deny_Postdata_High_Bit_Shellcode","Option",Deny_POST_HighBitShellcode?"1":"0",d.Deny_POST_HighBitShellcode?"1":"0","Do not allow high bit shellcode (ascii>127). This will restrict the web sites to US-ASCII only and block characters not in this character set. Not recommended for non-US-English web sites.");
			xml += ConvertStringToXML("Use_Denied_Post_Sequences","Option",UseDenyPOSTSequences?"1":"0",d.UseDenyPOSTSequences?"1":"0","Block the request if any of the character sequences specified in 'Denied Post Sequences' are present in the data.");
			xml += ConvertListToXML("Denied_Header_Sequences",Deny_HEADER_Sequences,"These are the character sequences not allowed in the headers.");
			xml += ConvertListToXML("Denied_Post_Sequences",Deny_POST_Sequences,"These are the character sequences not allowed in data (e.g. postdata).");

			//SQL Injection Keywords
			xml += "<SQL_Injection App='Separator'/>\r\n";
			xml += ConvertListToXML("SQL_Injection_Keywords",Deny_SQL_KeyWords,"These are the SQL keywords for the SQL injection scanning. If two or more are found an alert is triggered and the request will be blocked.");

			//Web Applications
			xml += "<Web_Applications App='Separator'/>\r\n";
			xml += ConvertStringToXML("Allow_Outlook_Web_Access","Option",Allow_WEBAPP_OWA?"1":"0",d.Allow_WEBAPP_OWA?"1":"0","Allows Outlook Web Access. This changes other settings so that OWA is enabled. This reduces the security of your system and it is not recommended that you run OWA as a virtual directory ('/Exchange'). It is better to assign a web site to OWA (Use MMC of Exchange Server: add HTTP server) and exclude this web instance from scanning!");
			xml += ConvertStringToXML("Allow_Frontpage_Extensions","Option",Allow_WEBAPP_Frontpage?"1":"0",d.Allow_WEBAPP_Frontpage?"1":"0","Allows Frontpage Extensions. This changes other settings so that the firewall will not block Frontpage Extensions. Enabling this reduces the security of your system and make sure you have the latest version of Frontpage installed and keep up with security patches!");
			xml += ConvertStringToXML("Allow_Coldfusion","Option",Allow_WEBAPP_Coldfusion?"1":"0",d.Allow_WEBAPP_Coldfusion?"1":"0","Allows Coldfusion. This changes other settings so that the firewall will not block these requests. This reduces the security of your system and you should follow security practices of Coldfusion and keep up with security fixes!");
			xml += ConvertStringToXML("Allow_Flash_MX","Option",Allow_WEBAPP_Flash?"1":"0",d.Allow_WEBAPP_Flash?"1":"0","Allows Flash MX. This changes other settings so that the firewall will not block these requests. This reduces the security of your system and you should follow security practices of Flash MX and keep up with security fixes!");
			xml += ConvertStringToXML("Allow_WebDAV","Option",Allow_WEBAPP_WebDAV?"1":"0",d.Allow_WEBAPP_WebDAV?"1":"0","Allows WebDAV. WebDAV is an HTTP extension for Distributed Authoring and Versioning. This changes other settings so that the firewall will not block these requests. This reduces the security of your system!");
			xml += ConvertStringToXML("Allow_IISADMPWD","Option",Allow_WEBAPP_IISADMPWD?"1":"0",d.Allow_WEBAPP_IISADMPWD?"1":"0","Allows IISADMPWD. IISADMPWD is a virtual directory that allows users to change their domain/local password over the HTTP protocol. Outlook Web Access can use this feature. Enabling this changes other settings so that the firewall will not block these requests. This reduces the security of your system and is not recommended!");
			xml += ConvertStringToXML("Allow_SharePoint_Portal_Server","Option",Allow_WEBAPP_SPPS?"1":"0",d.Allow_WEBAPP_SPPS?"1":"0","Allows SharePoint Portal Server. Enabling this changes other settings so that the firewall will not block these requests. This reduces the security of your system and is not recommended!");
			xml += ConvertStringToXML("Allow_SharePoint_Team_Services","Option",Allow_WEBAPP_SPTS?"1":"0",d.Allow_WEBAPP_SPTS?"1":"0","Allows SharePoint Team Services. Enabling this changes other settings so that the firewall will not block these requests. This reduces the security of your system and is not recommended!");
			xml += ConvertStringToXML("Allow_Certificate_Services","Option",Allow_WEBAPP_CertificateServices?"1":"0",d.Allow_WEBAPP_CertificateServices?"1":"0","Allows Certificate Services Web Interface. Certificate Services installs a virtual directory in your default web site for managing certificates via your browser. Enabling this changes other settings so that the firewall will not block these requests. This reduces the security of your system and is not recommended!");
			xml += ConvertStringToXML("Allow_BizTalk_Server","Option",Allow_WEBAPP_BizTalk?"1":"0",d.Allow_WEBAPP_BizTalk?"1":"0","Allows BizTalk Server. Enabling this changes other settings so that the firewall will not block these requests. This reduces the security of your system and is not recommended!");
			xml += ConvertStringToXML("Allow_Commerce_Server","Option",Allow_WEBAPP_Commerce?"1":"0",d.Allow_WEBAPP_Commerce?"1":"0","Allows Commerce Server. Enabling this changes other settings so that the firewall will not block these requests. This reduces the security of your system and is not recommended!");
			xml += ConvertStringToXML("Allow_Small_Business_Server","Option",Allow_WEBAPP_SBS?"1":"0",d.Allow_WEBAPP_SBS?"1":"0","Allows Small Business Server. This is the same as enabling Outlook Web Access. Enabling this changes other settings so that the firewall will not block these requests. This reduces the security of your system and is not recommended!");
			xml += ConvertStringToXML("Allow_ASP_NET","Option",Allow_WEBAPP_ASPNET?"1":"0",d.Allow_WEBAPP_ASPNET?"1":"0","Allows all features of ASP.NET. By default ASP.NET is partially enabled. You should select this option only if you really need debugging, tracing, remoting and SOAP for your ASP.NET.");
			xml += ConvertStringToXML("Allow_ASP","Option",Allow_WEBAPP_ASP?"1":"0",d.Allow_WEBAPP_ASP?"1":"0","Allows Active Server Pages 3.0 (and previous versions). By default ASP is fully enabled, but if you changed the default settings and disabled ASP then you can use this option to re-enable ASP.");

			xml += "</WebKnightSettings>\r\n";		//end entity

			f.Write(xml,strlen(xml));
			f.Close();
			return true;
		}else{
			return false;
		}
	}catch(...){
		return false;
	}
}

bool CWebKnightSettings::WriteToFileINI(LPCTSTR fn)
{
	/* 
	 * Write all settings to a simple INI file
	 * All checks on the file should have been
	 * done before this function is called
	 */
	try{

	const int sz = 255;
	char buf[sz];

	//Scanning engine
	WritePrivateProfileString("Scanning Engine","AllowLateScanning",AllowLateScanning?"1":"0",fn);
	WritePrivateProfileString("Scanning Engine","ScanNonSecurePort",ScanNonSecurePort?"1":"0",fn);
	WritePrivateProfileString("Scanning Engine","ScanSecurePort",ScanSecurePort?"1":"0",fn);
	WritePrivateProfileString("Scanning Engine","UseExcludedWebInstances",UseExcludedInstances?"1":"0",fn);

	//Excluded Instances
	WritePrivateProfileSection("Excluded Web Instances",ConvertListToString(ExcludedInstances),fn);

	//Response Handling
	WritePrivateProfileString("Intrusion Response Handling","ResponseDirectly",ResponseDirectly?"1":"0",fn);
	WritePrivateProfileString("Intrusion Response Handling","ResponseRedirect",ResponseRedirect?"1":"0",fn);
	WritePrivateProfileString("Intrusion Response Handling","ResponseRedirectURL",ResponseRedirectURL,fn);
	WritePrivateProfileString("Intrusion Response Handling","UseResponseStatus",UseResponseStatus?"1":"0",fn);
	WritePrivateProfileString("Intrusion Response Handling","ResponseStatus",ResponseStatus,fn);
	WritePrivateProfileString("Intrusion Response Handling","ResponseLogOnly",ResponseLogOnly?"1":"0",fn);

	//Logging
	WritePrivateProfileString("Logging","Enabled",Logging?"1":"0",fn);
	WritePrivateProfileString("Logging","LogDirectory",LogDir,fn);
	WritePrivateProfileString("Logging","UseGMT",LogGMT?"1":"0",fn);
	WritePrivateProfileString("Logging","PerProcessLogging",LogPerProcess?"1":"0",fn);
	WritePrivateProfileString("Logging","LogRetention",itoa(LogRetention,buf,10),fn);
	WritePrivateProfileString("Logging","LogAllowedRequests",LogAllowedRequests?"1":"0",fn);
	WritePrivateProfileString("Logging","LogClientIP",LogClientIP?"1":"0",fn);
	WritePrivateProfileString("Logging","LogUserName",LogUserName?"1":"0",fn);
	WritePrivateProfileString("Logging","Log_HTTP_VIA",Log_HTTP_VIA?"1":"0",fn);
	WritePrivateProfileString("Logging","Log_HTTP_X_FORWARDED_FOR",Log_HTTP_X_FORWARDED_FOR?"1":"0",fn);
	WritePrivateProfileString("Logging","LogUserAgent",LogUserAgent?"1":"0",fn);
	WritePrivateProfileString("Logging","Log_HTTP_ClientErrors",Log_HTTP_ERROR_ClientError?"1":"0",fn);
	WritePrivateProfileString("Logging","Log_HTTP_ServerErrors",Log_HTTP_ERROR_ServerError?"1":"0",fn);

	//Request Limits
	WritePrivateProfileString("Request Limits","LimitContentLength",LimitContentLength?"1":"0",fn);
	WritePrivateProfileString("Request Limits","LimitURL",LimitURL?"1":"0",fn);
	WritePrivateProfileString("Request Limits","LimitQueryString",LimitQueryString?"1":"0",fn);
	WritePrivateProfileString("Request Limits","LimitHTTPVersion",LimitVersion?"1":"0",fn);
	WritePrivateProfileString("Request Limits","MaxContentLength",MaxContentLength,fn);
	WritePrivateProfileString("Request Limits","MaxURL",ltoa(MaxURL,buf,10),fn);
	WritePrivateProfileString("Request Limits","MaxQueryString",ltoa(MaxQueryString,buf,10),fn);
	WritePrivateProfileString("Request Limits","MaxHTTPVersion",ltoa(MaxVersion,buf,10),fn);
	WritePrivateProfileString("Request Limits","UseAllowedHTTPVersions",UseAllowVersions?"1":"0",fn);
	WritePrivateProfileString("Request Limits","UseMaxHeaders",UseMaxHeaders?"1":"0",fn);
	WritePrivateProfileSection("Allowed HTTP Versions",ConvertListToString(Allow_VERSION_List),fn);
	WritePrivateProfileSection("MaxHeaders",ConvertMapToString(MaxHeaders),fn);

	//URL
	WritePrivateProfileString("URL Scanning","RFCCompliantUrl",RFCCompliantURL?"1":"0",fn);
	WritePrivateProfileString("URL Scanning","UseUrlRawScan",UseURLRawScan?"1":"0",fn);
	WritePrivateProfileString("URL Scanning","DenyUrlEncodingExploits",Deny_URL_EncodingExploits?"1":"0",fn);
	WritePrivateProfileString("URL Scanning","DenyUrlParentPath",Deny_URL_ParentPath?"1":"0",fn);
	WritePrivateProfileString("URL Scanning","DenyUrlTrailingDotInDir",Deny_URL_TrailingDotInDir?"1":"0",fn);
	WritePrivateProfileString("URL Scanning","DenyUrlBackslash",Deny_URL_Backslash?"1":"0",fn);
	WritePrivateProfileString("URL Scanning","DenyUrlAlternateStream",Deny_URL_AlternateStream?"1":"0",fn);
	WritePrivateProfileString("URL Scanning","DenyUrlEscaping",Deny_URL_Escaping?"1":"0",fn);
	WritePrivateProfileString("URL Scanning","DenyUrlRunningMultipleCGI",Deny_URL_MultipleCGI?"1":"0",fn);
	WritePrivateProfileString("URL Scanning","DenyUrlCharacters",Deny_URL_Characters,fn);
	WritePrivateProfileString("URL Scanning","DenyUrlHighBitShellcode",Deny_URL_HighBitShellcode?"1":"0",fn);
	WritePrivateProfileString("URL Scanning","UseDeniedUrlSequences",UseDenyURLSequences?"1":"0",fn);
	WritePrivateProfileString("URL Scanning","UseAllowedUrlStarts",UseAllowURL?"1":"0",fn);
	WritePrivateProfileSection("URL Denied Sequences",ConvertListToString(Deny_URL_Sequences),fn);
	WritePrivateProfileSection("URL Allowed starts",ConvertListToString(Allow_URL_List),fn);

	//PATH
	WritePrivateProfileString("Mapped Path","DenyParentPath",Deny_FULLPATH_ParentPath?"1":"0",fn);
	WritePrivateProfileString("Mapped Path","DenyBackspace",Deny_FULLPATH_Backspace?"1":"0",fn);
	WritePrivateProfileString("Mapped Path","DenyCarriageReturn",Deny_FULLPATH_CarriageReturn?"1":"0",fn);
	WritePrivateProfileString("Mapped Path","DenyNewLine",Deny_FULLPATH_NewLine?"1":"0",fn);
	WritePrivateProfileString("Mapped Path","DenyEscaping",Deny_FULLPATH_Escaping?"1":"0",fn);
	WritePrivateProfileString("Mapped Path","DenyCharacters",Deny_FULLPATH_Characters,fn);
	WritePrivateProfileString("Mapped Path","DenyDotInPath",Deny_PATH_Dot?"1":"0",fn);
	WritePrivateProfileString("Mapped Path","UseAllowedPaths",UseAllowPaths?"1":"0",fn);
	WritePrivateProfileSection("Allowed Paths",ConvertListToString(Allow_PATH_List),fn);

	//FILENAME
	WritePrivateProfileString("Requested File","DenyFilenameCharacters",Deny_FILENAME_Characters,fn);
	WritePrivateProfileString("Requested File","DenyFilenameDefaultDocument",Deny_FILENAME_DefaultDocument?"1":"0",fn);
	WritePrivateProfileString("Requested File","UseDeniedFiles",UseDenyFilenameCGIApplications?"1":"0",fn);
	WritePrivateProfileSection("Denied Files",ConvertListToString(Deny_FILENAME_CGIApplications),fn);
	
	//EXTENSION
	WritePrivateProfileString("Requested Extension","UseAllowedExtensions",UseAllowExtensions?"1":"0",fn);
	WritePrivateProfileString("Requested Extension","UseDeniedExtensions",UseDenyExtensions?"1":"0",fn);
	WritePrivateProfileSection("Allowed Extensions",ConvertListToString(Allow_EXTENSION_List),fn);
	WritePrivateProfileSection("Denied Extensions",ConvertListToString(Deny_EXTENSION_List),fn);

	//HEADERS
	WritePrivateProfileString("HTTP Headers","RemoveServerHeader",RemoveServerHeader?"1":"0",fn);
	WritePrivateProfileString("HTTP Headers","ChangeServerHeader",ChangeServerHeader?"1":"0",fn);
	WritePrivateProfileString("HTTP Headers","ServerHeader",ServerHeader,fn);
	WritePrivateProfileString("HTTP Headers","DenyCookieSQLinjection",Deny_COOKIE_SQLinjection?"1":"0",fn);
	WritePrivateProfileString("HTTP Headers","DenyCookieEncodingExploits",Deny_COOKIE_EncodingExploits?"1":"0",fn);
	WritePrivateProfileString("HTTP Headers","RFCCompliantHostHeader",RFCCompliantHostHeader?"1":"0",fn);
	WritePrivateProfileString("HTTP Headers","UseDeniedHeaders",UseDenyHeaders?"1":"0",fn);
	WritePrivateProfileSection("Denied Headers",ConvertListToString(Deny_HEADER_List),fn);
	WritePrivateProfileString("HTTP Headers","UseAllowedContentTypes",UseAllowHeaderContentType?"1":"0",fn);
	WritePrivateProfileSection("Allowed Content-Types",ConvertListToString(Allow_HEADER_ContentType),fn);
	
	//VERBS
	WritePrivateProfileString("HTTP Verbs (Request Methods)","UseAllowedVerbs",UseAllowVerbs?"1":"0",fn);
	WritePrivateProfileString("HTTP Verbs (Request Methods)","UseDeniedVerbs",UseDenyVerbs?"1":"0",fn);
	WritePrivateProfileSection("Allowed Verbs",ConvertListToString(Allow_VERB_List),fn);
	WritePrivateProfileSection("Denied Verbs",ConvertListToString(Deny_VERB_List),fn);

	//SQL Injection
	//WritePrivateProfileSection("SQL Injection Keywords",ConvertListToString(Deny_SQL_KeyWords),fn);
	//TO DO: writing ';' to ini file cannot be read again
	
	//QUERYSTRING
	WritePrivateProfileString("Querystring Scanning","UseQuerystringRawScanning",UseQuerystringRawScan?"1":"0",fn);
	WritePrivateProfileString("Querystring Scanning","DenyQuerystringSQLinjection",Deny_QUERYSTRING_SQLinjection?"1":"0",fn);
	WritePrivateProfileString("Querystring Scanning","DenyQuerystringEncodingExploits",Deny_QUERYSTRING_EncodingExploits?"1":"0",fn);
	WritePrivateProfileString("Querystring Scanning","DenyQuerystringDirectoryTraversal",Deny_QUERYSTRING_DirectoryTraversal?"1":"0",fn);
	WritePrivateProfileString("Querystring Scanning","DenyQuerystringHighBitShellcode",Deny_QUERYSTRING_HighBitShellcode?"1":"0",fn);
	WritePrivateProfileString("Querystring Scanning","UseDeniedQuerystringSequences",UseDenyQuerystringSequences?"1":"0",fn);
	WritePrivateProfileSection("Denied Querystring Sequences",ConvertListToString(Deny_QUERYSTRING_Sequences),fn);
	
	//Raw data (headers & POST data)
	WritePrivateProfileString("Global Filter Capabilities","IsInstalledAsGlobalFilter",IsInstalledAsGlobalFilter?"1":"0",fn);
	WritePrivateProfileString("Global Filter Capabilities","DenyHeaderSQLinjection",Deny_HEADER_SQLinjection?"1":"0",fn);
	WritePrivateProfileString("Global Filter Capabilities","DenyHeaderEncodingExploits",Deny_HEADER_EncodingExploits?"1":"0",fn);
	WritePrivateProfileString("Global Filter Capabilities","DenyHeaderDirectoryTraversal",Deny_HEADER_DirectoryTraversal?"1":"0",fn);
	WritePrivateProfileString("Global Filter Capabilities","DenyHeaderHighBitShellcode",Deny_HEADER_HighBitShellcode?"1":"0",fn);
	WritePrivateProfileString("Global Filter Capabilities","UseDeniedHeaderSequences",UseDenyHeaderSequences?"1":"0",fn);
	WritePrivateProfileString("Global Filter Capabilities","DenyPostDataSQLinjection",Deny_POST_SQLinjection?"1":"0",fn);
	WritePrivateProfileString("Global Filter Capabilities","DenyPostDataEncodingExploits",Deny_POST_EncodingExploits?"1":"0",fn);
	WritePrivateProfileString("Global Filter Capabilities","DenyPostDataDirectoryTraversal",Deny_POST_DirectoryTraversal?"1":"0",fn);
	WritePrivateProfileString("Global Filter Capabilities","DenyPostDataHighBitShellcode",Deny_POST_HighBitShellcode?"1":"0",fn);
	WritePrivateProfileString("Global Filter Capabilities","UseDeniedPostSequences",UseDenyPOSTSequences?"1":"0",fn);
	WritePrivateProfileSection("Denied Header Sequences",ConvertListToString(Deny_HEADER_Sequences),fn);
	WritePrivateProfileSection("Denied Post Sequences",ConvertListToString(Deny_POST_Sequences),fn);

	//Web applications
	WritePrivateProfileString("Web Applications","AllowOutlookWebAccess",Allow_WEBAPP_OWA?"1":"0",fn);
	WritePrivateProfileString("Web Applications","AllowFrontpageExtensions",Allow_WEBAPP_Frontpage?"1":"0",fn);
	WritePrivateProfileString("Web Applications","AllowColdfusion",Allow_WEBAPP_Coldfusion?"1":"0",fn);
	WritePrivateProfileString("Web Applications","AllowFlashMX",Allow_WEBAPP_Flash?"1":"0",fn);
	WritePrivateProfileString("Web Applications","AllowWebDAV",Allow_WEBAPP_WebDAV?"1":"0",fn);
	WritePrivateProfileString("Web Applications","AllowIISADMPWD",Allow_WEBAPP_IISADMPWD?"1":"0",fn);
	WritePrivateProfileString("Web Applications","AllowSharePointPortalServer",Allow_WEBAPP_SPPS?"1":"0",fn);
	WritePrivateProfileString("Web Applications","AllowSharePointTeamServices",Allow_WEBAPP_SPTS?"1":"0",fn);
	WritePrivateProfileString("Web Applications","AllowCertificateServices",Allow_WEBAPP_CertificateServices?"1":"0",fn);
	WritePrivateProfileString("Web Applications","AllowBizTalkServer",Allow_WEBAPP_BizTalk?"1":"0",fn);
	WritePrivateProfileString("Web Applications","AllowCommerceServer",Allow_WEBAPP_Commerce?"1":"0",fn);
	WritePrivateProfileString("Web Applications","AllowSmallBusinessServer",Allow_WEBAPP_SBS?"1":"0",fn);
	WritePrivateProfileString("Web Applications","AllowASP.NET",Allow_WEBAPP_ASPNET?"1":"0",fn);
	WritePrivateProfileString("Web Applications","AllowASP",Allow_WEBAPP_ASP?"1":"0",fn);
	
		return true;
	}catch(...){
		return false;
	}
}

bool CWebKnightSettings::ReadFromFile(LPCTSTR fn)
{
	try{

		//if file does not exist, return false
		DWORD attr = GetFileAttributes(fn);
		if (0xFFFFFFFF == attr)//doesn't exist
			return false;

		//if file cannot be opened, return false
		CFile f;
		if(!f.Open(fn,CFile::modeRead|CFile::shareDenyWrite))
			return false;
		
		//set info for run-time update
		_tzset();
		Update_Time = CTime::GetCurrentTime();
		Update_fn = fn;
		CFileStatus fs;
		if(f.GetStatus(fs))
			Update_fn_Timestamp = fs.m_mtime;

		//close the file
		f.Close();


		//if file is a urlscan.ini file
		CFileName cfn(fn);
		if(cfn.FileNameLCase == "urlscan.ini"){		//urlscan.ini file
			return ReadFromFileURLScan(fn);
		}else if(cfn.ExtensionLCase == ".ini"){		//.ini file
			return ReadFromFileINI(fn);
		}else if(cfn.ExtensionLCase == ".xml"){		//.xml file
			return ReadFromFileXML(fn);
		}else{
			return false;							//unknown file type
		}
		
	}catch(...){
		return false;
	}
}

bool CWebKnightSettings::ReadFromFileURLScan(LPCTSTR fn)
{
	/* Read settings from a urlscan.ini file
	 * All checks on the file (exists? access allowed? ...)
	 * should have been done before this function is called
	 */
	try{
		const int sz = MAX_INI_FILE_SIZE;
		char buf[sz];

		//[options]
		UseAllowVerbs = GetPrivateProfileInt("options","UseAllowVerbs",UseAllowVerbs,fn)>0;
		UseDenyVerbs = !GetPrivateProfileInt("options","UseAllowVerbs",!UseDenyVerbs,fn);
		UseAllowExtensions = GetPrivateProfileInt("options","UseAllowExtensions",UseAllowExtensions,fn)>0;
		UseDenyExtensions = !GetPrivateProfileInt("options","UseAllowExtensions",!UseDenyExtensions,fn);
		Deny_PATH_Dot = !GetPrivateProfileInt("options","AllowDotInPath",!Deny_PATH_Dot,fn);
		Logging = GetPrivateProfileInt("options","EnableLogging",Logging,fn)>0;
		AllowLateScanning = GetPrivateProfileInt("options","AllowLateScanning",AllowLateScanning,fn)>0;
		
		CString url(ResponseRedirectURL);
		GetPrivateProfileString("options","RejectResponseUrl","",buf,sz,fn);
		ResponseRedirectURL = buf; CleanupString(ResponseRedirectURL);
		if(ResponseRedirectURL=="") ResponseRedirectURL=url;
		ResponseRedirect = !GetPrivateProfileInt("options","UseFastPathReject",!ResponseRedirect,fn);
		ResponseDirectly = GetPrivateProfileInt("options","UseFastPathReject",0,fn)>0;

		RemoveServerHeader = GetPrivateProfileInt("options","RemoveServerHeader",RemoveServerHeader,fn)>0;
		GetPrivateProfileString("options","AlternateServerName","",buf,sz,fn);
		ServerHeader = buf; CleanupString(ServerHeader);
		ChangeServerHeader = ServerHeader.GetLength()>0;

		CString ldir(LogDir);
		GetPrivateProfileString("options","LoggingDirectory","",buf,sz,fn);
		LogDir = buf; CleanupString(LogDir);
		if(LogDir=="") LogDir=ldir;

		//NormalizeURLBeforeScan = GetPrivateProfileInt("options","NormalizeURLBeforeScan",1,fn);
		//VerifyNormalization = GetPrivateProfileInt("options","VerifyNormalization",1,fn);
		Deny_URL_HighBitShellcode = !GetPrivateProfileInt("options","AllowHighBitCharacters",!Deny_URL_HighBitShellcode,fn);
	
		//LogLongUrls = GetPrivateProfileInt("options","LogLongUrls",0,fn);
		//PerDayLogging = GetPrivateProfileInt("options","PerDayLogging",1,fn);
		LogPerProcess = GetPrivateProfileInt("options","PerProcessLogging",0,fn)>0;
		
		DWORD l;

		//[AllowVerbs]
		l = GetPrivateProfileSection("AllowVerbs",buf, sz,fn);
		Split(buf,l,Allow_VERB_List);
		CleanupList(Allow_VERB_List);
		
		//[DenyVerbs]
		l = GetPrivateProfileSection("DenyVerbs",buf, sz,fn);
		Split(buf,l,Deny_VERB_List);
		CleanupList(Deny_VERB_List);
		
		//[DenyHeaders]
		l = GetPrivateProfileSection("DenyHeaders",buf, sz,fn);
		Split(buf,l,Deny_HEADER_List);
		CleanupList(Deny_HEADER_List);
		UseDenyHeaders = l>0;

		//[AllowExtensions]
		l = GetPrivateProfileSection("AllowExtensions",buf, sz,fn);
		Split(buf,l,Allow_EXTENSION_List);
		if(l) Allow_EXTENSION_List.AddHead("");			//always allow default document!!!
		CleanupListLCase(Allow_EXTENSION_List);

		//[DenyExtensions]
		l = GetPrivateProfileSection("DenyExtensions",buf, sz,fn);
		Split(buf,l,Deny_EXTENSION_List);
		CleanupListLCase(Deny_EXTENSION_List);
		
		//[DenyUrlSequences]
		l = GetPrivateProfileSection("DenyUrlSequences",buf, sz,fn);
		Split(buf,l,Deny_URL_Sequences);
		CleanupListLCase(Deny_URL_Sequences);
		UseDenyURLSequences = l>0;

		//[RequestLimits]
		GetPrivateProfileString("RequestLimits","MaxAllowedContentLength",MaxContentLength,buf,sz,fn);
		MaxContentLength = buf; CleanupString(MaxContentLength);
		LimitContentLength = MaxContentLength.GetLength()>0;

		MaxURL = GetPrivateProfileInt("RequestLimits","MaxUrl",MaxURL,fn);
		LimitURL = MaxURL>0;

		MaxQueryString = GetPrivateProfileInt("RequestLimits","MaxQueryString",MaxQueryString,fn);
		LimitQueryString = MaxQueryString>0;

		l = GetPrivateProfileSection("RequestLimits",buf, sz,fn);
		CStringList MaxLimits; POSITION oldpos;
		Split(buf,l,MaxLimits);
		POSITION pos = MaxLimits.GetHeadPosition();
		while (pos != NULL){
			oldpos = pos;
			CString& str = MaxLimits.GetNext(pos);
			CleanupString(str);
			if (str.Left(4)!="Max-")
				MaxLimits.RemoveAt(oldpos);
		}
		if(l>0){
			MaxHeaders.RemoveAll();	//if section is found, remove defaults
			pos = MaxLimits.GetHeadPosition();
			CString val; int poseq;
			while (pos!=NULL){
				CString& str = MaxLimits.GetNext(pos);
				poseq = str.Find('=');
				if(poseq!=-1){
					if(str.Mid(poseq+1) != "")
						MaxHeaders.SetAt(str.Mid(4,poseq-4)+":",str.Mid(poseq+1));
				}
			}
			UseMaxHeaders = true;
		}else{
			UseMaxHeaders = false;
		}

		return true;
	}catch(...){
		LoadDefaults();
		return false;
	}
}

bool CWebKnightSettings::ReadFromFileINI(LPCTSTR fn)
{
	try{
		const int sz = MAX_INI_FILE_SIZE;
		char buf[sz];
		DWORD l;

		//Scanning Engine
		AllowLateScanning = GetPrivateProfileInt("Scanning Engine","AllowLateScanning",AllowLateScanning,fn)>0;
		ScanNonSecurePort = GetPrivateProfileInt("Scanning Engine","ScanNonSecurePort",ScanNonSecurePort,fn)>0;
		ScanSecurePort = GetPrivateProfileInt("Scanning Engine","ScanSecurePort",ScanSecurePort,fn)>0;
		UseExcludedInstances = GetPrivateProfileInt("Scanning Engine","UseExcludedWebInstances",UseExcludedInstances,fn)>0;

		//Excluded Web Instances
		l = GetPrivateProfileSection("Excluded Web Instances",buf,sz,fn);
		Split(buf,l,ExcludedInstances);
		CleanupList(ExcludedInstances);

		//Intrusion Response Handling
		ResponseDirectly = GetPrivateProfileInt("Intrusion Response Handling","ResponseDirectly",ResponseDirectly,fn)>0;
		ResponseRedirect = GetPrivateProfileInt("Intrusion Response Handling","ResponseRedirect",ResponseRedirect,fn)>0;
		CString url(ResponseRedirectURL);
		GetPrivateProfileString("Intrusion Response Handling","ResponseRedirectUrl","",buf,sz,fn);
		ResponseRedirectURL = buf; CleanupString(ResponseRedirectURL);
		if(ResponseRedirectURL=="") ResponseRedirectURL=url;
		ResponseLogOnly = GetPrivateProfileInt("Intrusion Response Handling","ResponseLogOnly",ResponseLogOnly,fn)>0;
		UseResponseStatus = GetPrivateProfileInt("Intrusion Response Handling","UseResponseStatus",UseResponseStatus,fn)>0;
		CString rstatus(ResponseStatus);
		GetPrivateProfileString("Intrusion Response Handling","ResponseStatus","",buf,sz,fn);
		ResponseStatus = buf; CleanupString(ResponseStatus);
		if(ResponseStatus=="") ResponseStatus=rstatus;
	
		//Logging
		Logging = GetPrivateProfileInt("Logging","Enabled",Logging,fn)>0;
		CString ldir(LogDir);
		GetPrivateProfileString("Logging","LogDirectory","",buf,sz,fn);
		LogDir = buf; CleanupString(LogDir);
		if(LogDir=="") LogDir=ldir;
		LogGMT = GetPrivateProfileInt("Logging","UseGMT",LogGMT,fn)>0;
		LogPerProcess = GetPrivateProfileInt("Logging","PerProcessLogging",LogPerProcess,fn)>0;
		LogRetention = GetPrivateProfileInt("Logging","LogRetention",LogRetention,fn);
		LogAllowedRequests = GetPrivateProfileInt("Logging","LogAllowedRequests",LogAllowedRequests,fn)>0;
		LogClientIP = GetPrivateProfileInt("Logging","LogClientIP",LogClientIP,fn)>0;
		LogUserName = GetPrivateProfileInt("Logging","LogUserName",LogUserName,fn)>0;
		Log_HTTP_VIA = GetPrivateProfileInt("Logging","Log_HTTP_VIA",Log_HTTP_VIA,fn)>0;
		Log_HTTP_X_FORWARDED_FOR = GetPrivateProfileInt("Logging","Log_HTTP_X_FORWARDED_FOR",Log_HTTP_X_FORWARDED_FOR,fn)>0;
		LogUserAgent = GetPrivateProfileInt("Logging","LogUserAgent",LogUserAgent,fn)>0;
		Log_HTTP_ERROR_ClientError = GetPrivateProfileInt("Logging","Log_HTTP_ClientErrors",Log_HTTP_ERROR_ClientError,fn)>0;
		Log_HTTP_ERROR_ServerError = GetPrivateProfileInt("Logging","Log_HTTP_ServerErrors",Log_HTTP_ERROR_ServerError,fn)>0;
		
		//Request Limits
		LimitContentLength = GetPrivateProfileInt("Request Limits","LimitContentLength",LimitContentLength,fn)>0;
		LimitURL = GetPrivateProfileInt("Request Limits","LimitURL",LimitURL,fn)>0;
		LimitQueryString = GetPrivateProfileInt("Request Limits","LimitQueryString",LimitQueryString,fn)>0;
		LimitVersion = GetPrivateProfileInt("Request Limits","LimitHTTPVersion",LimitVersion,fn)>0;
		CString cl(MaxContentLength);
		GetPrivateProfileString("Request Limits","MaxContentLength","",buf,sz,fn);
		MaxContentLength = buf; CleanupString(MaxContentLength);
		if(MaxContentLength=="") MaxContentLength=cl;
		MaxURL = GetPrivateProfileInt("Request Limits","MaxURL",MaxURL,fn);
		MaxQueryString = GetPrivateProfileInt("Request Limits","MaxQueryString",MaxQueryString,fn);
		MaxVersion = GetPrivateProfileInt("Request Limits","MaxHTTPVersion",MaxVersion,fn);
		UseAllowVersions = GetPrivateProfileInt("Request Limits","UseAllowedHTTPVersions",UseAllowVersions,fn)>0;
		UseMaxHeaders = GetPrivateProfileInt("Request Limits","UseMaxHeaders",UseMaxHeaders,fn)>0;

		//Allowed HTTP Versions
		l = GetPrivateProfileSection("Allowed HTTP Versions",buf,sz,fn);
		Split(buf,l,Allow_VERSION_List);
		CleanupListUCase(Allow_VERSION_List);
		
		//MaxHeaders
		l = GetPrivateProfileSection("MaxHeaders",buf, sz,fn);
		CStringList MaxLimits;
		Split(buf,l,MaxLimits);
		POSITION pos = MaxLimits.GetHeadPosition();
		while (pos != NULL){
			CString& str = MaxLimits.GetNext(pos);
			CleanupString(str);
		}
		if(l>0){
			MaxHeaders.RemoveAll();	//if section is found, remove defaults
			pos = MaxLimits.GetHeadPosition();
			CString val; int poseq; CString tmp;
			while (pos!=NULL){
				CString& str = MaxLimits.GetNext(pos);
				poseq = str.Find('=');
				if(poseq!=-1){
					if(str.Mid(poseq+1) != ""){
						tmp = str.Mid(0,poseq);
						if (tmp.Right(1) != ':')
							tmp += ':';				//add ':' if not there
						MaxHeaders.SetAt(tmp,str.Mid(poseq+1));
					}
				}
			}
		}

		//URL Scanning
		RFCCompliantURL = GetPrivateProfileInt("URL Scanning","RFCCompliantUrl",RFCCompliantURL,fn)>0;
		UseURLRawScan = GetPrivateProfileInt("URL Scanning","UseUrlRawScan",UseURLRawScan,fn)>0;
		Deny_URL_EncodingExploits = GetPrivateProfileInt("URL Scanning","DenyUrlEncodingExploits",Deny_URL_EncodingExploits,fn)>0;
		Deny_URL_ParentPath = GetPrivateProfileInt("URL Scanning","DenyUrlParentPath",Deny_URL_ParentPath,fn)>0;
		Deny_URL_TrailingDotInDir = GetPrivateProfileInt("URL Scanning","DenyTrailingDotInDir",Deny_URL_TrailingDotInDir,fn)>0;
		Deny_URL_Backslash = GetPrivateProfileInt("URL Scanning","DenyUrlBackslash",Deny_URL_Backslash,fn)>0;
		Deny_URL_AlternateStream = GetPrivateProfileInt("URL Scanning","DenyUrlAlternateStream",Deny_URL_AlternateStream,fn)>0;
		Deny_URL_Escaping = GetPrivateProfileInt("URL Scanning","DenyUrlEscaping",Deny_URL_Escaping,fn)>0;
		Deny_URL_MultipleCGI = GetPrivateProfileInt("URL Scanning","DenyUrlRunningMultipleCGI",Deny_URL_MultipleCGI,fn)>0;
		GetPrivateProfileString("URL Scanning","DenyUrlCharacters","",buf,sz,fn);
		Deny_URL_Characters = buf; CleanupString(Deny_URL_Characters);
		Deny_URL_HighBitShellcode = GetPrivateProfileInt("URL Scanning","DenyUrlHighBitShellcode",Deny_URL_HighBitShellcode,fn)>0;
		UseDenyURLSequences = GetPrivateProfileInt("URL Scanning","UseDeniedUrlSequences",UseDenyURLSequences,fn)>0;
		UseAllowURL = GetPrivateProfileInt("URL Scanning","UseAllowedURLStarts",UseAllowURL,fn)>0;

		//URL Denied sequences
		l = GetPrivateProfileSection("URL Denied Sequences",buf,sz,fn);
		Split(buf,l,Deny_URL_Sequences);
		CleanupListLCase(Deny_URL_Sequences);

		//URL Allowed starts
		l = GetPrivateProfileSection("URL Allowed starts",buf,sz,fn);
		Split(buf,l,Allow_URL_List);
		CleanupListLCase(Allow_URL_List);

		//Mapped Path
		Deny_FULLPATH_ParentPath = GetPrivateProfileInt("Mapped Path","DenyParentPath",Deny_FULLPATH_ParentPath,fn)>0;
		Deny_FULLPATH_Backspace = GetPrivateProfileInt("Mapped Path","DenyBackspace",Deny_FULLPATH_Backspace,fn)>0;
		Deny_FULLPATH_CarriageReturn = GetPrivateProfileInt("Mapped Path","DenyParentPath",Deny_FULLPATH_CarriageReturn,fn)>0;
		Deny_FULLPATH_NewLine = GetPrivateProfileInt("Mapped Path","DenyParentPath",Deny_FULLPATH_NewLine,fn)>0;
		Deny_FULLPATH_Escaping = GetPrivateProfileInt("Mapped Path","DenyParentPath",Deny_FULLPATH_Escaping,fn)>0;
		
		GetPrivateProfileString("Mapped Path","DenyCharacters","",buf,sz,fn);
		Deny_FULLPATH_Characters = buf; CleanupString(Deny_FULLPATH_Characters);
		
		Deny_PATH_Dot = GetPrivateProfileInt("Mapped Path","DenyDotInPath",Deny_PATH_Dot,fn)>0;
		UseAllowPaths = GetPrivateProfileInt("Mapped Path","UseAllowedPaths",UseAllowPaths,fn)>0;

		//Allowed Paths
		l = GetPrivateProfileSection("Allowed Paths",buf,sz,fn);
		Split(buf,l,Allow_PATH_List);
		CleanupListLCase(Allow_PATH_List);
		
		//Requested File
		GetPrivateProfileString("Requested File","DenyFilenameCharacters","",buf,sz,fn);
		Deny_FILENAME_Characters = buf; CleanupString(Deny_FILENAME_Characters);
		Deny_FILENAME_DefaultDocument = GetPrivateProfileInt("Requested File","DenyFilenameDefaultDocument",Deny_FILENAME_DefaultDocument,fn)>0;
		UseDenyFilenameCGIApplications = GetPrivateProfileInt("Requested File","UseDeniedFiles",UseDenyFilenameCGIApplications,fn)>0;

		//Denied files
		l = GetPrivateProfileSection("Denied Files",buf,sz,fn);
		Split(buf,l,Deny_FILENAME_CGIApplications);
		CleanupListLCase(Deny_FILENAME_CGIApplications);

		//Requested Extension
		UseAllowExtensions = GetPrivateProfileInt("Requested Extension","UseAllowedExtensions",UseAllowExtensions,fn)>0;
		UseDenyExtensions = GetPrivateProfileInt("Requested Extension","UseDeniedExtensions",UseDenyExtensions,fn)>0;
		
		//Allowed Extensions
		l = GetPrivateProfileSection("Allowed Extensions",buf,sz,fn);
		Split(buf,l,Allow_EXTENSION_List);
		if(l) Allow_EXTENSION_List.AddHead(""); //always allow default document
		CleanupListLCase(Allow_EXTENSION_List);

		//Denied Extensions
		l = GetPrivateProfileSection("Denied Extensions",buf,sz,fn);
		Split(buf,l,Deny_EXTENSION_List);
		CleanupListLCase(Deny_EXTENSION_List);

		//HTTP Headers
		RemoveServerHeader = GetPrivateProfileInt("HTTP Headers","RemoveServerHeader",RemoveServerHeader,fn)>0;
		ChangeServerHeader = GetPrivateProfileInt("HTTP Headers","ChangeServerHeader",ChangeServerHeader,fn)>0;
		GetPrivateProfileString("HTTP Headers","ServerHeader","",buf,sz,fn);
		ServerHeader = buf; CleanupString(ServerHeader);
		Deny_COOKIE_SQLinjection = GetPrivateProfileInt("HTTP Headers","DenyCookieSQLinjection",Deny_COOKIE_SQLinjection,fn)>0;
		Deny_COOKIE_EncodingExploits = GetPrivateProfileInt("HTTP Headers","DenyCookieEncodingExploits",Deny_COOKIE_EncodingExploits,fn)>0;
		RFCCompliantHostHeader = GetPrivateProfileInt("HTTP Headers","RFCCompliantHostHeader",RFCCompliantHostHeader,fn)>0;
		UseDenyHeaders = GetPrivateProfileInt("HTTP Headers","UseDeniedHeaders",UseDenyHeaders,fn)>0;
		UseAllowHeaderContentType = GetPrivateProfileInt("HTTP Headers","UseAllowedContentTypes",UseAllowHeaderContentType,fn)>0;
		
		//Denied Headers
		l = GetPrivateProfileSection("Denied Headers",buf,sz,fn);
		Split(buf,l,Deny_HEADER_List);
		CleanupList(Deny_HEADER_List);

		//Allowed Content-Types
		l = GetPrivateProfileSection("Allowed Content-Types",buf,sz,fn);
		Split(buf,l,Allow_HEADER_ContentType);
		CleanupListLCase(Allow_HEADER_ContentType);
		if(l) Allow_HEADER_ContentType.AddHead("");	//always allow no content-type

		//HTTP Verbs (Request Methods)
		UseAllowVerbs = GetPrivateProfileInt("HTTP Verbs (Request Methods)","UseAllowedVerbs",UseAllowVerbs,fn)>0;
		UseDenyVerbs = GetPrivateProfileInt("HTTP Verbs (Request Methods)","UseDeniedVerbs",UseDenyVerbs,fn)>0;

		//Allowed Verbs
		l = GetPrivateProfileSection("Allowed Verbs",buf,sz,fn);
		Split(buf,l,Allow_VERB_List);
		CleanupList(Allow_VERB_List);

		//Denied Verbs
		l = GetPrivateProfileSection("Denied Verbs",buf,sz,fn);
		Split(buf,l,Deny_VERB_List);
		CleanupListUCase(Deny_VERB_List);

		//SQL Injection Keywords (TO DO: ';' cannot be read again)
		//l = GetPrivateProfileSection("SQL Injection Keywords",buf,sz,fn);
		//Split(buf,l,Deny_SQL_KeyWords);
		//CleanupListLCase(Deny_SQL_KeyWords);

		//Querystring Scanning
		UseQuerystringRawScan = GetPrivateProfileInt("Querystring Scanning","UseQuerystringRawScan",UseQuerystringRawScan,fn)>0;
		Deny_QUERYSTRING_SQLinjection = GetPrivateProfileInt("Querystring Scanning","DenyQuerystringSQLinjection",Deny_QUERYSTRING_SQLinjection,fn)>0;
		Deny_QUERYSTRING_EncodingExploits = GetPrivateProfileInt("Querystring Scanning","DenyQuerystringEncodingExploits",Deny_QUERYSTRING_EncodingExploits,fn)>0;
		Deny_QUERYSTRING_DirectoryTraversal = GetPrivateProfileInt("Querystring Scanning","DenyQuerystringDirectoryTraversal",Deny_QUERYSTRING_DirectoryTraversal,fn)>0;
		Deny_QUERYSTRING_HighBitShellcode = GetPrivateProfileInt("Querystring Scanning","DenyQuerystringHighBitShellcode",Deny_QUERYSTRING_HighBitShellcode,fn)>0;
		UseDenyQuerystringSequences = GetPrivateProfileInt("Querystring Scanning","UseDeniedQuerystringSequences",UseDenyQuerystringSequences,fn)>0;

		//Denied Querystring Sequences
		l = GetPrivateProfileSection("Denied Querystring Sequences",buf,sz,fn);
		Split(buf,l,Deny_QUERYSTRING_Sequences);
		CleanupListLCase(Deny_QUERYSTRING_Sequences);

		//Global Filter Capabilities
		IsInstalledAsGlobalFilter = GetPrivateProfileInt("Global Filter Capabilities","IsInstalledAsGlobalFilter",IsInstalledAsGlobalFilter,fn)>0;
		Deny_HEADER_SQLinjection = GetPrivateProfileInt("Global Filter Capabilities","DenyHeaderSQLinjection",Deny_HEADER_SQLinjection,fn)>0;
		Deny_HEADER_EncodingExploits = GetPrivateProfileInt("Global Filter Capabilities","DenyHeaderEncodingExploits",Deny_HEADER_EncodingExploits,fn)>0;
		Deny_HEADER_DirectoryTraversal = GetPrivateProfileInt("Global Filter Capabilities","DenyHeaderDirectoryTraversal",Deny_HEADER_DirectoryTraversal,fn)>0;
		Deny_HEADER_HighBitShellcode = GetPrivateProfileInt("Global Filter Capabilities","DenyHeaderHighBitShellcode",Deny_HEADER_HighBitShellcode,fn)>0;
		UseDenyHeaderSequences = GetPrivateProfileInt("Global Filter Capabilities","UseDeniedHeaderSequences",UseDenyHeaderSequences,fn)>0;
		Deny_POST_SQLinjection = GetPrivateProfileInt("Global Filter Capabilities","DenyPostDataSQLinjection",Deny_POST_SQLinjection,fn)>0;
		Deny_POST_EncodingExploits = GetPrivateProfileInt("Global Filter Capabilities","DenyPostDataEncodingExploits",Deny_POST_EncodingExploits,fn)>0;
		Deny_POST_DirectoryTraversal = GetPrivateProfileInt("Global Filter Capabilities","DenyPostDataDirectoryTraversal",Deny_POST_DirectoryTraversal,fn)>0;
		Deny_POST_HighBitShellcode = GetPrivateProfileInt("Global Filter Capabilities","DenyPostDataHighBitShellcode",Deny_POST_HighBitShellcode,fn)>0;
		UseDenyPOSTSequences = GetPrivateProfileInt("Global Filter Capabilities","UseDeniedPostDataSequences",UseDenyPOSTSequences,fn)>0;

		//Denied Header Sequences
		l = GetPrivateProfileSection("Denied Header Sequences",buf,sz,fn);
		Split(buf,l,Deny_HEADER_Sequences);
		CleanupListLCase(Deny_HEADER_Sequences);

		//Denied Post Sequences
		l = GetPrivateProfileSection("Denied Post Sequences",buf,sz,fn);
		Split(buf,l,Deny_POST_Sequences);
		CleanupListLCase(Deny_POST_Sequences);

		//Web applications
		Allow_WEBAPP_OWA = GetPrivateProfileInt("Web Applications","AllowOutlookWebAccess",Allow_WEBAPP_OWA,fn)>0;
		if(Allow_WEBAPP_OWA) EnableOWA();
		Allow_WEBAPP_Frontpage = GetPrivateProfileInt("Web Applications","AllowFrontpageExtensions",Allow_WEBAPP_Frontpage,fn)>0;
		if(Allow_WEBAPP_Frontpage) EnableFrontpage();
		Allow_WEBAPP_Coldfusion = GetPrivateProfileInt("Web Applications","AllowColdfusion",Allow_WEBAPP_Coldfusion,fn)>0;
		if(Allow_WEBAPP_Coldfusion) EnableColdfusion();
		Allow_WEBAPP_Flash = GetPrivateProfileInt("Web Applications","AllowFlashMX",Allow_WEBAPP_Flash,fn)>0;
		if(Allow_WEBAPP_Flash) EnableFlash();
		Allow_WEBAPP_WebDAV = GetPrivateProfileInt("Web Applications","AllowWebDAV",Allow_WEBAPP_WebDAV,fn)>0;
		if(Allow_WEBAPP_WebDAV) EnableWebDAV();
		Allow_WEBAPP_IISADMPWD = GetPrivateProfileInt("Web Applications","AllowIISADMPWD",Allow_WEBAPP_IISADMPWD,fn)>0;
		if(Allow_WEBAPP_IISADMPWD) EnableIISADMPWD();
		Allow_WEBAPP_SPPS = GetPrivateProfileInt("Web Applications","AllowSharePointPortalServer",Allow_WEBAPP_SPPS,fn)>0;
		if(Allow_WEBAPP_SPPS) EnableSPPortalServer();
		Allow_WEBAPP_SPTS = GetPrivateProfileInt("Web Applications","AllowSharePointTeamServices",Allow_WEBAPP_SPTS,fn)>0;
		if(Allow_WEBAPP_SPTS) EnableSPTeamServices();
		Allow_WEBAPP_CertificateServices = GetPrivateProfileInt("Web Applications","AllowCertificateServices",Allow_WEBAPP_CertificateServices,fn)>0;
		if(Allow_WEBAPP_CertificateServices) EnableCertificateServices();
		Allow_WEBAPP_BizTalk = GetPrivateProfileInt("Web Applications","AllowBizTalkServer",Allow_WEBAPP_BizTalk,fn)>0;
		if(Allow_WEBAPP_BizTalk) EnableBizTalk();
		Allow_WEBAPP_Commerce = GetPrivateProfileInt("Web Applications","AllowCommerceServer",Allow_WEBAPP_Commerce,fn)>0;
		if(Allow_WEBAPP_Commerce) EnableCommerceServer();
		Allow_WEBAPP_SBS = GetPrivateProfileInt("Web Applications","AllowSmallBusinessServer",Allow_WEBAPP_SBS,fn)>0;
		if(Allow_WEBAPP_SBS) EnableSBS();
		Allow_WEBAPP_ASPNET = GetPrivateProfileInt("Web Applications","AllowASP.NET",Allow_WEBAPP_ASPNET,fn)>0;
		if(Allow_WEBAPP_ASPNET) EnableASPDotNet();
		Allow_WEBAPP_ASP = GetPrivateProfileInt("Web Applications","AllowASP",Allow_WEBAPP_ASP,fn)>0;
		if(Allow_WEBAPP_ASP) EnableASP();
		
		return true;
	}catch(...){
		LoadDefaults();
		return false;
	}
}

void CWebKnightSettings::LoadInvalidValues()
{
	/*
	 * Function which loads invalid settings into memory. This function is for
	 * testing the ReadFrom & WriteTo functions family. It is for debugging only.
	 * Call this function te check if all settings are parsed correctly.
	 * Procedure:
	 * 1) Call LoadDefaults() and write settings with your WriteTo function
	 * 2) Call LoadInvalidValues()
	 * 3) Call your ReadFrom function and read the file you made with default settings,
	 *    write these to another file and they should match the default settings.
	 *    If not, you forgot to write or read a setting or it was not done correctly.
	 */

	//Scanning engine
	AllowLateScanning = true;
	ScanNonSecurePort = false;
	ScanSecurePort = false;
	UseExcludedInstances = false;
	ExcludedInstances.RemoveAll();
	ExcludedInstances.AddTail("invalidinstance");

	//Response handling
	UseResponseStatus = false;			//if ResponseRedirect=true, then this has no effect
	ResponseStatus = "INVALID";			//"404 Object Not Found"; //maybe an invalid response will crash script kiddiots tools (TO DO: check this)
	ResponseRedirect = false;
	ResponseRedirectURL = "/invalid.htm";
	ResponseDirectly = false;
	ResponseLogOnly = true;			//if true, log only (debug function)

	//logging
	Logging = false;
	LogDir = "c:\\WINNT\\System32\\LogFiles\\WebKnight\\invalid\\";
	LogGMT = false;
	LogPerProcess = true;			//unique logdir per process (for multiple site filters)
	LogRetention = 343;				//number of days to keep log files
	LogAllowedRequests = true;		//log requests which are allowed
	LogClientIP = false;
	LogUserName = false;
	Log_HTTP_VIA = true;			//HTTP VIA headers (used in TRACE)
	Log_HTTP_X_FORWARDED_FOR = true;//certain ISPs use a proxy which add this header (one of those ISPs is telenet.be)
	LogUserAgent = true;			//not essential information except maybe to identify script kiddie tools
	Log_HTTP_ERROR_ClientError = true;
	Log_HTTP_ERROR_ServerError = false;

	//Limits
	LimitContentLength = false;
	LimitURL = false;
	LimitQueryString = false;
	LimitVersion = false;
	MaxContentLength="66666";//max value of content length header (to prevent overflow, it cannot be longer than 9 characters)
	MaxURL = 666; //16384;		//max length of url (MAX_PATH=260)
	MaxQueryString=666;		//max length of querystring
	MaxVersion=666;				//max length of version string, in the form of HTTP/<major>.<minor> (e.g. HTTP/1.1) (HTTP/0.9 bug? buffer overflow? TO DO)
	UseMaxHeaders = false;
	MaxHeaders.SetAt("invalid header:","10");

	//URL
	Deny_URL_ParentPath = false;		//no ".." in url
	Deny_URL_TrailingDotInDir = false;	//no "./" in url
	Deny_URL_Backslash = false;			//no '\' in url
	Deny_URL_AlternateStream = true;	//you should allow ':' in url (see rfc 2616), urlscan blocks it, which is wrong!!!
	Deny_URL_Escaping = false;			//no '%' in url (after url normalization)
	Deny_URL_MultipleCGI = false;		//no '&' in url
	Deny_URL_Characters = "!@";			//other characters to block (TO DO: check this)
	Deny_URL_HighBitShellcode = false;

	//URL: Deny certain sequences (must be lowercase)
	UseDenyURLSequences = false;
	Deny_URL_Sequences.RemoveAll();
	Deny_URL_Sequences.AddTail("/invalid/");

	//URL: Only allow certain first characters of a URL (must be lowercase)
	UseAllowURL = false;
	Allow_URL_List.RemoveAll();
	Allow_URL_List.AddTail("invalid");		//allow all relative URLs (http 1.0)
	
	//FULLPATH
	Deny_FULLPATH_ParentPath = false;
	Deny_FULLPATH_Backspace = false;			
	Deny_FULLPATH_CarriageReturn = false;	
	Deny_FULLPATH_NewLine = false;			
	Deny_FULLPATH_Escaping = false;			
	Deny_FULLPATH_Characters = "!@";	

	//PATH
	Deny_PATH_Dot = true; //deny dot in path

	UseAllowPaths = false;
	Allow_PATH_List.RemoveAll();
	Allow_PATH_List.AddTail("c:\\invalid");		

	//FILENAME
	Deny_FILENAME_Characters = "!@";
	Deny_FILENAME_DefaultDocument = true;

	//FILENAME: Deny almost all Windows 2000 commands (must be lowercase)
	UseDenyFilenameCGIApplications = false;
	Deny_FILENAME_CGIApplications.RemoveAll();
	Deny_FILENAME_CGIApplications.AddTail("invalid.exe");

	//EXTENSION: Allowed extensions (Common MIME types)
	UseAllowExtensions = true;
	Allow_EXTENSION_List.RemoveAll();
	Allow_EXTENSION_List.AddTail("");			
	Allow_EXTENSION_List.AddTail(".invalidallowed");	

	//EXTENSION: Denied extensions
	UseDenyExtensions = false;
	Deny_EXTENSION_List.RemoveAll();
	Deny_EXTENSION_List.AddTail(".invaliddenied");

	//HEADERS
	RemoveServerHeader = true;
	ChangeServerHeader = false;
	ServerHeader = "INVALID/1.1";//"Apache/0.8.4"; //in the form of <application>/<version_major>.<version_minor>
	Deny_COOKIE_SQLinjection = false;
	Deny_COOKIE_EncodingExploits = false;
	RFCCompliantHostHeader = false;
	UseDenyHeaders = false;
	Deny_HEADER_List.RemoveAll();
	Deny_HEADER_List.AddTail("Invalid:");
	UseAllowHeaderContentType = false;
	Allow_HEADER_ContentType.RemoveAll();
	Allow_HEADER_ContentType.AddTail("<invalid>");

	//VERBS: Allowed verbs list (case sensitive: see RFC 2616!)
	UseAllowVerbs = false;
	Allow_VERB_List.RemoveAll();
	Allow_VERB_List.AddTail("INVALIDALLOWED");

	//VERBS: Denied verbs list (must be uppercase! lowercase & variants will also be blocked)
	UseDenyVerbs = true;
	Deny_VERB_List.RemoveAll();
	Deny_VERB_List.AddTail("INVALIDDENIED");

	//SQL Injection (words in list should be lowercase, if 2 or more are found, request will be blocked)
	Deny_SQL_KeyWords.RemoveAll();
	Deny_SQL_KeyWords.AddTail("invalidSQLkeyword");

	//QUERYSTRING
	Deny_QUERYSTRING_SQLinjection = false;
	Deny_QUERYSTRING_EncodingExploits = false;
	Deny_QUERYSTRING_DirectoryTraversal = false;
	Deny_QUERYSTRING_HighBitShellcode = true;
	UseDenyQuerystringSequences = false;				
	Deny_QUERYSTRING_Sequences.RemoveAll();
	Deny_QUERYSTRING_Sequences.AddTail("<invalid>");	
	
	//raw data (POST data & others)
	IsInstalledAsGlobalFilter = false;
	Deny_POST_SQLinjection = false;
	Deny_POST_EncodingExploits = false;
	Deny_POST_DirectoryTraversal = true;
	Deny_POST_HighBitShellcode = true;
	UseDenyPOSTSequences = false;
	Deny_POST_Sequences.RemoveAll();
	Deny_POST_Sequences.AddTail("<invalidrawpost>");
	Deny_HEADER_SQLinjection = false;
	Deny_HEADER_EncodingExploits = false;
	Deny_HEADER_DirectoryTraversal = false;
	Deny_HEADER_HighBitShellcode = true;
	UseDenyHeaderSequences = false;
	Deny_HEADER_Sequences.RemoveAll();
	Deny_HEADER_Sequences.AddTail("<invalidrawheaders>");

	//Allow compatible web applications
	Allow_WEBAPP_OWA = true;
	Allow_WEBAPP_Frontpage = true;
	Allow_WEBAPP_Coldfusion = true;
	Allow_WEBAPP_Flash = true;
	Allow_WEBAPP_WebDAV = true;
	Allow_WEBAPP_IISADMPWD = true;
	Allow_WEBAPP_SPPS = true;
	Allow_WEBAPP_SPTS = true;
	Allow_WEBAPP_CertificateServices = true;
	Allow_WEBAPP_BizTalk = true;
	Allow_WEBAPP_Commerce = true;
	Allow_WEBAPP_SBS = true;
	Allow_WEBAPP_ASPNET = true;
	Allow_WEBAPP_ASP = true;
}


void CWebKnightSettings::ParseXML(CString key, CString val)
{
	//if not a valid key or value
	if (key=="" || val=="")
		return;

	CString name = XMLDecode(XMLKeyParameter(key,"name"));
	
	//Scanning Engine
	if(name.CompareNoCase("Allow_Late_Scanning")==0){
		AllowLateScanning = val=="1"?true:false;
	}else if(name.CompareNoCase("Scan_Non_Secure_Port")==0){
		ScanNonSecurePort = val=="1"?true:false;
	}else if(name.CompareNoCase("Scan_Secure_Port")==0){
		ScanSecurePort = val=="1"?true:false;
	}else if(name.CompareNoCase("Use_Excluded_Web_Instances")==0){
		UseExcludedInstances = val=="1"?true:false;
	}else if(name.CompareNoCase("Excluded_Web_Instances")==0){
		ConvertXMLToList(val,ExcludedInstances);

	//Response Handling
	}else if(name.CompareNoCase("Response_Directly")==0){
		ResponseDirectly = val=="1"?true:false;
	}else if(name.CompareNoCase("Response_Redirect")==0){
		ResponseRedirect = val=="1"?true:false;
	}else if(name.CompareNoCase("Response_Redirect_URL")==0){
		ResponseRedirectURL = XMLDecode(val);
	}else if(name.CompareNoCase("Use_Response_Status")==0){
		UseResponseStatus = val=="1"?true:false;
	}else if(name.CompareNoCase("Response_Status")==0){
		ResponseStatus = XMLDecode(val);
	}else if(name.CompareNoCase("Response_Log_Only")==0){
		ResponseLogOnly = val=="1"?true:false;

	//Logging
	}else if(name.CompareNoCase("Enabled")==0){
		Logging = val=="1"?true:false;
	}else if(name.CompareNoCase("Log_Directory")==0){
		LogDir = XMLDecode(val);
	}else if(name.CompareNoCase("Use_GMT")==0){
		LogGMT = val=="1"?true:false;
	}else if(name.CompareNoCase("Per_Process_Logging")==0){
		LogPerProcess = val=="1"?true:false;
	}else if(name.CompareNoCase("Log_Retention")==0){
		LogRetention = atoi(XMLDecode(val));
	}else if(name.CompareNoCase("Log_Allowed_Requests")==0){
		LogAllowedRequests = val=="1"?true:false;
	}else if(name.CompareNoCase("Log_Client_IP")==0){
		LogClientIP = val=="1"?true:false;
	}else if(name.CompareNoCase("Log_User_Name")==0){
		LogUserName = val=="1"?true:false;
	}else if(name.CompareNoCase("Log_HTTP_VIA")==0){
		Log_HTTP_VIA = val=="1"?true:false;
	}else if(name.CompareNoCase("Log_HTTP_X_FORWARDED_FOR")==0){
		Log_HTTP_X_FORWARDED_FOR = val=="1"?true:false;
	}else if(name.CompareNoCase("Log_User_Agent")==0){
		LogUserAgent = val=="1"?true:false;
	}else if(name.CompareNoCase("Log_HTTP_Client_Errors")==0){
		Log_HTTP_ERROR_ClientError = val=="1"?true:false;
	}else if(name.CompareNoCase("Log_HTTP_Server_Errors")==0){
		Log_HTTP_ERROR_ServerError = val=="1"?true:false;
		
	//Request Limits
	}else if(name.CompareNoCase("Limit_Content_Length")==0){
		LimitContentLength = val=="1"?true:false;
	}else if(name.CompareNoCase("Limit_URL")==0){
		LimitURL = val=="1"?true:false;
	}else if(name.CompareNoCase("Limit_Querystring")==0){
		LimitQueryString = val=="1"?true:false;
	}else if(name.CompareNoCase("Limit_HTTP_Version")==0){
		LimitVersion = val=="1"?true:false;
	}else if(name.CompareNoCase("Max_Content_Length")==0){
		MaxContentLength = XMLDecode(val);
	}else if(name.CompareNoCase("Max_URL")==0){
		MaxURL = atol(XMLDecode(val));
	}else if(name.CompareNoCase("Max_Querystring")==0){
		MaxQueryString = atol(XMLDecode(val));
	}else if(name.CompareNoCase("Max_HTTP_Version")==0){
		MaxVersion = atol(XMLDecode(val));
	}else if(name.CompareNoCase("Use_Allowed_HTTP_Versions")==0){
		UseAllowVersions = val=="1"?true:false;
	}else if(name.CompareNoCase("Use_Max_Headers")==0){
		UseMaxHeaders = val=="1"?true:false;
	}else if(name.CompareNoCase("Allowed_HTTP_Versions")==0){
		ConvertXMLToList(val,Allow_VERSION_List);
	}else if(name.CompareNoCase("Max_Headers")==0){
		ConvertXMLToMap(val,MaxHeaders);

	//URL Scanning
	}else if(name.CompareNoCase("RFC_Compliant_Url")==0){
		RFCCompliantURL = val=="1"?true:false;
	}else if(name.CompareNoCase("Use_Url_Raw_Scan")==0){
		UseURLRawScan = val=="1"?true:false;
	}else if(name.CompareNoCase("Deny_Url_Encoding_Exploits")==0){
		Deny_URL_EncodingExploits = val=="1"?true:false;
	}else if(name.CompareNoCase("Deny_Url_Parent_Path")==0){
		Deny_URL_ParentPath = val=="1"?true:false;
	}else if(name.CompareNoCase("Deny_Url_Trailing_Dot_In_Dir")==0){
		Deny_URL_TrailingDotInDir = val=="1"?true:false;
	}else if(name.CompareNoCase("Deny_Url_Backslash")==0){
		Deny_URL_Backslash = val=="1"?true:false;
	}else if(name.CompareNoCase("Deny_Url_Alternate_Stream")==0){
		Deny_URL_AlternateStream = val=="1"?true:false;
	}else if(name.CompareNoCase("Deny_Url_Escaping")==0){
		Deny_URL_Escaping = val=="1"?true:false;
	}else if(name.CompareNoCase("Deny_Url_Running_Multiple_CGI")==0){
		Deny_URL_MultipleCGI = val=="1"?true:false;
	}else if(name.CompareNoCase("Deny_Url_Characters")==0){
		Deny_URL_Characters = XMLDecode(val);
	}else if(name.CompareNoCase("Deny_Url_HighBitShellcode")==0){
		Deny_URL_HighBitShellcode = val=="1"?true:false;
	}else if(name.CompareNoCase("Use_Denied_Url_Sequences")==0){
		UseDenyURLSequences = val=="1"?true:false;
	}else if(name.CompareNoCase("UseAllowedUrlStarts")==0){	//for backward compatibility with WebKnight 1.0 & 1.1
		UseAllowURL = val=="1"?true:false;
	}else if(name.CompareNoCase("Use_Allowed_Url_Starts")==0){
		UseAllowURL = val=="1"?true:false;
	}else if(name.CompareNoCase("URL_Denied_Sequences")==0){
		ConvertXMLToList(val,Deny_URL_Sequences);
	}else if(name.CompareNoCase("URL_Allowed_starts")==0){
		ConvertXMLToList(val,Allow_URL_List);
			
	//Mapped Path
	}else if(name.CompareNoCase("Deny_Parent_Path")==0){
		Deny_FULLPATH_ParentPath = val=="1"?true:false;
	}else if(name.CompareNoCase("Deny_Backspace")==0){
		Deny_FULLPATH_Backspace = val=="1"?true:false;
	}else if(name.CompareNoCase("Deny_Carriage_Return")==0){
		Deny_FULLPATH_CarriageReturn = val=="1"?true:false;
	}else if(name.CompareNoCase("Deny_New_Line")==0){
		Deny_FULLPATH_NewLine = val=="1"?true:false;
	}else if(name.CompareNoCase("Deny_Escaping")==0){
		Deny_FULLPATH_Escaping = val=="1"?true:false;
	}else if(name.CompareNoCase("Deny_Characters")==0){
		Deny_FULLPATH_Characters = XMLDecode(val);
	}else if(name.CompareNoCase("Deny_Dot_In_Path")==0){
		Deny_PATH_Dot = val=="1"?true:false;
	}else if(name.CompareNoCase("Use_Allowed_Paths")==0){
		UseAllowPaths = val=="1"?true:false;
	}else if(name.CompareNoCase("Allowed_Paths")==0){
		ConvertXMLToList(val,Allow_PATH_List);

	//Requested File
	}else if(name.CompareNoCase("Deny_Filename_Characters")==0){
		Deny_FILENAME_Characters = XMLDecode(val);
	}else if(name.CompareNoCase("Deny_Default_Document")==0){
		Deny_FILENAME_DefaultDocument = val=="1"?true:false;
	}else if(name.CompareNoCase("Use_Denied_Files")==0){
		UseDenyFilenameCGIApplications = val=="1"?true:false;
	}else if(name.CompareNoCase("Denied_Files")==0){
		ConvertXMLToList(val,Deny_FILENAME_CGIApplications);

	//Requested Extension
	}else if(name.CompareNoCase("Use_Allowed_Extensions")==0){
		UseAllowExtensions = val=="1"?true:false;
	}else if(name.CompareNoCase("Use_Denied_Extensions")==0){
		UseDenyExtensions = val=="1"?true:false;
	}else if(name.CompareNoCase("Allowed_Extensions")==0){
		ConvertXMLToList(val,Allow_EXTENSION_List);
	}else if(name.CompareNoCase("Denied_Extensions")==0){
		ConvertXMLToList(val,Deny_EXTENSION_List);

	//HTTP Headers
	}else if(name.CompareNoCase("Remove_Server_Header")==0){
		RemoveServerHeader = val=="1"?true:false;
	}else if(name.CompareNoCase("Change_Server_Header")==0){
		ChangeServerHeader = val=="1"?true:false;
	}else if(name.CompareNoCase("Server_Header")==0){
		ServerHeader = XMLDecode(val);
	}else if(name.CompareNoCase("Deny_Cookie_SQL_Injection")==0){
		Deny_COOKIE_SQLinjection = val=="1"?true:false;
	}else if(name.CompareNoCase("Deny_Cookie_Encoding_Exploits")==0){
		Deny_COOKIE_EncodingExploits = val=="1"?true:false;
	}else if(name.CompareNoCase("RFC_Compliant_Host_Header")==0){
		RFCCompliantHostHeader = val=="1"?true:false;
	}else if(name.CompareNoCase("Use_Denied_Headers")==0){
		UseDenyHeaders = val=="1"?true:false;
	}else if(name.CompareNoCase("Denied_Headers")==0){
		ConvertXMLToList(val,Deny_HEADER_List);
	}else if(name.CompareNoCase("Use_Allowed_Content_Types")==0){
		UseAllowHeaderContentType = val=="1"?true:false;
	}else if(name.CompareNoCase("Allowed_Content_Types")==0){
		ConvertXMLToList(val,Allow_HEADER_ContentType);
	
	//HTTP Verbs (Request Methods)
	}else if(name.CompareNoCase("Use_Allowed_Verbs")==0){
		UseAllowVerbs = val=="1"?true:false;
	}else if(name.CompareNoCase("Use_Denied_Verbs")==0){
		UseDenyVerbs = val=="1"?true:false;
	}else if(name.CompareNoCase("Allowed_Verbs")==0){
		ConvertXMLToList(val,Allow_VERB_List);
	}else if(name.CompareNoCase("Denied_Verbs")==0){
		ConvertXMLToList(val,Deny_VERB_List);

	//Querystring Scanning
	}else if(name.CompareNoCase("Use_Querystring_Raw_Scan")==0){
		UseQuerystringRawScan = val=="1"?true:false;
	}else if(name.CompareNoCase("Deny_Querystring_SQL_Injection")==0){
		Deny_QUERYSTRING_SQLinjection = val=="1"?true:false;
	}else if(name.CompareNoCase("Deny_Querystring_Encoding_Exploits")==0){
		Deny_QUERYSTRING_EncodingExploits = val=="1"?true:false;
	}else if(name.CompareNoCase("Deny_Querystring_Directory_Traversal")==0){
		Deny_QUERYSTRING_DirectoryTraversal = val=="1"?true:false;
	}else if(name.CompareNoCase("Deny_Querystring_High_Bit_Shellcode")==0){
		Deny_QUERYSTRING_HighBitShellcode = val=="1"?true:false;
	}else if(name.CompareNoCase("Use_Denied_Querystring_Sequences")==0){
		UseDenyQuerystringSequences = val=="1"?true:false;
	}else if(name.CompareNoCase("Denied_Querystring_Sequences")==0){
		ConvertXMLToList(val,Deny_QUERYSTRING_Sequences);
			
	//Global Filter Capabilities
	}else if(name.CompareNoCase("Is_Installed_As_Global_Filter")==0){
		IsInstalledAsGlobalFilter = val=="1"?true:false;
	}else if(name.CompareNoCase("Deny_Header_SQL_Injection")==0){
		Deny_HEADER_SQLinjection = val=="1"?true:false;
	}else if(name.CompareNoCase("Deny_Header_Encoding_Exploits")==0){
		Deny_HEADER_EncodingExploits = val=="1"?true:false;
	}else if(name.CompareNoCase("Deny_Header_Directory_Traversal")==0){
		Deny_HEADER_DirectoryTraversal = val=="1"?true:false;
	}else if(name.CompareNoCase("Deny_Header_High_Bit_Shellcode")==0){
		Deny_HEADER_HighBitShellcode = val=="1"?true:false;
	}else if(name.CompareNoCase("Use_Denied_Header_Sequences")==0){
		UseDenyHeaderSequences = val=="1"?true:false;
	}else if(name.CompareNoCase("Deny_Postdata_SQL_Injection")==0){
		Deny_POST_SQLinjection = val=="1"?true:false;
	}else if(name.CompareNoCase("Deny_Postdata_Encoding_Exploits")==0){
		Deny_POST_EncodingExploits = val=="1"?true:false;
	}else if(name.CompareNoCase("Deny_Postdata_Directory_Traversal")==0){
		Deny_POST_DirectoryTraversal = val=="1"?true:false;
	}else if(name.CompareNoCase("Deny_Postdata_High_Bit_Shellcode")==0){
		Deny_POST_HighBitShellcode = val=="1"?true:false;
	}else if(name.CompareNoCase("Use_Denied_Post_Sequences")==0){
		UseDenyPOSTSequences = val=="1"?true:false;
	}else if(name.CompareNoCase("Denied_Header_Sequences")==0){
		ConvertXMLToList(val,Deny_HEADER_Sequences);
	}else if(name.CompareNoCase("Denied_Post_Sequences")==0){
		ConvertXMLToList(val,Deny_POST_Sequences);

	//SQL Injection Keywords
	}else if(name.CompareNoCase("SQL_Injection_Keywords")==0){
		ConvertXMLToList(val,Deny_SQL_KeyWords);
	
	//Web applications
	}else if(name.CompareNoCase("Allow_Outlook_Web_Access")==0){
		if(val=="1"){
			Allow_WEBAPP_OWA = true;
			EnableOWA();
		}
	}else if(name.CompareNoCase("Allow_Frontpage_Extensions")==0){
		if(val=="1"){
			Allow_WEBAPP_Frontpage = true;
			EnableFrontpage();
		}
	}else if(name.CompareNoCase("Allow_Coldfusion")==0){
		if(val=="1"){
			Allow_WEBAPP_Coldfusion = true;
			EnableColdfusion();
		}
	}else if(name.CompareNoCase("Allow_Flash_MX")==0){
		if(val=="1"){
			Allow_WEBAPP_Flash = true;
			EnableFlash();
		}
	}else if(name.CompareNoCase("Allow_WebDAV")==0){
		if(val=="1"){
			Allow_WEBAPP_WebDAV = true;
			EnableWebDAV();
		}
	}else if(name.CompareNoCase("Allow_IISADMPWD")==0){
		if(val=="1"){
			Allow_WEBAPP_IISADMPWD = true;
			EnableIISADMPWD();
		}
	}else if(name.CompareNoCase("Allow_SPPS")==0){
		if(val=="1"){
			Allow_WEBAPP_SPPS = true;
			EnableSPPortalServer();
		}
	}else if(name.CompareNoCase("Allow_SPTS")==0){
		if(val=="1"){
			Allow_WEBAPP_SPTS = true;
			EnableSPTeamServices();
		}
	}else if(name.CompareNoCase("Allow_CertificateServices")==0){
		if(val=="1"){
			Allow_WEBAPP_CertificateServices = true;
			EnableCertificateServices();
		}
	}else if(name.CompareNoCase("Allow_BizTalk_Server")==0){
		if(val=="1"){
			Allow_WEBAPP_BizTalk = true;
			EnableBizTalk();
		}
	}else if(name.CompareNoCase("Allow_Commerce_Server")==0){
		if(val=="1"){
			Allow_WEBAPP_Commerce = true;
			EnableCommerceServer();
		}
	}else if(name.CompareNoCase("Allow_Small_Business_Server")==0){
		if(val=="1"){
			Allow_WEBAPP_SBS = true;
			EnableSBS();
		}
	}else if(name.CompareNoCase("Allow_ASP_NET")==0){
		if(val=="1"){
			Allow_WEBAPP_ASPNET = true;
			EnableASPDotNet();
		}
	}else if(name.CompareNoCase("Allow_ASP")==0){
		if(val=="1"){
			Allow_WEBAPP_ASP = true;
			EnableASP();
		}
	}
}

bool CWebKnightSettings::ReadFromRegistry()
{
	/*
	 * read settings from the windows registry
	 */
	try{
		//TO DO
		//parser gebruiken?

		return true;
	}catch(...){
		LoadDefaults();
		return false;
	}
}

bool CWebKnightSettings::ReadFromActiveDirectory()
{
	/* Read settings from Active Directory (enterprise policy)
	 * If you want to apply the same settings for multiple
	 * machines, use this.
	 * Requires domain controller (LDAP server)
	 */
	try{
		//TO DO
		
		return true;
	}catch(...){
		LoadDefaults();
		return false;
	}
}

bool CWebKnightSettings::WriteToRegistry()
{
	/*
	 * Write settings to the windows registry (machine policy)
	 */
	try{
		//TO DO
		
		return true;
	}catch(...){
		return false;
	}
}

bool CWebKnightSettings::WriteToActiveDirectory()
{
	/*
	 * Write settings to Active Directory (enterprise policy)
	 */
	try{
		//TO DO

		return true;
	}catch(...){
		return false;
	}
}

bool CWebKnightSettings::EnableOWA()
{
	try{
		//From urlscan and http://support.microsoft.com/default.aspx?scid=kb;EN-US;309508
		EnableWebDAV(); //WebDAV required for OWA

		//Allow embedded encoding
		Deny_POST_EncodingExploits = false;

		//Allow virtual dirs
		RemoveFrom(Deny_URL_Sequences,"/exchange");
		RemoveFrom(Deny_URL_Sequences,"/public");
		RemoveFrom(Deny_URL_Sequences,"/exadmin");
		RemoveFrom(Deny_URL_Sequences,"/exchweb");
		RemoveFrom(Deny_URL_Sequences,"/bin/");

		//Allow logon form (Exchange 5.5)
		RemoveFrom(Deny_FILENAME_CGIApplications,"logonfrm.asp");
		RemoveFrom(Deny_FILENAME_CGIApplications,"prn");

		//Allow certain paths
		AddTo(Allow_PATH_List,"m:\\");
		AddTo(Allow_PATH_List,"c:\\program files\\exchange\\exchweb");
		AddTo(Allow_PATH_List,"d:\\exchange\\exchweb");
		AddTo(Allow_PATH_List,"\\.\backofficestorage");

		//Allow certain content-types (from testing)
		AddTo(Allow_HEADER_ContentType,"");
		AddTo(Allow_HEADER_ContentType,"application/x-www-UTF8-encoded");
		AddTo(Allow_HEADER_ContentType,"application/x-www-form-urlencoded");
		AddTo(Allow_HEADER_ContentType,"multipart/form-data");

	//EXCHANGE 2000
		//Allow certain settings (from urlscan)
		Deny_PATH_Dot = false;				//allow dot in path
		Deny_URL_HighBitShellcode = false;	//allow high bit characters in url
		Deny_URL_AlternateStream = false;	//allow ':' in url

		//Allow these verbs not in WebDAV (from urlscan)
		AddTo(Allow_VERB_List,"POLL");		RemoveFrom(Deny_VERB_List,"POLL");
		AddTo(Allow_VERB_List,"BMOVE");		RemoveFrom(Deny_VERB_List,"BMOVE");
		AddTo(Allow_VERB_List,"BCOPY");		RemoveFrom(Deny_VERB_List,"BCOPY");
		AddTo(Allow_VERB_List,"BPROPPATCH");RemoveFrom(Deny_VERB_List,"BPROPPATCH");
		AddTo(Allow_VERB_List,"BDELETE");	RemoveFrom(Deny_VERB_List,"BDELETE");
		AddTo(Allow_VERB_List,"SUBSCRIBE");	RemoveFrom(Deny_VERB_List,"SUBSCRIBE");
		AddTo(Allow_VERB_List,"UNSUBSCRIBE");RemoveFrom(Deny_VERB_List,"UNSUBSCRIBE");
		AddTo(Allow_VERB_List,"SUBSCRIPTIONS");RemoveFrom(Deny_VERB_List,"SUBSCRIPTIONS");
		AddTo(Allow_VERB_List,"ACL");		RemoveFrom(Deny_VERB_List,"ACL");
		AddTo(Allow_VERB_List,"NOTIFY");	RemoveFrom(Deny_VERB_List,"NOTIFY");

		//Allow these filename extensions (from URLScan)
		EnableASP();
		EnableStaticHTML();
		//AddTo(Allow_EXTENSION_List,".dll"); RemoveFrom(Deny_EXTENSION_List,".dll");

		//Allow these filename extensions (from testing)
		AddTo(Allow_EXTENSION_List,".css"); RemoveFrom(Deny_EXTENSION_List,".css");
		AddTo(Allow_EXTENSION_List,".js");  RemoveFrom(Deny_EXTENSION_List,".js");
		AddTo(Allow_EXTENSION_List,".htc"); RemoveFrom(Deny_EXTENSION_List,".htc");
		AddTo(Allow_EXTENSION_List,".xsl"); RemoveFrom(Deny_EXTENSION_List,".xsl");
		AddTo(Allow_EXTENSION_List,".eml"); RemoveFrom(Deny_EXTENSION_List,".eml");
		AddTo(Allow_EXTENSION_List,".wav"); RemoveFrom(Deny_EXTENSION_List,".wav");

		//Allow # in url
		Deny_URL_Characters.Remove('#');

		//Allow certain characters in path
		Deny_FULLPATH_Characters.Remove(':');
		Deny_FULLPATH_Characters.Remove('=');
		Deny_FULLPATH_Characters.Remove('+');
		Deny_FULLPATH_Characters.Remove('$');
		Deny_FULLPATH_Characters.Remove('<');
		Deny_FULLPATH_Characters.Remove('>');
		Deny_FULLPATH_Characters.Remove('"');
		Deny_FULLPATH_Characters.Remove('*');
		Deny_FULLPATH_Characters.Remove('|');
		Deny_FULLPATH_Characters.Remove('#');
		Deny_FULLPATH_Characters.Remove('^');

		//Allow certain characters in filename
		Deny_FILENAME_Characters.Remove(':');
		Deny_FILENAME_Characters.Remove('=');
		Deny_FILENAME_Characters.Remove('+');
		Deny_FILENAME_Characters.Remove('$');
		Deny_FILENAME_Characters.Remove('<');
		Deny_FILENAME_Characters.Remove('>');
		Deny_FILENAME_Characters.Remove('"');
		Deny_FILENAME_Characters.Remove('*');
		Deny_FILENAME_Characters.Remove('|');
		Deny_FILENAME_Characters.Remove('#');
		Deny_FILENAME_Characters.Remove('^');

	//EXCHANGE 5.5
		//already in exchange 2000 settings
	
		//EnableIISADMPWD(); (for OWA change password in options - not recommended)

		return true;
	}catch(...){
		return false;
	}
}

bool CWebKnightSettings::EnableColdfusion()
{
	try{
		//TO DO

		return true;
	}catch(...){
		return false;
	}
}

bool CWebKnightSettings::EnableFrontpage()
{
	try{
		//From urlscan and: http://support.microsoft.com/default.aspx?scid=kb;EN-US;318290

		//AllowLateScanning (from urlscan)
		AllowLateScanning = true;

		//Allow these headers (from urlscan)
		AddTo(Allow_VERB_List,"GET");		RemoveFrom(Deny_VERB_List,"GET");
		AddTo(Allow_VERB_List,"HEAD");		RemoveFrom(Deny_VERB_List,"HEAD");
		AddTo(Allow_VERB_List,"POST");		RemoveFrom(Deny_VERB_List,"POST");
		AddTo(Allow_VERB_List,"OPTIONS");	RemoveFrom(Deny_VERB_List,"OPTIONS");

		//Allow headers (from URLScan)
		RemoveFrom(Deny_HEADER_List,"Translate:"); //"Translate:f" means: no script execution (is needed for script source access)

		//Allow certain url sequences (TO DO: test this what can be left out?)
		//RemoveFrom(Deny_URL_Sequences,"/scripts");
		//RemoveFrom(Deny_URL_Sequences,"/_vti_aut");
		//RemoveFrom(Deny_URL_Sequences,"/_vti_bin");
		//RemoveFrom(Deny_URL_Sequences,"/_vti_rpc");
		//RemoveFrom(Deny_URL_Sequences,"/admcgi");
		//RemoveFrom(Deny_URL_Sequences,"/admisapi");
		//RemoveFrom(Deny_URL_Sequences,"/_vti_script");
		//RemoveFrom(Deny_URL_Sequences,"/_mem_bin");

		//Allow certain paths
		AddTo(Allow_PATH_List,"c:\\program files\\common files\\microsoft shared\\web server extensions");

		//Allow certain filenames (TO DO: test this what can be left out?)
		RemoveFrom(Deny_FILENAME_CGIApplications,"_vti_inf.html");
		//RemoveFrom(Deny_FILENAME_CGIApplications,"postinfo.html");  
		//RemoveFrom(Deny_FILENAME_CGIApplications,"fp30reg.dll");	 
		//RemoveFrom(Deny_FILENAME_CGIApplications,"fp4areg.dll");	 
		//RemoveFrom(Deny_FILENAME_CGIApplications,"shtml.dll");		 
		//RemoveFrom(Deny_FILENAME_CGIApplications,"shtml.exe");		 
		//RemoveFrom(Deny_FILENAME_CGIApplications,"fpsrvadm.exe");	 
		//RemoveFrom(Deny_FILENAME_CGIApplications,"fpremadm.exe");	 
		//RemoveFrom(Deny_FILENAME_CGIApplications,"fpadmin.htm");	 
		//RemoveFrom(Deny_FILENAME_CGIApplications,"fpadmcgi.exe");	 
		//RemoveFrom(Deny_FILENAME_CGIApplications,"cfgwiz.exe");	 
		//RemoveFrom(Deny_FILENAME_CGIApplications,"authors.pwd");	 
		//RemoveFrom(Deny_FILENAME_CGIApplications,"author.exe");	 
		//RemoveFrom(Deny_FILENAME_CGIApplications,"author.dll");	 
		//RemoveFrom(Deny_FILENAME_CGIApplications,"administrators.pwd"); 
		//RemoveFrom(Deny_FILENAME_CGIApplications,"access.cnf");	 
		//RemoveFrom(Deny_FILENAME_CGIApplications,"service.cnf");	 
		//RemoveFrom(Deny_FILENAME_CGIApplications,"service.pwd");	 
		//RemoveFrom(Deny_FILENAME_CGIApplications,"service.stp");	 
		//RemoveFrom(Deny_FILENAME_CGIApplications,"services.cnf");	 
		//RemoveFrom(Deny_FILENAME_CGIApplications,"svcacl.cnf");	 
		//RemoveFrom(Deny_FILENAME_CGIApplications,"users.pwd");		 
		//RemoveFrom(Deny_FILENAME_CGIApplications,"writeto.cnf");	 
		//RemoveFrom(Deny_FILENAME_CGIApplications,"dvwssr.dll");	 

		//Allow these filename extensions (from URLScan)
		AddTo(Allow_EXTENSION_List,".htm"); RemoveFrom(Deny_EXTENSION_List,".htm");
		AddTo(Allow_EXTENSION_List,".html");RemoveFrom(Deny_EXTENSION_List,".html");
		AddTo(Allow_EXTENSION_List,".dll"); RemoveFrom(Deny_EXTENSION_List,".dll");

		return true;
	}catch(...){
		return false;
	}
}

bool CWebKnightSettings::EnableFlash()
{
	try{
		//TO DO

		return true;
	}catch(...){
		return false;
	}
}

bool CWebKnightSettings::EnableWebDAV()
{
	/*
	 * Enable HTTP Extension: WebDAV
	 * See rfc 2518
	 */
	try{
		//Allow WebDAV verbs (from rfc 2518)
		AddTo(Allow_VERB_List,"PROPFIND");	RemoveFrom(Deny_VERB_List,"PROPFIND");
		AddTo(Allow_VERB_List,"PROPPATCH");	RemoveFrom(Deny_VERB_List,"PROPPATCH");
		AddTo(Allow_VERB_List,"MKCOL");		RemoveFrom(Deny_VERB_List,"MKCOL");
		AddTo(Allow_VERB_List,"GET");		RemoveFrom(Deny_VERB_List,"GET");
		AddTo(Allow_VERB_List,"HEAD");		RemoveFrom(Deny_VERB_List,"HEAD");
		AddTo(Allow_VERB_List,"POST");		RemoveFrom(Deny_VERB_List,"POST");
		AddTo(Allow_VERB_List,"DELETE");	RemoveFrom(Deny_VERB_List,"DELETE");
		AddTo(Allow_VERB_List,"PUT");		RemoveFrom(Deny_VERB_List,"PUT");
		AddTo(Allow_VERB_List,"COPY");		RemoveFrom(Deny_VERB_List,"COPY");
		AddTo(Allow_VERB_List,"MOVE");		RemoveFrom(Deny_VERB_List,"MOVE");
		AddTo(Allow_VERB_List,"LOCK");		RemoveFrom(Deny_VERB_List,"LOCK");
		AddTo(Allow_VERB_List,"UNLOCK");	RemoveFrom(Deny_VERB_List,"UNLOCK");
		AddTo(Allow_VERB_List,"OPTIONS");	RemoveFrom(Deny_VERB_List,"OPTIONS");

		//Allow MS-WebDAV verbs (from iishelp)
		AddTo(Allow_VERB_List,"SEARCH");	RemoveFrom(Deny_VERB_List,"SEARCH");

		//Allow WebDAV headers (from rfc 2518)
		RemoveFrom(Deny_HEADER_List,"Depth:");
		RemoveFrom(Deny_HEADER_List,"Destination:");
		RemoveFrom(Deny_HEADER_List,"If:");
		RemoveFrom(Deny_HEADER_List,"Lock-Token:");
		RemoveFrom(Deny_HEADER_List,"Overwrite:");
		RemoveFrom(Deny_HEADER_List,"Timeout:");
		//Allow MS-WebDAV headers (from testing)
		RemoveFrom(Deny_HEADER_List,"Translate:"); //"Translate:f" means: no script execution (is needed for script source access)

		//Allow text/xml content type (from rfc 2518);
		AddTo(Allow_HEADER_ContentType,"text/xml");

		return true;
	}catch(...){
		return false;
	}
}

bool CWebKnightSettings::EnableIISADMPWD()
{
	try{
		//Allow virtual dir /iisadmpwd
		RemoveFrom(Deny_URL_Sequences,"/iisadmpwd");
		
		//Allow certain filenames
		RemoveFrom(Deny_FILENAME_CGIApplications,"achg.htr");
		RemoveFrom(Deny_FILENAME_CGIApplications,"anot.htr");
		
		//Allow .htr extension
		AddTo(Allow_EXTENSION_List,".htr"); RemoveFrom(Deny_EXTENSION_List,".htr");
		
		//Allow iisadmpwd path
		AddTo(Allow_PATH_List,"c:\\winnt\\system32\\inetsrv\\iisadmpwd");

		//Allow certain verbs
		AddTo(Allow_VERB_List,"GET");		RemoveFrom(Deny_VERB_List,"GET");
		AddTo(Allow_VERB_List,"HEAD");		RemoveFrom(Deny_VERB_List,"HEAD");
		AddTo(Allow_VERB_List,"POST");		RemoveFrom(Deny_VERB_List,"POST");

		//Allow certain content types
		AddTo(Allow_HEADER_ContentType,"application/x-www-form-urlencoded");

		return true;
	}catch(...){
		return false;
	}
}

bool CWebKnightSettings::EnableSPPortalServer()
{
	try{

		EnableWebDAV();	//uses WebDAV

		//verbs not in WebDAV (from urlscan)
		AddTo(Allow_VERB_List,"SUBSCRIBE");	RemoveFrom(Deny_VERB_List,"SUBSCRIBE");
		AddTo(Allow_VERB_List,"INVOKE");	RemoveFrom(Deny_VERB_List,"INVOKE");
		AddTo(Allow_VERB_List,"POLL");		RemoveFrom(Deny_VERB_List,"POLL");
		AddTo(Allow_VERB_List,"UNSUBSCRIBE");RemoveFrom(Deny_VERB_List,"INVOKE");
		AddTo(Allow_VERB_List,"TRANSLATE");	RemoveFrom(Deny_VERB_List,"TRANSLATE");

		//Allow certain content-types
		AddTo(Allow_HEADER_ContentType,"");
		AddTo(Allow_HEADER_ContentType,"application/x-www-form-urlencoded");
		
		//Allow these filename extensions (from URLScan)
		EnableASP();
		EnableStaticHTML();

		return true;
	}catch(...){
		return false;
	}
}

bool CWebKnightSettings::EnableSPTeamServices()
{
	try{

		EnableFrontpage();	//uses frontpage

		//Allow certain content-types
		AddTo(Allow_HEADER_ContentType,"");
		AddTo(Allow_HEADER_ContentType,"application/x-www-form-urlencoded");
		
		//Allow these filename extensions (from URLScan)
		EnableASP();
		EnableStaticHTML();
		AddTo(Allow_EXTENSION_List,".htw"); RemoveFrom(Deny_EXTENSION_List,".htw");
		AddTo(Allow_EXTENSION_List,".ida"); RemoveFrom(Deny_EXTENSION_List,".ida");
		AddTo(Allow_EXTENSION_List,".idq"); RemoveFrom(Deny_EXTENSION_List,".idq");

		return true;
	}catch(...){
		return false;
	}
}

bool CWebKnightSettings::EnableCertificateServices()
{
	try{
		//Allow virtual dirs
		RemoveFrom(Deny_URL_Sequences,"/certsrv");
		RemoveFrom(Deny_URL_Sequences,"/certcontrol");
		RemoveFrom(Deny_URL_Sequences,"/certenroll");

		//Allow path
		AddTo(Allow_PATH_List,"c:\\winnt\\system32\\certsrv");

		//Allow these verbs
		AddTo(Allow_VERB_List,"GET");		RemoveFrom(Deny_VERB_List,"GET");
		AddTo(Allow_VERB_List,"HEAD");		RemoveFrom(Deny_VERB_List,"HEAD");
		AddTo(Allow_VERB_List,"POST");		RemoveFrom(Deny_VERB_List,"POST");

		//Allow certain content-types
		AddTo(Allow_HEADER_ContentType,"");
		AddTo(Allow_HEADER_ContentType,"application/x-www-form-urlencoded");
		
		//Allow these filename extensions (from testing)
		EnableASP();
		AddTo(Allow_EXTENSION_List,".cer"); RemoveFrom(Deny_EXTENSION_List,".cer"); //Certificate (X.509)
		AddTo(Allow_EXTENSION_List,".gif"); RemoveFrom(Deny_EXTENSION_List,".gif");
		AddTo(Allow_EXTENSION_List,".p7b"); RemoveFrom(Deny_EXTENSION_List,".p7b"); //Certificate PKCS#7
		AddTo(Allow_EXTENSION_List,".pfx"); RemoveFrom(Deny_EXTENSION_List,".pfx"); //Certificate PKCS#12
		AddTo(Allow_EXTENSION_List,".crl"); RemoveFrom(Deny_EXTENSION_List,".crl"); //revocation list
		AddTo(Allow_EXTENSION_List,".crt"); RemoveFrom(Deny_EXTENSION_List,".crt"); //authority
		AddTo(Allow_EXTENSION_List,".cab"); RemoveFrom(Deny_EXTENSION_List,".cab");
		AddTo(Allow_EXTENSION_List,".dll"); RemoveFrom(Deny_EXTENSION_List,".dll");

		return true;
	}catch(...){
		return false;
	}
}

bool CWebKnightSettings::EnableBizTalk()
{
	try{
		EnableWebDAV(); //uses WebDAV

		//Allow virtual dirs
		RemoveFrom(Deny_URL_Sequences,"/biztalkserverdocs");
		RemoveFrom(Deny_URL_Sequences,"/biztalkserverrepository");
		RemoveFrom(Deny_URL_Sequences,"/messagingmanager");
		RemoveFrom(Deny_URL_Sequences,"/biztalktracking");

		//Allow path
		AddTo(Allow_PATH_List,"c:\\program files\\microsoft biztalk server\\docs");
		AddTo(Allow_PATH_List,"c:\\program files\\microsoft biztalk server\\biztalkserverrepository");
		AddTo(Allow_PATH_List,"c:\\program files\\microsoft biztalk server\\messagingmanager");
		AddTo(Allow_PATH_List,"c:\\program files\\microsoft biztalk server\\biztalktracking");

		//Allow certain content-types
		AddTo(Allow_HEADER_ContentType,"");
		AddTo(Allow_HEADER_ContentType,"application/x-www-form-urlencoded");
		
		//Allow these filename extensions (from research TODO: test this)
		EnableASP();
		EnableStaticHTML();
		AddTo(Allow_EXTENSION_List,".css"); RemoveFrom(Deny_EXTENSION_List,".css");
		AddTo(Allow_EXTENSION_List,".js");  RemoveFrom(Deny_EXTENSION_List,".js");
		//AddTo(Allow_EXTENSION_List,".xsl"); RemoveFrom(Deny_EXTENSION_List,".xsl");
		//AddTo(Allow_EXTENSION_List,".xml"); RemoveFrom(Deny_EXTENSION_List,".xml");

		return true;
	}catch(...){
		return false;
	}
}

bool CWebKnightSettings::EnableCommerceServer()
{
	try{
		//TO DO: requires SOAP? OWA? WebDAV?
		//TO DO: check all those that are commented out

		//Allow virtual dir
		RemoveFrom(Deny_URL_Sequences,"/widgets");
		//RemoveFrom(Deny_URL_Sequences,"/retail");
		//RemoveFrom(Deny_URL_Sequences,"/retailbizdesk");
		//RemoveFrom(Deny_URL_Sequences,"/supplier");
		
		//Allow certain paths (from research)
		AddTo(Allow_PATH_List,"c:\\inetpub\\wwwroot");
		//AddTo(Allow_PATH_List,"c:\\program files\\microsoft commerce server"); //TODO: needed & subdirs?
		//AddTo(Allow_PATH_List,"c:\\program files\\microsoft commerce server\\widgets"); //TODO: needed?

		//allow verbs (from urlscan: HEAD not allowed!?)
		AddTo(Allow_VERB_List,"GET");	RemoveFrom(Deny_VERB_List,"GET");
		AddTo(Allow_VERB_List,"POST");	RemoveFrom(Deny_VERB_List,"POST");
		AddTo(Allow_VERB_List,"PUT");	RemoveFrom(Deny_VERB_List,"PUT");

		//Allow certain headers (from urlscan)
		RemoveFrom(Deny_HEADER_List,"Translate:");
		RemoveFrom(Deny_HEADER_List,"If:");
		RemoveFrom(Deny_HEADER_List,"Lock-Token:");

		//Allow certain content-types
		AddTo(Allow_HEADER_ContentType,"");
		AddTo(Allow_HEADER_ContentType,"application/x-www-form-urlencoded");
		
		//Allow these filename extensions (from URLScan)
		EnableASP();
		EnableStaticHTML();

		return true;
	}catch(...){
		return false;
	}
}

bool CWebKnightSettings::EnableSBS()
{
	try{
		EnableOWA();

		return true;
	}catch(...){
		return false;
	}
}


bool CWebKnightSettings::EnableASPDotNet()
{
	try{
		//Allow ASP.NET (TO DO: error Niels?)

		//allow verbs
		AddTo(Allow_VERB_List,"GET");	RemoveFrom(Deny_VERB_List,"GET");
		AddTo(Allow_VERB_List,"HEAD");	RemoveFrom(Deny_VERB_List,"HEAD");
		AddTo(Allow_VERB_List,"POST");	RemoveFrom(Deny_VERB_List,"POST");
		AddTo(Allow_VERB_List,"DEBUG");	RemoveFrom(Deny_VERB_List,"DEBUG");

		//Allow certain filenames
		RemoveFrom(Deny_FILENAME_CGIApplications,"trace.axd");

		//allow these extensions
		//AddTo(Allow_EXTENSION_List,".asax");	RemoveFrom(Deny_EXTENSION_List,".asax");		//asa
		//AddTo(Allow_EXTENSION_List,".asa");	RemoveFrom(Deny_EXTENSION_List,".asa");			//".asax" contains ".asa"
		//AddTo(Allow_EXTENSION_List,".ascx");	RemoveFrom(Deny_EXTENSION_List,".ascx");		//forbidden by asp.net (custom user controls)
		AddTo(Allow_EXTENSION_List,".ashx");	RemoveFrom(Deny_EXTENSION_List,".ashx");		//custom httphandler
		AddTo(Allow_EXTENSION_List,".asmx");	RemoveFrom(Deny_EXTENSION_List,".asmx");		//web service
		AddTo(Allow_EXTENSION_List,".aspx");	RemoveFrom(Deny_EXTENSION_List,".aspx");		//asp
		AddTo(Allow_EXTENSION_List,".asp");		RemoveFrom(Deny_EXTENSION_List,".asp");			//".aspx" contains ".asp"
		AddTo(Allow_EXTENSION_List,".axd");		RemoveFrom(Deny_EXTENSION_List,".axd");			//trace (possible information disclosure)
		//AddTo(Allow_EXTENSION_List,".config");RemoveFrom(Deny_EXTENSION_List,".config");		//forbidden by asp.net (config)
		//AddTo(Allow_EXTENSION_List,".cs");	RemoveFrom(Deny_EXTENSION_List,".cs");			//forbidden by asp.net (c#)
		//AddTo(Allow_EXTENSION_List,".csproj");RemoveFrom(Deny_EXTENSION_List,".csproj");		//forbidden by asp.net (c# project)
		//AddTo(Allow_EXTENSION_List,".licx");	RemoveFrom(Deny_EXTENSION_List,".licx");		//forbidden by asp.net (licenses)
		AddTo(Allow_EXTENSION_List,".rem");		RemoveFrom(Deny_EXTENSION_List,".rem");			//remoting
		//AddTo(Allow_EXTENSION_List,".resources");RemoveFrom(Deny_EXTENSION_List,".resources");//forbidden by asp.net (resources)
		//AddTo(Allow_EXTENSION_List,".resx");	RemoveFrom(Deny_EXTENSION_List,".resx");		//forbidden by asp.net (resources in xml file)
		AddTo(Allow_EXTENSION_List,".soap");	RemoveFrom(Deny_EXTENSION_List,".soap");		//remoting & handling soap requests
		//AddTo(Allow_EXTENSION_List,".vb");	RemoveFrom(Deny_EXTENSION_List,".vb");			//forbidden by asp.net (vb code)
		//AddTo(Allow_EXTENSION_List,".vbproj");RemoveFrom(Deny_EXTENSION_List,".vbproj");		//forbidden by asp.net (vb project)
		//AddTo(Allow_EXTENSION_List,".vsdisco");RemoveFrom(Deny_EXTENSION_List,".vsdisco");	//dynamic discovery: security issue!!!! http://msdn.microsoft.com/msdnmag/issues/02/08/XMLFiles/default.aspx
		//AddTo(Allow_EXTENSION_List,".webinfo");RemoveFrom(Deny_EXTENSION_List,".webinfo");	//forbidden by asp.net


		return true;
	}catch(...){
		return false;
	}
}

bool CWebKnightSettings::EnableStaticHTML()
{
	try{

		//Allow these verbs
		AddTo(Allow_VERB_List,"GET");		RemoveFrom(Deny_VERB_List,"GET");
		AddTo(Allow_VERB_List,"HEAD");		RemoveFrom(Deny_VERB_List,"HEAD");
		
		//Allow these extensions
		AddTo(Allow_EXTENSION_List,".htm"); RemoveFrom(Deny_EXTENSION_List,".htm");
		AddTo(Allow_EXTENSION_List,".html");RemoveFrom(Deny_EXTENSION_List,".html");
		AddTo(Allow_EXTENSION_List,".txt"); RemoveFrom(Deny_EXTENSION_List,".txt");
		AddTo(Allow_EXTENSION_List,".jpg"); RemoveFrom(Deny_EXTENSION_List,".jpg");
		AddTo(Allow_EXTENSION_List,".jpeg");RemoveFrom(Deny_EXTENSION_List,".jpeg");
		AddTo(Allow_EXTENSION_List,".gif"); RemoveFrom(Deny_EXTENSION_List,".gif");

		return true;
	}catch(...){
		return false;
	}
}

bool CWebKnightSettings::EnableASP()
{
	try{

		//Allow these verbs
		AddTo(Allow_VERB_List,"GET");		RemoveFrom(Deny_VERB_List,"GET");
		AddTo(Allow_VERB_List,"HEAD");		RemoveFrom(Deny_VERB_List,"HEAD");
		AddTo(Allow_VERB_List,"POST");		RemoveFrom(Deny_VERB_List,"POST");
		//AddTo(Allow_VERB_List,"TRACE");		RemoveFrom(Deny_VERB_List,"TRACE");

		//Allow these extensions
		AddTo(Allow_EXTENSION_List,".asp"); RemoveFrom(Deny_EXTENSION_List,".asp");
		AddTo(Allow_EXTENSION_List,".cer"); RemoveFrom(Deny_EXTENSION_List,".cer");
		AddTo(Allow_EXTENSION_List,".cdx"); RemoveFrom(Deny_EXTENSION_List,".cdx");
		//AddTo(Allow_EXTENSION_List,".asa"); RemoveFrom(Deny_EXTENSION_List,".asa");	

		return true;
	}catch(...){
		return false;
	}
}
